/****************************************************************************
** Meta object code from reading C++ file 'qquickloader_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickloader_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickloader_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickLoader_t {
    QByteArrayData data[29];
    char stringdata0[320];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickLoader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickLoader_t qt_meta_stringdata_QQuickLoader = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QQuickLoader"
QT_MOC_LITERAL(1, 13, 11), // "QML.Element"
QT_MOC_LITERAL(2, 25, 6), // "Loader"
QT_MOC_LITERAL(3, 32, 11), // "itemChanged"
QT_MOC_LITERAL(4, 44, 0), // ""
QT_MOC_LITERAL(5, 45, 13), // "activeChanged"
QT_MOC_LITERAL(6, 59, 13), // "sourceChanged"
QT_MOC_LITERAL(7, 73, 22), // "sourceComponentChanged"
QT_MOC_LITERAL(8, 96, 13), // "statusChanged"
QT_MOC_LITERAL(9, 110, 15), // "progressChanged"
QT_MOC_LITERAL(10, 126, 6), // "loaded"
QT_MOC_LITERAL(11, 133, 19), // "asynchronousChanged"
QT_MOC_LITERAL(12, 153, 15), // "_q_sourceLoaded"
QT_MOC_LITERAL(13, 169, 13), // "_q_updateSize"
QT_MOC_LITERAL(14, 183, 9), // "setSource"
QT_MOC_LITERAL(15, 193, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(16, 209, 6), // "active"
QT_MOC_LITERAL(17, 216, 6), // "source"
QT_MOC_LITERAL(18, 223, 15), // "sourceComponent"
QT_MOC_LITERAL(19, 239, 14), // "QQmlComponent*"
QT_MOC_LITERAL(20, 254, 4), // "item"
QT_MOC_LITERAL(21, 259, 6), // "status"
QT_MOC_LITERAL(22, 266, 6), // "Status"
QT_MOC_LITERAL(23, 273, 8), // "progress"
QT_MOC_LITERAL(24, 282, 12), // "asynchronous"
QT_MOC_LITERAL(25, 295, 4), // "Null"
QT_MOC_LITERAL(26, 300, 5), // "Ready"
QT_MOC_LITERAL(27, 306, 7), // "Loading"
QT_MOC_LITERAL(28, 314, 5) // "Error"

    },
    "QQuickLoader\0QML.Element\0Loader\0"
    "itemChanged\0\0activeChanged\0sourceChanged\0"
    "sourceComponentChanged\0statusChanged\0"
    "progressChanged\0loaded\0asynchronousChanged\0"
    "_q_sourceLoaded\0_q_updateSize\0setSource\0"
    "QQmlV4Function*\0active\0source\0"
    "sourceComponent\0QQmlComponent*\0item\0"
    "status\0Status\0progress\0asynchronous\0"
    "Null\0Ready\0Loading\0Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickLoader[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      11,   16, // methods
       7,   84, // properties
       1,  112, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   71,    4, 0x06 /* Public */,
       5,    0,   72,    4, 0x06 /* Public */,
       6,    0,   73,    4, 0x06 /* Public */,
       7,    0,   74,    4, 0x06 /* Public */,
       8,    0,   75,    4, 0x06 /* Public */,
       9,    0,   76,    4, 0x06 /* Public */,
      10,    0,   77,    4, 0x06 /* Public */,
      11,    0,   78,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,   79,    4, 0x08 /* Private */,
      13,    0,   80,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      14,    1,   81,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 15,    4,

 // properties: name, type, flags
      16, QMetaType::Bool, 0x00495103,
      17, QMetaType::QUrl, 0x00495103,
      18, 0x80000000 | 19, 0x0049510f,
      20, QMetaType::QObjectStar, 0x00495001,
      21, 0x80000000 | 22, 0x00495009,
      23, QMetaType::QReal, 0x00495001,
      24, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       1,
       2,
       3,
       0,
       4,
       5,
       7,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    4,  117,

 // enum data: key, value
      25, uint(QQuickLoader::Null),
      26, uint(QQuickLoader::Ready),
      27, uint(QQuickLoader::Loading),
      28, uint(QQuickLoader::Error),

       0        // eod
};

void QQuickLoader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickLoader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemChanged(); break;
        case 1: _t->activeChanged(); break;
        case 2: _t->sourceChanged(); break;
        case 3: _t->sourceComponentChanged(); break;
        case 4: _t->statusChanged(); break;
        case 5: _t->progressChanged(); break;
        case 6: _t->loaded(); break;
        case 7: _t->asynchronousChanged(); break;
        case 8: _t->d_func()->_q_sourceLoaded(); break;
        case 9: _t->d_func()->_q_updateSize(); break;
        case 10: _t->setSource((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::itemChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::activeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::sourceChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::sourceComponentChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::statusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::progressChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::loaded)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLoader::asynchronousChanged)) {
                *result = 7;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickLoader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 2: *reinterpret_cast< QQmlComponent**>(_v) = _t->sourceComponent(); break;
        case 3: *reinterpret_cast< QObject**>(_v) = _t->item(); break;
        case 4: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->progress(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->asynchronous(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickLoader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActive(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 2: _t->setSourceComponent(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 6: _t->setAsynchronous(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickLoader *_t = static_cast<QQuickLoader *>(_o);
        (void)_t;
        switch (_id) {
        case 2: _t->resetSourceComponent(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickLoader::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickLoader.data,
    qt_meta_data_QQuickLoader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickLoader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickLoader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickLoader.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickLoader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickLoader::itemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickLoader::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickLoader::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickLoader::sourceComponentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickLoader::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickLoader::progressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickLoader::loaded()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickLoader::asynchronousChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
