/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <private/qquickworkerscript_p.h>

void qml_register_types_QtQml_WorkerScript()
{
    qmlRegisterTypesAndRevisions<QQuickWorkerScript>("QtQml.WorkerScript", 2);
    qmlRegisterModule("QtQml.WorkerScript", 2, 15);
}

static const QQmlModuleRegistration registration("QtQml.WorkerScript", 2, qml_register_types_QtQml_WorkerScript);
