/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 5.15.4
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

static const unsigned char qt_resource_data[] = {
  // CompositeType
  0x0,0x0,0x0,0x34,
  0x69,
  0x6d,0x70,0x6f,0x72,0x74,0x20,0x51,0x74,0x51,0x6d,0x6c,0x20,0x32,0x2e,0x30,0xa,
  0xa,0x51,0x74,0x4f,0x62,0x6a,0x65,0x63,0x74,0x20,0x7b,0xa,0x20,0x20,0x20,0x20,
  0x70,0x72,0x6f,0x70,0x65,0x72,0x74,0x79,0x20,0x69,0x6e,0x74,0x20,0x66,0x6f,0x6f,
  0xa,0x7d,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // tstqqmlmetatype
  0x0,0xf,
  0xb,0xb9,0x68,0xa5,
  0x0,0x74,
  0x0,0x73,0x0,0x74,0x0,0x71,0x0,0x71,0x0,0x6d,0x0,0x6c,0x0,0x6d,0x0,0x65,0x0,0x74,0x0,0x61,0x0,0x74,0x0,0x79,0x0,0x70,0x0,0x65,
    // data
  0x0,0x4,
  0x0,0x6,0xa8,0xa1,
  0x0,0x64,
  0x0,0x61,0x0,0x74,0x0,0x61,
    // CompositeType.qml
  0x0,0x11,
  0xc,0x10,0x94,0xfc,
  0x0,0x43,
  0x0,0x6f,0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x73,0x0,0x69,0x0,0x74,0x0,0x65,0x0,0x54,0x0,0x79,0x0,0x70,0x0,0x65,0x0,0x2e,0x0,0x71,0x0,0x6d,0x0,0x6c,
  
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/tstqqmlmetatype
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/tstqqmlmetatype/data
  0x0,0x0,0x0,0x24,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/tstqqmlmetatype/data/CompositeType.qml
  0x0,0x0,0x0,0x32,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#ifdef QT_NAMESPACE
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmlfiles)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmlfiles)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmlfiles)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmlfiles)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmlfiles)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmlfiles)(); }
   } dummy;
}
