/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <attached.h>

void qml_register_types_org_qtproject_Test()
{
    qmlRegisterTypesAndRevisions<Attached>("org.qtproject.Test", 1);
    qmlRegisterModule("org.qtproject.Test", 1, 0);
}

static const QQmlModuleRegistration registration("org.qtproject.Test", 1, qml_register_types_org_qtproject_Test);
