/****************************************************************************
** Meta object code from reading C++ file 'qqmltablemodelcolumn_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/labsmodels/qqmltablemodelcolumn_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmltablemodelcolumn_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlTableModelColumn_t {
    QByteArrayData data[62];
    char stringdata0[953];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlTableModelColumn_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlTableModelColumn_t qt_meta_stringdata_QQmlTableModelColumn = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQmlTableModelColumn"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 16), // "TableModelColumn"
QT_MOC_LITERAL(3, 50, 12), // "indexChanged"
QT_MOC_LITERAL(4, 63, 0), // ""
QT_MOC_LITERAL(5, 64, 14), // "displayChanged"
QT_MOC_LITERAL(6, 79, 17), // "setDisplayChanged"
QT_MOC_LITERAL(7, 97, 17), // "decorationChanged"
QT_MOC_LITERAL(8, 115, 20), // "setDecorationChanged"
QT_MOC_LITERAL(9, 136, 11), // "editChanged"
QT_MOC_LITERAL(10, 148, 14), // "setEditChanged"
QT_MOC_LITERAL(11, 163, 14), // "toolTipChanged"
QT_MOC_LITERAL(12, 178, 17), // "setToolTipChanged"
QT_MOC_LITERAL(13, 196, 16), // "statusTipChanged"
QT_MOC_LITERAL(14, 213, 19), // "setStatusTipChanged"
QT_MOC_LITERAL(15, 233, 16), // "whatsThisChanged"
QT_MOC_LITERAL(16, 250, 19), // "setWhatsThisChanged"
QT_MOC_LITERAL(17, 270, 11), // "fontChanged"
QT_MOC_LITERAL(18, 282, 14), // "setFontChanged"
QT_MOC_LITERAL(19, 297, 20), // "textAlignmentChanged"
QT_MOC_LITERAL(20, 318, 23), // "setTextAlignmentChanged"
QT_MOC_LITERAL(21, 342, 17), // "backgroundChanged"
QT_MOC_LITERAL(22, 360, 20), // "setBackgroundChanged"
QT_MOC_LITERAL(23, 381, 17), // "foregroundChanged"
QT_MOC_LITERAL(24, 399, 20), // "setForegroundChanged"
QT_MOC_LITERAL(25, 420, 17), // "checkStateChanged"
QT_MOC_LITERAL(26, 438, 20), // "setCheckStateChanged"
QT_MOC_LITERAL(27, 459, 21), // "accessibleTextChanged"
QT_MOC_LITERAL(28, 481, 24), // "setAccessibleTextChanged"
QT_MOC_LITERAL(29, 506, 28), // "accessibleDescriptionChanged"
QT_MOC_LITERAL(30, 535, 31), // "setAccessibleDescriptionChanged"
QT_MOC_LITERAL(31, 567, 15), // "sizeHintChanged"
QT_MOC_LITERAL(32, 583, 18), // "setSizeHintChanged"
QT_MOC_LITERAL(33, 602, 7), // "display"
QT_MOC_LITERAL(34, 610, 8), // "QJSValue"
QT_MOC_LITERAL(35, 619, 10), // "setDisplay"
QT_MOC_LITERAL(36, 630, 10), // "decoration"
QT_MOC_LITERAL(37, 641, 13), // "setDecoration"
QT_MOC_LITERAL(38, 655, 4), // "edit"
QT_MOC_LITERAL(39, 660, 7), // "setEdit"
QT_MOC_LITERAL(40, 668, 7), // "toolTip"
QT_MOC_LITERAL(41, 676, 10), // "setToolTip"
QT_MOC_LITERAL(42, 687, 9), // "statusTip"
QT_MOC_LITERAL(43, 697, 12), // "setStatusTip"
QT_MOC_LITERAL(44, 710, 9), // "whatsThis"
QT_MOC_LITERAL(45, 720, 12), // "setWhatsThis"
QT_MOC_LITERAL(46, 733, 4), // "font"
QT_MOC_LITERAL(47, 738, 7), // "setFont"
QT_MOC_LITERAL(48, 746, 13), // "textAlignment"
QT_MOC_LITERAL(49, 760, 16), // "setTextAlignment"
QT_MOC_LITERAL(50, 777, 10), // "background"
QT_MOC_LITERAL(51, 788, 13), // "setBackground"
QT_MOC_LITERAL(52, 802, 10), // "foreground"
QT_MOC_LITERAL(53, 813, 13), // "setForeground"
QT_MOC_LITERAL(54, 827, 10), // "checkState"
QT_MOC_LITERAL(55, 838, 13), // "setCheckState"
QT_MOC_LITERAL(56, 852, 14), // "accessibleText"
QT_MOC_LITERAL(57, 867, 17), // "setAccessibleText"
QT_MOC_LITERAL(58, 885, 21), // "accessibleDescription"
QT_MOC_LITERAL(59, 907, 24), // "setAccessibleDescription"
QT_MOC_LITERAL(60, 932, 8), // "sizeHint"
QT_MOC_LITERAL(61, 941, 11) // "setSizeHint"

    },
    "QQmlTableModelColumn\0QML.Element\0"
    "TableModelColumn\0indexChanged\0\0"
    "displayChanged\0setDisplayChanged\0"
    "decorationChanged\0setDecorationChanged\0"
    "editChanged\0setEditChanged\0toolTipChanged\0"
    "setToolTipChanged\0statusTipChanged\0"
    "setStatusTipChanged\0whatsThisChanged\0"
    "setWhatsThisChanged\0fontChanged\0"
    "setFontChanged\0textAlignmentChanged\0"
    "setTextAlignmentChanged\0backgroundChanged\0"
    "setBackgroundChanged\0foregroundChanged\0"
    "setForegroundChanged\0checkStateChanged\0"
    "setCheckStateChanged\0accessibleTextChanged\0"
    "setAccessibleTextChanged\0"
    "accessibleDescriptionChanged\0"
    "setAccessibleDescriptionChanged\0"
    "sizeHintChanged\0setSizeHintChanged\0"
    "display\0QJSValue\0setDisplay\0decoration\0"
    "setDecoration\0edit\0setEdit\0toolTip\0"
    "setToolTip\0statusTip\0setStatusTip\0"
    "whatsThis\0setWhatsThis\0font\0setFont\0"
    "textAlignment\0setTextAlignment\0"
    "background\0setBackground\0foreground\0"
    "setForeground\0checkState\0setCheckState\0"
    "accessibleText\0setAccessibleText\0"
    "accessibleDescription\0setAccessibleDescription\0"
    "sizeHint\0setSizeHint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlTableModelColumn[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      29,   16, // methods
      28,  190, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      29,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  161,    4, 0x06 /* Public */,
       5,    0,  162,    4, 0x06 /* Public */,
       6,    0,  163,    4, 0x06 /* Public */,
       7,    0,  164,    4, 0x06 /* Public */,
       8,    0,  165,    4, 0x06 /* Public */,
       9,    0,  166,    4, 0x06 /* Public */,
      10,    0,  167,    4, 0x06 /* Public */,
      11,    0,  168,    4, 0x06 /* Public */,
      12,    0,  169,    4, 0x06 /* Public */,
      13,    0,  170,    4, 0x06 /* Public */,
      14,    0,  171,    4, 0x06 /* Public */,
      15,    0,  172,    4, 0x06 /* Public */,
      16,    0,  173,    4, 0x06 /* Public */,
      17,    0,  174,    4, 0x06 /* Public */,
      18,    0,  175,    4, 0x06 /* Public */,
      19,    0,  176,    4, 0x06 /* Public */,
      20,    0,  177,    4, 0x06 /* Public */,
      21,    0,  178,    4, 0x06 /* Public */,
      22,    0,  179,    4, 0x06 /* Public */,
      23,    0,  180,    4, 0x06 /* Public */,
      24,    0,  181,    4, 0x06 /* Public */,
      25,    0,  182,    4, 0x06 /* Public */,
      26,    0,  183,    4, 0x06 /* Public */,
      27,    0,  184,    4, 0x06 /* Public */,
      28,    0,  185,    4, 0x06 /* Public */,
      29,    0,  186,    4, 0x06 /* Public */,
      30,    0,  187,    4, 0x06 /* Public */,
      31,    0,  188,    4, 0x06 /* Public */,
      32,    0,  189,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      33, 0x80000000 | 34, 0x0049590b,
      35, 0x80000000 | 34, 0x0049510b,
      36, 0x80000000 | 34, 0x0049590b,
      37, 0x80000000 | 34, 0x0049590b,
      38, 0x80000000 | 34, 0x0049590b,
      39, 0x80000000 | 34, 0x0049590b,
      40, 0x80000000 | 34, 0x0049590b,
      41, 0x80000000 | 34, 0x0049590b,
      42, 0x80000000 | 34, 0x0049590b,
      43, 0x80000000 | 34, 0x0049590b,
      44, 0x80000000 | 34, 0x0049590b,
      45, 0x80000000 | 34, 0x0049590b,
      46, 0x80000000 | 34, 0x0049590b,
      47, 0x80000000 | 34, 0x0049590b,
      48, 0x80000000 | 34, 0x0049590b,
      49, 0x80000000 | 34, 0x0049590b,
      50, 0x80000000 | 34, 0x0049590b,
      51, 0x80000000 | 34, 0x0049590b,
      52, 0x80000000 | 34, 0x0049590b,
      53, 0x80000000 | 34, 0x0049590b,
      54, 0x80000000 | 34, 0x0049590b,
      55, 0x80000000 | 34, 0x0049590b,
      56, 0x80000000 | 34, 0x0049590b,
      57, 0x80000000 | 34, 0x0049590b,
      58, 0x80000000 | 34, 0x0049590b,
      59, 0x80000000 | 34, 0x0049590b,
      60, 0x80000000 | 34, 0x0049590b,
      61, 0x80000000 | 34, 0x0049590b,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,
      18,
      19,
      20,
      21,
      22,
      23,
      24,
      25,
      26,
      27,
      28,

       0        // eod
};

void QQmlTableModelColumn::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlTableModelColumn *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->indexChanged(); break;
        case 1: _t->displayChanged(); break;
        case 2: _t->setDisplayChanged(); break;
        case 3: _t->decorationChanged(); break;
        case 4: _t->setDecorationChanged(); break;
        case 5: _t->editChanged(); break;
        case 6: _t->setEditChanged(); break;
        case 7: _t->toolTipChanged(); break;
        case 8: _t->setToolTipChanged(); break;
        case 9: _t->statusTipChanged(); break;
        case 10: _t->setStatusTipChanged(); break;
        case 11: _t->whatsThisChanged(); break;
        case 12: _t->setWhatsThisChanged(); break;
        case 13: _t->fontChanged(); break;
        case 14: _t->setFontChanged(); break;
        case 15: _t->textAlignmentChanged(); break;
        case 16: _t->setTextAlignmentChanged(); break;
        case 17: _t->backgroundChanged(); break;
        case 18: _t->setBackgroundChanged(); break;
        case 19: _t->foregroundChanged(); break;
        case 20: _t->setForegroundChanged(); break;
        case 21: _t->checkStateChanged(); break;
        case 22: _t->setCheckStateChanged(); break;
        case 23: _t->accessibleTextChanged(); break;
        case 24: _t->setAccessibleTextChanged(); break;
        case 25: _t->accessibleDescriptionChanged(); break;
        case 26: _t->setAccessibleDescriptionChanged(); break;
        case 27: _t->sizeHintChanged(); break;
        case 28: _t->setSizeHintChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::indexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::displayChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setDisplayChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::decorationChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setDecorationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::editChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setEditChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::toolTipChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setToolTipChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::statusTipChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setStatusTipChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::whatsThisChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setWhatsThisChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::fontChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setFontChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::textAlignmentChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setTextAlignmentChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::backgroundChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setBackgroundChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::foregroundChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setForegroundChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::checkStateChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setCheckStateChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::accessibleTextChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setAccessibleTextChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::accessibleDescriptionChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setAccessibleDescriptionChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::sizeHintChanged)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQmlTableModelColumn::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModelColumn::setSizeHintChanged)) {
                *result = 28;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 27:
        case 26:
        case 25:
        case 24:
        case 23:
        case 22:
        case 21:
        case 20:
        case 19:
        case 18:
        case 17:
        case 16:
        case 15:
        case 14:
        case 13:
        case 12:
        case 11:
        case 10:
        case 9:
        case 8:
        case 7:
        case 6:
        case 5:
        case 4:
        case 3:
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlTableModelColumn *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QJSValue*>(_v) = _t->display(); break;
        case 1: *reinterpret_cast< QJSValue*>(_v) = _t->getSetDisplay(); break;
        case 2: *reinterpret_cast< QJSValue*>(_v) = _t->decoration(); break;
        case 3: *reinterpret_cast< QJSValue*>(_v) = _t->getSetDecoration(); break;
        case 4: *reinterpret_cast< QJSValue*>(_v) = _t->edit(); break;
        case 5: *reinterpret_cast< QJSValue*>(_v) = _t->getSetEdit(); break;
        case 6: *reinterpret_cast< QJSValue*>(_v) = _t->toolTip(); break;
        case 7: *reinterpret_cast< QJSValue*>(_v) = _t->getSetToolTip(); break;
        case 8: *reinterpret_cast< QJSValue*>(_v) = _t->statusTip(); break;
        case 9: *reinterpret_cast< QJSValue*>(_v) = _t->getSetStatusTip(); break;
        case 10: *reinterpret_cast< QJSValue*>(_v) = _t->whatsThis(); break;
        case 11: *reinterpret_cast< QJSValue*>(_v) = _t->getSetWhatsThis(); break;
        case 12: *reinterpret_cast< QJSValue*>(_v) = _t->font(); break;
        case 13: *reinterpret_cast< QJSValue*>(_v) = _t->getSetFont(); break;
        case 14: *reinterpret_cast< QJSValue*>(_v) = _t->textAlignment(); break;
        case 15: *reinterpret_cast< QJSValue*>(_v) = _t->getSetTextAlignment(); break;
        case 16: *reinterpret_cast< QJSValue*>(_v) = _t->background(); break;
        case 17: *reinterpret_cast< QJSValue*>(_v) = _t->getSetBackground(); break;
        case 18: *reinterpret_cast< QJSValue*>(_v) = _t->foreground(); break;
        case 19: *reinterpret_cast< QJSValue*>(_v) = _t->getSetForeground(); break;
        case 20: *reinterpret_cast< QJSValue*>(_v) = _t->checkState(); break;
        case 21: *reinterpret_cast< QJSValue*>(_v) = _t->getSetCheckState(); break;
        case 22: *reinterpret_cast< QJSValue*>(_v) = _t->accessibleText(); break;
        case 23: *reinterpret_cast< QJSValue*>(_v) = _t->getSetAccessibleText(); break;
        case 24: *reinterpret_cast< QJSValue*>(_v) = _t->accessibleDescription(); break;
        case 25: *reinterpret_cast< QJSValue*>(_v) = _t->getSetAccessibleDescription(); break;
        case 26: *reinterpret_cast< QJSValue*>(_v) = _t->sizeHint(); break;
        case 27: *reinterpret_cast< QJSValue*>(_v) = _t->getSetSizeHint(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlTableModelColumn *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDisplay(*reinterpret_cast< QJSValue*>(_v)); break;
        case 1: _t->setSetDisplay(*reinterpret_cast< QJSValue*>(_v)); break;
        case 2: _t->setDecoration(*reinterpret_cast< QJSValue*>(_v)); break;
        case 3: _t->setSetDecoration(*reinterpret_cast< QJSValue*>(_v)); break;
        case 4: _t->setEdit(*reinterpret_cast< QJSValue*>(_v)); break;
        case 5: _t->setSetEdit(*reinterpret_cast< QJSValue*>(_v)); break;
        case 6: _t->setToolTip(*reinterpret_cast< QJSValue*>(_v)); break;
        case 7: _t->setSetToolTip(*reinterpret_cast< QJSValue*>(_v)); break;
        case 8: _t->setStatusTip(*reinterpret_cast< QJSValue*>(_v)); break;
        case 9: _t->setSetStatusTip(*reinterpret_cast< QJSValue*>(_v)); break;
        case 10: _t->setWhatsThis(*reinterpret_cast< QJSValue*>(_v)); break;
        case 11: _t->setSetWhatsThis(*reinterpret_cast< QJSValue*>(_v)); break;
        case 12: _t->setFont(*reinterpret_cast< QJSValue*>(_v)); break;
        case 13: _t->setSetFont(*reinterpret_cast< QJSValue*>(_v)); break;
        case 14: _t->setTextAlignment(*reinterpret_cast< QJSValue*>(_v)); break;
        case 15: _t->setSetTextAlignment(*reinterpret_cast< QJSValue*>(_v)); break;
        case 16: _t->setBackground(*reinterpret_cast< QJSValue*>(_v)); break;
        case 17: _t->setSetBackground(*reinterpret_cast< QJSValue*>(_v)); break;
        case 18: _t->setForeground(*reinterpret_cast< QJSValue*>(_v)); break;
        case 19: _t->setSetForeground(*reinterpret_cast< QJSValue*>(_v)); break;
        case 20: _t->setCheckState(*reinterpret_cast< QJSValue*>(_v)); break;
        case 21: _t->setSetCheckState(*reinterpret_cast< QJSValue*>(_v)); break;
        case 22: _t->setAccessibleText(*reinterpret_cast< QJSValue*>(_v)); break;
        case 23: _t->setSetAccessibleText(*reinterpret_cast< QJSValue*>(_v)); break;
        case 24: _t->setAccessibleDescription(*reinterpret_cast< QJSValue*>(_v)); break;
        case 25: _t->setSetAccessibleDescription(*reinterpret_cast< QJSValue*>(_v)); break;
        case 26: _t->setSizeHint(*reinterpret_cast< QJSValue*>(_v)); break;
        case 27: _t->setSetSizeHint(*reinterpret_cast< QJSValue*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlTableModelColumn::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlTableModelColumn.data,
    qt_meta_data_QQmlTableModelColumn,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlTableModelColumn::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlTableModelColumn::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlTableModelColumn.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlTableModelColumn::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 29;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 28;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 28;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 28;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 28;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 28;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlTableModelColumn::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlTableModelColumn::displayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlTableModelColumn::setDisplayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlTableModelColumn::decorationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQmlTableModelColumn::setDecorationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQmlTableModelColumn::editChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQmlTableModelColumn::setEditChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQmlTableModelColumn::toolTipChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQmlTableModelColumn::setToolTipChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQmlTableModelColumn::statusTipChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQmlTableModelColumn::setStatusTipChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQmlTableModelColumn::whatsThisChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQmlTableModelColumn::setWhatsThisChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQmlTableModelColumn::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQmlTableModelColumn::setFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQmlTableModelColumn::textAlignmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQmlTableModelColumn::setTextAlignmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQmlTableModelColumn::backgroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QQmlTableModelColumn::setBackgroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QQmlTableModelColumn::foregroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQmlTableModelColumn::setForegroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQmlTableModelColumn::checkStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQmlTableModelColumn::setCheckStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQmlTableModelColumn::accessibleTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQmlTableModelColumn::setAccessibleTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQmlTableModelColumn::accessibleDescriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQmlTableModelColumn::setAccessibleDescriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QQmlTableModelColumn::sizeHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQmlTableModelColumn::setSizeHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
