/****************************************************************************
** Meta object code from reading C++ file 'qquickmousearea_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickmousearea_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickmousearea_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickMouseArea_t {
    QByteArrayData data[48];
    char stringdata0[705];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickMouseArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickMouseArea_t qt_meta_stringdata_QQuickMouseArea = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickMouseArea"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "MouseArea"
QT_MOC_LITERAL(3, 38, 14), // "hoveredChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 14), // "pressedChanged"
QT_MOC_LITERAL(6, 69, 14), // "enabledChanged"
QT_MOC_LITERAL(7, 84, 27), // "scrollGestureEnabledChanged"
QT_MOC_LITERAL(8, 112, 21), // "pressedButtonsChanged"
QT_MOC_LITERAL(9, 134, 22), // "acceptedButtonsChanged"
QT_MOC_LITERAL(10, 157, 19), // "hoverEnabledChanged"
QT_MOC_LITERAL(11, 177, 18), // "cursorShapeChanged"
QT_MOC_LITERAL(12, 196, 15), // "positionChanged"
QT_MOC_LITERAL(13, 212, 17), // "QQuickMouseEvent*"
QT_MOC_LITERAL(14, 230, 5), // "mouse"
QT_MOC_LITERAL(15, 236, 13), // "mouseXChanged"
QT_MOC_LITERAL(16, 250, 13), // "mouseYChanged"
QT_MOC_LITERAL(17, 264, 22), // "preventStealingChanged"
QT_MOC_LITERAL(18, 287, 30), // "propagateComposedEventsChanged"
QT_MOC_LITERAL(19, 318, 7), // "pressed"
QT_MOC_LITERAL(20, 326, 12), // "pressAndHold"
QT_MOC_LITERAL(21, 339, 8), // "released"
QT_MOC_LITERAL(22, 348, 7), // "clicked"
QT_MOC_LITERAL(23, 356, 13), // "doubleClicked"
QT_MOC_LITERAL(24, 370, 5), // "wheel"
QT_MOC_LITERAL(25, 376, 17), // "QQuickWheelEvent*"
QT_MOC_LITERAL(26, 394, 7), // "entered"
QT_MOC_LITERAL(27, 402, 6), // "exited"
QT_MOC_LITERAL(28, 409, 8), // "canceled"
QT_MOC_LITERAL(29, 418, 20), // "containsPressChanged"
QT_MOC_LITERAL(30, 439, 27), // "pressAndHoldIntervalChanged"
QT_MOC_LITERAL(31, 467, 6), // "mouseX"
QT_MOC_LITERAL(32, 474, 6), // "mouseY"
QT_MOC_LITERAL(33, 481, 13), // "containsMouse"
QT_MOC_LITERAL(34, 495, 7), // "enabled"
QT_MOC_LITERAL(35, 503, 20), // "scrollGestureEnabled"
QT_MOC_LITERAL(36, 524, 14), // "pressedButtons"
QT_MOC_LITERAL(37, 539, 16), // "Qt::MouseButtons"
QT_MOC_LITERAL(38, 556, 15), // "acceptedButtons"
QT_MOC_LITERAL(39, 572, 12), // "hoverEnabled"
QT_MOC_LITERAL(40, 585, 4), // "drag"
QT_MOC_LITERAL(41, 590, 11), // "QQuickDrag*"
QT_MOC_LITERAL(42, 602, 15), // "preventStealing"
QT_MOC_LITERAL(43, 618, 23), // "propagateComposedEvents"
QT_MOC_LITERAL(44, 642, 11), // "cursorShape"
QT_MOC_LITERAL(45, 654, 15), // "Qt::CursorShape"
QT_MOC_LITERAL(46, 670, 13), // "containsPress"
QT_MOC_LITERAL(47, 684, 20) // "pressAndHoldInterval"

    },
    "QQuickMouseArea\0QML.Element\0MouseArea\0"
    "hoveredChanged\0\0pressedChanged\0"
    "enabledChanged\0scrollGestureEnabledChanged\0"
    "pressedButtonsChanged\0acceptedButtonsChanged\0"
    "hoverEnabledChanged\0cursorShapeChanged\0"
    "positionChanged\0QQuickMouseEvent*\0"
    "mouse\0mouseXChanged\0mouseYChanged\0"
    "preventStealingChanged\0"
    "propagateComposedEventsChanged\0pressed\0"
    "pressAndHold\0released\0clicked\0"
    "doubleClicked\0wheel\0QQuickWheelEvent*\0"
    "entered\0exited\0canceled\0containsPressChanged\0"
    "pressAndHoldIntervalChanged\0mouseX\0"
    "mouseY\0containsMouse\0enabled\0"
    "scrollGestureEnabled\0pressedButtons\0"
    "Qt::MouseButtons\0acceptedButtons\0"
    "hoverEnabled\0drag\0QQuickDrag*\0"
    "preventStealing\0propagateComposedEvents\0"
    "cursorShape\0Qt::CursorShape\0containsPress\0"
    "pressAndHoldInterval"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickMouseArea[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      24,   16, // methods
      15,  202, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      24,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  160,    4, 0x06 /* Public */,
       5,    0,  161,    4, 0x06 /* Public */,
       6,    0,  162,    4, 0x06 /* Public */,
       7,    0,  163,    4, 0x86 /* Public | MethodRevisioned */,
       8,    0,  164,    4, 0x06 /* Public */,
       9,    0,  165,    4, 0x06 /* Public */,
      10,    0,  166,    4, 0x06 /* Public */,
      11,    0,  167,    4, 0x06 /* Public */,
      12,    1,  168,    4, 0x06 /* Public */,
      15,    1,  171,    4, 0x06 /* Public */,
      16,    1,  174,    4, 0x06 /* Public */,
      17,    0,  177,    4, 0x06 /* Public */,
      18,    0,  178,    4, 0x06 /* Public */,
      19,    1,  179,    4, 0x06 /* Public */,
      20,    1,  182,    4, 0x06 /* Public */,
      21,    1,  185,    4, 0x06 /* Public */,
      22,    1,  188,    4, 0x06 /* Public */,
      23,    1,  191,    4, 0x06 /* Public */,
      24,    1,  194,    4, 0x06 /* Public */,
      26,    0,  197,    4, 0x06 /* Public */,
      27,    0,  198,    4, 0x06 /* Public */,
      28,    0,  199,    4, 0x06 /* Public */,
      29,    0,  200,    4, 0x86 /* Public | MethodRevisioned */,
      30,    0,  201,    4, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       5,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       4,
       9,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 25,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      31, QMetaType::QReal, 0x00495001,
      32, QMetaType::QReal, 0x00495001,
      33, QMetaType::Bool, 0x00495001,
      19, QMetaType::Bool, 0x00495001,
      34, QMetaType::Bool, 0x00495103,
      35, QMetaType::Bool, 0x00c95103,
      36, 0x80000000 | 37, 0x00495009,
      38, 0x80000000 | 37, 0x0049510b,
      39, QMetaType::Bool, 0x00495103,
      40, 0x80000000 | 41, 0x00095409,
      42, QMetaType::Bool, 0x00495103,
      43, QMetaType::Bool, 0x00495103,
      44, 0x80000000 | 45, 0x0049510f,
      46, QMetaType::Bool, 0x00c95001,
      47, QMetaType::Int, 0x00c95107,

 // properties: notify_signal_id
       9,
      10,
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       0,
      11,
      12,
       7,
      22,
      23,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       5,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       4,
       9,

       0        // eod
};

void QQuickMouseArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickMouseArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->hoveredChanged(); break;
        case 1: _t->pressedChanged(); break;
        case 2: _t->enabledChanged(); break;
        case 3: _t->scrollGestureEnabledChanged(); break;
        case 4: _t->pressedButtonsChanged(); break;
        case 5: _t->acceptedButtonsChanged(); break;
        case 6: _t->hoverEnabledChanged(); break;
        case 7: _t->cursorShapeChanged(); break;
        case 8: _t->positionChanged((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 9: _t->mouseXChanged((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 10: _t->mouseYChanged((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 11: _t->preventStealingChanged(); break;
        case 12: _t->propagateComposedEventsChanged(); break;
        case 13: _t->pressed((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 14: _t->pressAndHold((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 15: _t->released((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 16: _t->clicked((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 17: _t->doubleClicked((*reinterpret_cast< QQuickMouseEvent*(*)>(_a[1]))); break;
        case 18: _t->wheel((*reinterpret_cast< QQuickWheelEvent*(*)>(_a[1]))); break;
        case 19: _t->entered(); break;
        case 20: _t->exited(); break;
        case 21: _t->canceled(); break;
        case 22: _t->containsPressChanged(); break;
        case 23: _t->pressAndHoldIntervalChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::hoveredChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::pressedChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::enabledChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::scrollGestureEnabledChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::pressedButtonsChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::acceptedButtonsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::hoverEnabledChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::cursorShapeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::positionChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::mouseXChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::mouseYChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::preventStealingChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::propagateComposedEventsChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::pressed)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::pressAndHold)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::released)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::clicked)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::doubleClicked)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)(QQuickWheelEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::wheel)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::entered)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::exited)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::canceled)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::containsPressChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickMouseArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMouseArea::pressAndHoldIntervalChanged)) {
                *result = 23;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickMouseArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->mouseX(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->mouseY(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->hovered(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->pressed(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isEnabled(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->isScrollGestureEnabled(); break;
        case 6: *reinterpret_cast< Qt::MouseButtons*>(_v) = _t->pressedButtons(); break;
        case 7: *reinterpret_cast< Qt::MouseButtons*>(_v) = _t->acceptedButtons(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->hoverEnabled(); break;
        case 9: *reinterpret_cast< QQuickDrag**>(_v) = _t->drag(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->preventStealing(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->propagateComposedEvents(); break;
        case 12: *reinterpret_cast< Qt::CursorShape*>(_v) = _t->cursorShape(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->containsPress(); break;
        case 14: *reinterpret_cast< int*>(_v) = _t->pressAndHoldInterval(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickMouseArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setScrollGestureEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setAcceptedButtons(*reinterpret_cast< Qt::MouseButtons*>(_v)); break;
        case 8: _t->setHoverEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setPreventStealing(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setPropagateComposedEvents(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setCursorShape(*reinterpret_cast< Qt::CursorShape*>(_v)); break;
        case 14: _t->setPressAndHoldInterval(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickMouseArea *_t = static_cast<QQuickMouseArea *>(_o);
        (void)_t;
        switch (_id) {
        case 12: _t->unsetCursor(); break;
        case 14: _t->resetPressAndHoldInterval(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickMouseArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickMouseArea.data,
    qt_meta_data_QQuickMouseArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickMouseArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickMouseArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickMouseArea.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickMouseArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickMouseArea::hoveredChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickMouseArea::pressedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickMouseArea::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickMouseArea::scrollGestureEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickMouseArea::pressedButtonsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickMouseArea::acceptedButtonsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickMouseArea::hoverEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickMouseArea::cursorShapeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickMouseArea::positionChanged(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickMouseArea::mouseXChanged(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickMouseArea::mouseYChanged(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickMouseArea::preventStealingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickMouseArea::propagateComposedEventsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickMouseArea::pressed(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickMouseArea::pressAndHold(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickMouseArea::released(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickMouseArea::clicked(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QQuickMouseArea::doubleClicked(QQuickMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickMouseArea::wheel(QQuickWheelEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickMouseArea::entered()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickMouseArea::exited()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQuickMouseArea::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickMouseArea::containsPressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickMouseArea::pressAndHoldIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
