/****************************************************************************
** Meta object code from reading C++ file 'quicktestresultforeign_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/testlib/quicktestresultforeign_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quicktestresultforeign_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickTestResultForeign_t {
    QByteArrayData data[5];
    char stringdata0[74];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickTestResultForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickTestResultForeign_t qt_meta_stringdata_QuickTestResultForeign = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QuickTestResultForeign"
QT_MOC_LITERAL(1, 23, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 35, 15), // "QuickTestResult"
QT_MOC_LITERAL(3, 51, 11), // "QML.Element"
QT_MOC_LITERAL(4, 63, 10) // "TestResult"

    },
    "QuickTestResultForeign\0QML.Foreign\0"
    "QuickTestResult\0QML.Element\0TestResult"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickTestResultForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QuickTestResultForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QuickTestResultForeign.data,
    qt_meta_data_QuickTestResultForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
