/****************************************************************************
** Meta object code from reading C++ file 'qquickfontmetrics_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/util/qquickfontmetrics_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickfontmetrics_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickFontMetrics_t {
    QByteArrayData data[31];
    char stringdata0[377];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickFontMetrics_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickFontMetrics_t qt_meta_stringdata_QQuickFontMetrics = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQuickFontMetrics"
QT_MOC_LITERAL(1, 18, 11), // "QML.Element"
QT_MOC_LITERAL(2, 30, 11), // "FontMetrics"
QT_MOC_LITERAL(3, 42, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 66, 1), // "4"
QT_MOC_LITERAL(5, 68, 11), // "fontChanged"
QT_MOC_LITERAL(6, 80, 0), // ""
QT_MOC_LITERAL(7, 81, 4), // "font"
QT_MOC_LITERAL(8, 86, 12), // "advanceWidth"
QT_MOC_LITERAL(9, 99, 4), // "text"
QT_MOC_LITERAL(10, 104, 12), // "boundingRect"
QT_MOC_LITERAL(11, 117, 17), // "tightBoundingRect"
QT_MOC_LITERAL(12, 135, 10), // "elidedText"
QT_MOC_LITERAL(13, 146, 17), // "Qt::TextElideMode"
QT_MOC_LITERAL(14, 164, 4), // "mode"
QT_MOC_LITERAL(15, 169, 5), // "width"
QT_MOC_LITERAL(16, 175, 5), // "flags"
QT_MOC_LITERAL(17, 181, 6), // "ascent"
QT_MOC_LITERAL(18, 188, 7), // "descent"
QT_MOC_LITERAL(19, 196, 6), // "height"
QT_MOC_LITERAL(20, 203, 7), // "leading"
QT_MOC_LITERAL(21, 211, 11), // "lineSpacing"
QT_MOC_LITERAL(22, 223, 18), // "minimumLeftBearing"
QT_MOC_LITERAL(23, 242, 19), // "minimumRightBearing"
QT_MOC_LITERAL(24, 262, 21), // "maximumCharacterWidth"
QT_MOC_LITERAL(25, 284, 7), // "xHeight"
QT_MOC_LITERAL(26, 292, 21), // "averageCharacterWidth"
QT_MOC_LITERAL(27, 314, 17), // "underlinePosition"
QT_MOC_LITERAL(28, 332, 16), // "overlinePosition"
QT_MOC_LITERAL(29, 349, 17), // "strikeOutPosition"
QT_MOC_LITERAL(30, 367, 9) // "lineWidth"

    },
    "QQuickFontMetrics\0QML.Element\0FontMetrics\0"
    "QML.AddedInMinorVersion\0""4\0fontChanged\0"
    "\0font\0advanceWidth\0text\0boundingRect\0"
    "tightBoundingRect\0elidedText\0"
    "Qt::TextElideMode\0mode\0width\0flags\0"
    "ascent\0descent\0height\0leading\0lineSpacing\0"
    "minimumLeftBearing\0minimumRightBearing\0"
    "maximumCharacterWidth\0xHeight\0"
    "averageCharacterWidth\0underlinePosition\0"
    "overlinePosition\0strikeOutPosition\0"
    "lineWidth"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickFontMetrics[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       6,   18, // methods
      15,   76, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   48,    6, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       8,    1,   51,    6, 0x02 /* Public */,
      10,    1,   54,    6, 0x02 /* Public */,
      11,    1,   57,    6, 0x02 /* Public */,
      12,    4,   60,    6, 0x02 /* Public */,
      12,    3,   69,    6, 0x22 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::QFont,    7,

 // methods: parameters
    QMetaType::QReal, QMetaType::QString,    9,
    QMetaType::QRectF, QMetaType::QString,    9,
    QMetaType::QRectF, QMetaType::QString,    9,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13, QMetaType::QReal, QMetaType::Int,    9,   14,   15,   16,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 13, QMetaType::QReal,    9,   14,   15,

 // properties: name, type, flags
       7, QMetaType::QFont, 0x00495103,
      17, QMetaType::QReal, 0x00495001,
      18, QMetaType::QReal, 0x00495001,
      19, QMetaType::QReal, 0x00495001,
      20, QMetaType::QReal, 0x00495001,
      21, QMetaType::QReal, 0x00495001,
      22, QMetaType::QReal, 0x00495001,
      23, QMetaType::QReal, 0x00495001,
      24, QMetaType::QReal, 0x00495001,
      25, QMetaType::QReal, 0x00495001,
      26, QMetaType::QReal, 0x00495001,
      27, QMetaType::QReal, 0x00495001,
      28, QMetaType::QReal, 0x00495001,
      29, QMetaType::QReal, 0x00495001,
      30, QMetaType::QReal, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

       0        // eod
};

void QQuickFontMetrics::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickFontMetrics *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 1: { qreal _r = _t->advanceWidth((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< qreal*>(_a[0]) = std::move(_r); }  break;
        case 2: { QRectF _r = _t->boundingRect((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 3: { QRectF _r = _t->tightBoundingRect((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->elidedText((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Qt::TextElideMode(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->elidedText((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Qt::TextElideMode(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickFontMetrics::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFontMetrics::fontChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickFontMetrics *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->ascent(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->descent(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->height(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->leading(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->lineSpacing(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->minimumLeftBearing(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->minimumRightBearing(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->maximumCharacterWidth(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->xHeight(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->averageCharacterWidth(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->underlinePosition(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->overlinePosition(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->strikeOutPosition(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->lineWidth(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickFontMetrics *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFont(*reinterpret_cast< QFont*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickFontMetrics::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickFontMetrics.data,
    qt_meta_data_QQuickFontMetrics,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickFontMetrics::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickFontMetrics::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickFontMetrics.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickFontMetrics::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickFontMetrics::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
