/****************************************************************************
** Meta object code from reading C++ file 'qquickgraphicsinfo_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickgraphicsinfo_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgraphicsinfo_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickGraphicsInfo_t {
    QByteArrayData data[62];
    char stringdata0[937];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGraphicsInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGraphicsInfo_t qt_meta_stringdata_QQuickGraphicsInfo = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickGraphicsInfo"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "GraphicsInfo"
QT_MOC_LITERAL(3, 44, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 68, 1), // "8"
QT_MOC_LITERAL(5, 70, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 84, 5), // "false"
QT_MOC_LITERAL(7, 90, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 112, 55), // "GraphicsInfo is only availabl..."
QT_MOC_LITERAL(9, 168, 12), // "QML.Attached"
QT_MOC_LITERAL(10, 181, 10), // "apiChanged"
QT_MOC_LITERAL(11, 192, 0), // ""
QT_MOC_LITERAL(12, 193, 17), // "shaderTypeChanged"
QT_MOC_LITERAL(13, 211, 28), // "shaderCompilationTypeChanged"
QT_MOC_LITERAL(14, 240, 23), // "shaderSourceTypeChanged"
QT_MOC_LITERAL(15, 264, 19), // "majorVersionChanged"
QT_MOC_LITERAL(16, 284, 19), // "minorVersionChanged"
QT_MOC_LITERAL(17, 304, 14), // "profileChanged"
QT_MOC_LITERAL(18, 319, 21), // "renderableTypeChanged"
QT_MOC_LITERAL(19, 341, 10), // "updateInfo"
QT_MOC_LITERAL(20, 352, 9), // "setWindow"
QT_MOC_LITERAL(21, 362, 13), // "QQuickWindow*"
QT_MOC_LITERAL(22, 376, 6), // "window"
QT_MOC_LITERAL(23, 383, 3), // "api"
QT_MOC_LITERAL(24, 387, 11), // "GraphicsApi"
QT_MOC_LITERAL(25, 399, 10), // "shaderType"
QT_MOC_LITERAL(26, 410, 10), // "ShaderType"
QT_MOC_LITERAL(27, 421, 21), // "shaderCompilationType"
QT_MOC_LITERAL(28, 443, 21), // "ShaderCompilationType"
QT_MOC_LITERAL(29, 465, 16), // "shaderSourceType"
QT_MOC_LITERAL(30, 482, 16), // "ShaderSourceType"
QT_MOC_LITERAL(31, 499, 12), // "majorVersion"
QT_MOC_LITERAL(32, 512, 12), // "minorVersion"
QT_MOC_LITERAL(33, 525, 7), // "profile"
QT_MOC_LITERAL(34, 533, 20), // "OpenGLContextProfile"
QT_MOC_LITERAL(35, 554, 14), // "renderableType"
QT_MOC_LITERAL(36, 569, 14), // "RenderableType"
QT_MOC_LITERAL(37, 584, 7), // "Unknown"
QT_MOC_LITERAL(38, 592, 8), // "Software"
QT_MOC_LITERAL(39, 601, 6), // "OpenGL"
QT_MOC_LITERAL(40, 608, 10), // "Direct3D12"
QT_MOC_LITERAL(41, 619, 6), // "OpenVG"
QT_MOC_LITERAL(42, 626, 9), // "OpenGLRhi"
QT_MOC_LITERAL(43, 636, 13), // "Direct3D11Rhi"
QT_MOC_LITERAL(44, 650, 9), // "VulkanRhi"
QT_MOC_LITERAL(45, 660, 8), // "MetalRhi"
QT_MOC_LITERAL(46, 669, 7), // "NullRhi"
QT_MOC_LITERAL(47, 677, 22), // "UnknownShadingLanguage"
QT_MOC_LITERAL(48, 700, 4), // "GLSL"
QT_MOC_LITERAL(49, 705, 4), // "HLSL"
QT_MOC_LITERAL(50, 710, 9), // "RhiShader"
QT_MOC_LITERAL(51, 720, 18), // "RuntimeCompilation"
QT_MOC_LITERAL(52, 739, 18), // "OfflineCompilation"
QT_MOC_LITERAL(53, 758, 18), // "ShaderSourceString"
QT_MOC_LITERAL(54, 777, 16), // "ShaderSourceFile"
QT_MOC_LITERAL(55, 794, 14), // "ShaderByteCode"
QT_MOC_LITERAL(56, 809, 15), // "OpenGLNoProfile"
QT_MOC_LITERAL(57, 825, 17), // "OpenGLCoreProfile"
QT_MOC_LITERAL(58, 843, 26), // "OpenGLCompatibilityProfile"
QT_MOC_LITERAL(59, 870, 24), // "SurfaceFormatUnspecified"
QT_MOC_LITERAL(60, 895, 19), // "SurfaceFormatOpenGL"
QT_MOC_LITERAL(61, 915, 21) // "SurfaceFormatOpenGLES"

    },
    "QQuickGraphicsInfo\0QML.Element\0"
    "GraphicsInfo\0QML.AddedInMinorVersion\0"
    "8\0QML.Creatable\0false\0QML.UncreatableReason\0"
    "GraphicsInfo is only available via attached properties.\0"
    "QML.Attached\0apiChanged\0\0shaderTypeChanged\0"
    "shaderCompilationTypeChanged\0"
    "shaderSourceTypeChanged\0majorVersionChanged\0"
    "minorVersionChanged\0profileChanged\0"
    "renderableTypeChanged\0updateInfo\0"
    "setWindow\0QQuickWindow*\0window\0api\0"
    "GraphicsApi\0shaderType\0ShaderType\0"
    "shaderCompilationType\0ShaderCompilationType\0"
    "shaderSourceType\0ShaderSourceType\0"
    "majorVersion\0minorVersion\0profile\0"
    "OpenGLContextProfile\0renderableType\0"
    "RenderableType\0Unknown\0Software\0OpenGL\0"
    "Direct3D12\0OpenVG\0OpenGLRhi\0Direct3D11Rhi\0"
    "VulkanRhi\0MetalRhi\0NullRhi\0"
    "UnknownShadingLanguage\0GLSL\0HLSL\0"
    "RhiShader\0RuntimeCompilation\0"
    "OfflineCompilation\0ShaderSourceString\0"
    "ShaderSourceFile\0ShaderByteCode\0"
    "OpenGLNoProfile\0OpenGLCoreProfile\0"
    "OpenGLCompatibilityProfile\0"
    "SurfaceFormatUnspecified\0SurfaceFormatOpenGL\0"
    "SurfaceFormatOpenGLES"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGraphicsInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
      10,   24, // methods
       8,   86, // properties
       6,  118, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,    0,

 // signals: name, argc, parameters, tag, flags
      10,    0,   74,   11, 0x06 /* Public */,
      12,    0,   75,   11, 0x06 /* Public */,
      13,    0,   76,   11, 0x06 /* Public */,
      14,    0,   77,   11, 0x06 /* Public */,
      15,    0,   78,   11, 0x06 /* Public */,
      16,    0,   79,   11, 0x06 /* Public */,
      17,    0,   80,   11, 0x06 /* Public */,
      18,    0,   81,   11, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    0,   82,   11, 0x08 /* Private */,
      20,    1,   83,   11, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,   22,

 // properties: name, type, flags
      23, 0x80000000 | 24, 0x00495809,
      25, 0x80000000 | 26, 0x00495809,
      27, 0x80000000 | 28, 0x00495809,
      29, 0x80000000 | 30, 0x00495809,
      31, QMetaType::Int, 0x00495801,
      32, QMetaType::Int, 0x00495801,
      33, 0x80000000 | 34, 0x00495809,
      35, 0x80000000 | 36, 0x00495809,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,

 // enums: name, alias, flags, count, data
      24,   24, 0x0,   10,  148,
      26,   26, 0x0,    4,  168,
      28,   28, 0x0,    2,  176,
      30,   30, 0x0,    3,  180,
      34,   34, 0x0,    3,  186,
      36,   36, 0x0,    3,  192,

 // enum data: key, value
      37, uint(QQuickGraphicsInfo::Unknown),
      38, uint(QQuickGraphicsInfo::Software),
      39, uint(QQuickGraphicsInfo::OpenGL),
      40, uint(QQuickGraphicsInfo::Direct3D12),
      41, uint(QQuickGraphicsInfo::OpenVG),
      42, uint(QQuickGraphicsInfo::OpenGLRhi),
      43, uint(QQuickGraphicsInfo::Direct3D11Rhi),
      44, uint(QQuickGraphicsInfo::VulkanRhi),
      45, uint(QQuickGraphicsInfo::MetalRhi),
      46, uint(QQuickGraphicsInfo::NullRhi),
      47, uint(QQuickGraphicsInfo::UnknownShadingLanguage),
      48, uint(QQuickGraphicsInfo::GLSL),
      49, uint(QQuickGraphicsInfo::HLSL),
      50, uint(QQuickGraphicsInfo::RhiShader),
      51, uint(QQuickGraphicsInfo::RuntimeCompilation),
      52, uint(QQuickGraphicsInfo::OfflineCompilation),
      53, uint(QQuickGraphicsInfo::ShaderSourceString),
      54, uint(QQuickGraphicsInfo::ShaderSourceFile),
      55, uint(QQuickGraphicsInfo::ShaderByteCode),
      56, uint(QQuickGraphicsInfo::OpenGLNoProfile),
      57, uint(QQuickGraphicsInfo::OpenGLCoreProfile),
      58, uint(QQuickGraphicsInfo::OpenGLCompatibilityProfile),
      59, uint(QQuickGraphicsInfo::SurfaceFormatUnspecified),
      60, uint(QQuickGraphicsInfo::SurfaceFormatOpenGL),
      61, uint(QQuickGraphicsInfo::SurfaceFormatOpenGLES),

       0        // eod
};

void QQuickGraphicsInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGraphicsInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->apiChanged(); break;
        case 1: _t->shaderTypeChanged(); break;
        case 2: _t->shaderCompilationTypeChanged(); break;
        case 3: _t->shaderSourceTypeChanged(); break;
        case 4: _t->majorVersionChanged(); break;
        case 5: _t->minorVersionChanged(); break;
        case 6: _t->profileChanged(); break;
        case 7: _t->renderableTypeChanged(); break;
        case 8: _t->updateInfo(); break;
        case 9: _t->setWindow((*reinterpret_cast< QQuickWindow*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::apiChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::shaderTypeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::shaderCompilationTypeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::shaderSourceTypeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::majorVersionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::minorVersionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::profileChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickGraphicsInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGraphicsInfo::renderableTypeChanged)) {
                *result = 7;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGraphicsInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< GraphicsApi*>(_v) = _t->api(); break;
        case 1: *reinterpret_cast< ShaderType*>(_v) = _t->shaderType(); break;
        case 2: *reinterpret_cast< ShaderCompilationType*>(_v) = _t->shaderCompilationType(); break;
        case 3: *reinterpret_cast< ShaderSourceType*>(_v) = _t->shaderSourceType(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 6: *reinterpret_cast< OpenGLContextProfile*>(_v) = _t->profile(); break;
        case 7: *reinterpret_cast< RenderableType*>(_v) = _t->renderableType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickGraphicsInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickGraphicsInfo.data,
    qt_meta_data_QQuickGraphicsInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGraphicsInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGraphicsInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGraphicsInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickGraphicsInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickGraphicsInfo::apiChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickGraphicsInfo::shaderTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickGraphicsInfo::shaderCompilationTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickGraphicsInfo::shaderSourceTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickGraphicsInfo::majorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickGraphicsInfo::minorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickGraphicsInfo::profileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickGraphicsInfo::renderableTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
