/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <quicktestevent_p.h>
#include <quicktestresultforeign_p.h>
#include <quicktestutil_p.h>

void qml_register_types_QtTest()
{
    qmlRegisterTypesAndRevisions<QQuickTouchEventSequence>("QtTest", 1);
    qmlRegisterTypesAndRevisions<QuickTestEvent>("QtTest", 1);
    qmlRegisterTypesAndRevisions<QuickTestResultForeign>("QtTest", 1);
    qmlRegisterTypesAndRevisions<QuickTestUtil>("QtTest", 1);
    qmlRegisterModule("QtTest", 1, 15);
}

static const QQmlModuleRegistration registration("QtTest", 1, qml_register_types_QtTest);
