/****************************************************************************
** Meta object code from reading C++ file 'qv4debugclient_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmldebug/qv4debugclient_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qv4debugclient_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QV4DebugClient_t {
    QByteArrayData data[7];
    char stringdata0[61];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QV4DebugClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QV4DebugClient_t qt_meta_stringdata_QV4DebugClient = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QV4DebugClient"
QT_MOC_LITERAL(1, 15, 9), // "connected"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 11), // "interrupted"
QT_MOC_LITERAL(4, 38, 6), // "result"
QT_MOC_LITERAL(5, 45, 7), // "failure"
QT_MOC_LITERAL(6, 53, 7) // "stopped"

    },
    "QV4DebugClient\0connected\0\0interrupted\0"
    "result\0failure\0stopped"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QV4DebugClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    0,   42,    2, 0x06 /* Public */,
       6,    0,   43,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QV4DebugClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QV4DebugClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->interrupted(); break;
        case 2: _t->result(); break;
        case 3: _t->failure(); break;
        case 4: _t->stopped(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QV4DebugClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QV4DebugClient::connected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QV4DebugClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QV4DebugClient::interrupted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QV4DebugClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QV4DebugClient::result)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QV4DebugClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QV4DebugClient::failure)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QV4DebugClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QV4DebugClient::stopped)) {
                *result = 4;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QV4DebugClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugClient::staticMetaObject>(),
    qt_meta_stringdata_QV4DebugClient.data,
    qt_meta_data_QV4DebugClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QV4DebugClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QV4DebugClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QV4DebugClient.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugClient::qt_metacast(_clname);
}

int QV4DebugClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugClient::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QV4DebugClient::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QV4DebugClient::interrupted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QV4DebugClient::result()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QV4DebugClient::failure()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QV4DebugClient::stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
