/****************************************************************************
** Meta object code from reading C++ file 'qqmlobjectmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmlmodels/qqmlobjectmodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlobjectmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlInstanceModel_t {
    QByteArrayData data[17];
    char stringdata0[174];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlInstanceModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlInstanceModel_t qt_meta_stringdata_QQmlInstanceModel = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQmlInstanceModel"
QT_MOC_LITERAL(1, 18, 11), // "QML.Element"
QT_MOC_LITERAL(2, 30, 9), // "anonymous"
QT_MOC_LITERAL(3, 40, 12), // "countChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 12), // "modelUpdated"
QT_MOC_LITERAL(6, 67, 13), // "QQmlChangeSet"
QT_MOC_LITERAL(7, 81, 9), // "changeSet"
QT_MOC_LITERAL(8, 91, 5), // "reset"
QT_MOC_LITERAL(9, 97, 11), // "createdItem"
QT_MOC_LITERAL(10, 109, 5), // "index"
QT_MOC_LITERAL(11, 115, 6), // "object"
QT_MOC_LITERAL(12, 122, 8), // "initItem"
QT_MOC_LITERAL(13, 131, 14), // "destroyingItem"
QT_MOC_LITERAL(14, 146, 10), // "itemPooled"
QT_MOC_LITERAL(15, 157, 10), // "itemReused"
QT_MOC_LITERAL(16, 168, 5) // "count"

    },
    "QQmlInstanceModel\0QML.Element\0anonymous\0"
    "countChanged\0\0modelUpdated\0QQmlChangeSet\0"
    "changeSet\0reset\0createdItem\0index\0"
    "object\0initItem\0destroyingItem\0"
    "itemPooled\0itemReused\0count"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlInstanceModel[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       1,   87, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   58,    4, 0x06 /* Public */,
       5,    2,   59,    4, 0x06 /* Public */,
       9,    2,   64,    4, 0x06 /* Public */,
      12,    2,   69,    4, 0x06 /* Public */,
      13,    1,   74,    4, 0x06 /* Public */,
      14,    2,   77,    4, 0x86 /* Public | MethodRevisioned */,
      15,    2,   82,    4, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
      15,
      15,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool,    7,    8,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   10,   11,
    QMetaType::Void, QMetaType::QObjectStar,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   10,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   10,   11,

 // properties: name, type, flags
      16, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQmlInstanceModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlInstanceModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->modelUpdated((*reinterpret_cast< const QQmlChangeSet(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->createdItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 3: _t->initItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 4: _t->destroyingItem((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 5: _t->itemPooled((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 6: _t->itemReused((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlInstanceModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::countChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(const QQmlChangeSet & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::modelUpdated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::createdItem)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::initItem)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::destroyingItem)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::itemPooled)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQmlInstanceModel::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstanceModel::itemReused)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlInstanceModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlInstanceModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlInstanceModel.data,
    qt_meta_data_QQmlInstanceModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlInstanceModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlInstanceModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlInstanceModel.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlInstanceModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlInstanceModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlInstanceModel::modelUpdated(const QQmlChangeSet & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQmlInstanceModel::createdItem(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQmlInstanceModel::initItem(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQmlInstanceModel::destroyingItem(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQmlInstanceModel::itemPooled(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQmlInstanceModel::itemReused(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
struct qt_meta_stringdata_QQmlObjectModel_t {
    QByteArrayData data[23];
    char stringdata0[230];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlObjectModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlObjectModel_t qt_meta_stringdata_QQmlObjectModel = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQmlObjectModel"
QT_MOC_LITERAL(1, 16, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 32, 8), // "children"
QT_MOC_LITERAL(3, 41, 11), // "QML.Element"
QT_MOC_LITERAL(4, 53, 11), // "ObjectModel"
QT_MOC_LITERAL(5, 65, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 89, 1), // "1"
QT_MOC_LITERAL(7, 91, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 104, 23), // "QQmlObjectModelAttached"
QT_MOC_LITERAL(9, 128, 15), // "childrenChanged"
QT_MOC_LITERAL(10, 144, 0), // ""
QT_MOC_LITERAL(11, 145, 5), // "clear"
QT_MOC_LITERAL(12, 151, 3), // "get"
QT_MOC_LITERAL(13, 155, 5), // "index"
QT_MOC_LITERAL(14, 161, 6), // "append"
QT_MOC_LITERAL(15, 168, 6), // "object"
QT_MOC_LITERAL(16, 175, 6), // "insert"
QT_MOC_LITERAL(17, 182, 4), // "move"
QT_MOC_LITERAL(18, 187, 4), // "from"
QT_MOC_LITERAL(19, 192, 2), // "to"
QT_MOC_LITERAL(20, 195, 1), // "n"
QT_MOC_LITERAL(21, 197, 6), // "remove"
QT_MOC_LITERAL(22, 204, 25) // "QQmlListProperty<QObject>"

    },
    "QQmlObjectModel\0DefaultProperty\0"
    "children\0QML.Element\0ObjectModel\0"
    "QML.AddedInMinorVersion\0""1\0QML.Attached\0"
    "QQmlObjectModelAttached\0childrenChanged\0"
    "\0clear\0get\0index\0append\0object\0insert\0"
    "move\0from\0to\0n\0remove\0QQmlListProperty<QObject>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlObjectModel[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       9,   22, // methods
       1,  109, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,   76,   10, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   77,   10, 0x8a /* Public | MethodRevisioned */,

 // methods: name, argc, parameters, tag, flags
      12,    1,   78,   10, 0x82 /* Public | MethodRevisioned */,
      14,    1,   81,   10, 0x82 /* Public | MethodRevisioned */,
      16,    2,   84,   10, 0x82 /* Public | MethodRevisioned */,
      17,    3,   89,   10, 0x82 /* Public | MethodRevisioned */,
      17,    2,   96,   10, 0xa2 /* Public | MethodCloned | MethodRevisioned */,
      21,    2,  101,   10, 0x82 /* Public | MethodRevisioned */,
      21,    1,  106,   10, 0xa2 /* Public | MethodCloned | MethodRevisioned */,

 // signals: revision
       0,

 // slots: revision
       3,

 // methods: revision
       3,
       3,
       3,
       3,
       3,
       3,
       3,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::QObjectStar, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::QObjectStar,   15,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   13,   15,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   18,   19,   20,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   18,   19,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   13,   20,
    QMetaType::Void, QMetaType::Int,   13,

 // properties: name, type, flags
       2, 0x80000000 | 22, 0x00494009,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQmlObjectModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlObjectModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->childrenChanged(); break;
        case 1: _t->clear(); break;
        case 2: { QObject* _r = _t->get((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 3: _t->append((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 4: _t->insert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 5: _t->move((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->move((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->remove((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->remove((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlObjectModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlObjectModel::childrenChanged)) {
                *result = 0;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlObjectModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->children(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlObjectModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlInstanceModel::staticMetaObject>(),
    qt_meta_stringdata_QQmlObjectModel.data,
    qt_meta_data_QQmlObjectModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlObjectModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlObjectModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlObjectModel.stringdata0))
        return static_cast<void*>(this);
    return QQmlInstanceModel::qt_metacast(_clname);
}

int QQmlObjectModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlInstanceModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlObjectModel::childrenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQmlObjectModelAttached_t {
    QByteArrayData data[4];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlObjectModelAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlObjectModelAttached_t qt_meta_stringdata_QQmlObjectModelAttached = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQmlObjectModelAttached"
QT_MOC_LITERAL(1, 24, 12), // "indexChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 5) // "index"

    },
    "QQmlObjectModelAttached\0indexChanged\0"
    "\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlObjectModelAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   20, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       3, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQmlObjectModelAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlObjectModelAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->indexChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlObjectModelAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlObjectModelAttached::indexChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlObjectModelAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->index(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlObjectModelAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlObjectModelAttached.data,
    qt_meta_data_QQmlObjectModelAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlObjectModelAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlObjectModelAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlObjectModelAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlObjectModelAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlObjectModelAttached::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
