/****************************************************************************
** Meta object code from reading C++ file 'qquickgridview_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickgridview_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickgridview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickGridView_t {
    QByteArrayData data[28];
    char stringdata0[378];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGridView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGridView_t qt_meta_stringdata_QQuickGridView = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickGridView"
QT_MOC_LITERAL(1, 15, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 31, 4), // "data"
QT_MOC_LITERAL(3, 36, 11), // "QML.Element"
QT_MOC_LITERAL(4, 48, 8), // "GridView"
QT_MOC_LITERAL(5, 57, 12), // "QML.Attached"
QT_MOC_LITERAL(6, 70, 22), // "QQuickGridViewAttached"
QT_MOC_LITERAL(7, 93, 16), // "cellWidthChanged"
QT_MOC_LITERAL(8, 110, 0), // ""
QT_MOC_LITERAL(9, 111, 17), // "cellHeightChanged"
QT_MOC_LITERAL(10, 129, 28), // "highlightMoveDurationChanged"
QT_MOC_LITERAL(11, 158, 11), // "flowChanged"
QT_MOC_LITERAL(12, 170, 15), // "snapModeChanged"
QT_MOC_LITERAL(13, 186, 18), // "moveCurrentIndexUp"
QT_MOC_LITERAL(14, 205, 20), // "moveCurrentIndexDown"
QT_MOC_LITERAL(15, 226, 20), // "moveCurrentIndexLeft"
QT_MOC_LITERAL(16, 247, 21), // "moveCurrentIndexRight"
QT_MOC_LITERAL(17, 269, 4), // "flow"
QT_MOC_LITERAL(18, 274, 4), // "Flow"
QT_MOC_LITERAL(19, 279, 9), // "cellWidth"
QT_MOC_LITERAL(20, 289, 10), // "cellHeight"
QT_MOC_LITERAL(21, 300, 8), // "snapMode"
QT_MOC_LITERAL(22, 309, 8), // "SnapMode"
QT_MOC_LITERAL(23, 318, 15), // "FlowLeftToRight"
QT_MOC_LITERAL(24, 334, 15), // "FlowTopToBottom"
QT_MOC_LITERAL(25, 350, 6), // "NoSnap"
QT_MOC_LITERAL(26, 357, 9), // "SnapToRow"
QT_MOC_LITERAL(27, 367, 10) // "SnapOneRow"

    },
    "QQuickGridView\0DefaultProperty\0data\0"
    "QML.Element\0GridView\0QML.Attached\0"
    "QQuickGridViewAttached\0cellWidthChanged\0"
    "\0cellHeightChanged\0highlightMoveDurationChanged\0"
    "flowChanged\0snapModeChanged\0"
    "moveCurrentIndexUp\0moveCurrentIndexDown\0"
    "moveCurrentIndexLeft\0moveCurrentIndexRight\0"
    "flow\0Flow\0cellWidth\0cellHeight\0snapMode\0"
    "SnapMode\0FlowLeftToRight\0FlowTopToBottom\0"
    "NoSnap\0SnapToRow\0SnapOneRow"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGridView[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       9,   20, // methods
       4,   74, // properties
       2,   90, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   65,    8, 0x06 /* Public */,
       9,    0,   66,    8, 0x06 /* Public */,
      10,    0,   67,    8, 0x06 /* Public */,
      11,    0,   68,    8, 0x06 /* Public */,
      12,    0,   69,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    0,   70,    8, 0x0a /* Public */,
      14,    0,   71,    8, 0x0a /* Public */,
      15,    0,   72,    8, 0x0a /* Public */,
      16,    0,   73,    8, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      17, 0x80000000 | 18, 0x0049510b,
      19, QMetaType::QReal, 0x00495103,
      20, QMetaType::QReal, 0x00495103,
      21, 0x80000000 | 22, 0x0049510b,

 // properties: notify_signal_id
       3,
       0,
       1,
       4,

 // enums: name, alias, flags, count, data
      18,   18, 0x0,    2,  100,
      22,   22, 0x0,    3,  104,

 // enum data: key, value
      23, uint(QQuickGridView::FlowLeftToRight),
      24, uint(QQuickGridView::FlowTopToBottom),
      25, uint(QQuickGridView::NoSnap),
      26, uint(QQuickGridView::SnapToRow),
      27, uint(QQuickGridView::SnapOneRow),

       0        // eod
};

void QQuickGridView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGridView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cellWidthChanged(); break;
        case 1: _t->cellHeightChanged(); break;
        case 2: _t->highlightMoveDurationChanged(); break;
        case 3: _t->flowChanged(); break;
        case 4: _t->snapModeChanged(); break;
        case 5: _t->moveCurrentIndexUp(); break;
        case 6: _t->moveCurrentIndexDown(); break;
        case 7: _t->moveCurrentIndexLeft(); break;
        case 8: _t->moveCurrentIndexRight(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickGridView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridView::cellWidthChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickGridView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridView::cellHeightChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickGridView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridView::highlightMoveDurationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickGridView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridView::flowChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickGridView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridView::snapModeChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGridView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Flow*>(_v) = _t->flow(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->cellWidth(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->cellHeight(); break;
        case 3: *reinterpret_cast< SnapMode*>(_v) = _t->snapMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickGridView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlow(*reinterpret_cast< Flow*>(_v)); break;
        case 1: _t->setCellWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setCellHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setSnapMode(*reinterpret_cast< SnapMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickGridView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItemView::staticMetaObject>(),
    qt_meta_stringdata_QQuickGridView.data,
    qt_meta_data_QQuickGridView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGridView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGridView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGridView.stringdata0))
        return static_cast<void*>(this);
    return QQuickItemView::qt_metacast(_clname);
}

int QQuickGridView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItemView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickGridView::cellWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickGridView::cellHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickGridView::highlightMoveDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickGridView::flowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickGridView::snapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_QQuickGridViewAttached_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGridViewAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGridViewAttached_t qt_meta_stringdata_QQuickGridViewAttached = {
    {
QT_MOC_LITERAL(0, 0, 22) // "QQuickGridViewAttached"

    },
    "QQuickGridViewAttached"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGridViewAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickGridViewAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickGridViewAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItemViewAttached::staticMetaObject>(),
    qt_meta_stringdata_QQuickGridViewAttached.data,
    qt_meta_data_QQuickGridViewAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGridViewAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGridViewAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGridViewAttached.stringdata0))
        return static_cast<void*>(this);
    return QQuickItemViewAttached::qt_metacast(_clname);
}

int QQuickGridViewAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItemViewAttached::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
