/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qquicklayout_p.h>
#include <qquicklinearlayout_p.h>
#include <qquickstacklayout_p.h>

void qml_register_types_QtQuick_Layouts()
{
    qmlRegisterTypesAndRevisions<QQuickColumnLayout>("QtQuick.Layouts", 1);
    qmlRegisterTypesAndRevisions<QQuickGridLayout>("QtQuick.Layouts", 1);
    qmlRegisterTypesAndRevisions<QQuickGridLayoutBase>("QtQuick.Layouts", 1);
    qmlRegisterTypesAndRevisions<QQuickLayout>("QtQuick.Layouts", 1);
    qmlRegisterTypesAndRevisions<QQuickRowLayout>("QtQuick.Layouts", 1);
    qmlRegisterTypesAndRevisions<QQuickStackLayout>("QtQuick.Layouts", 1);
    qmlRegisterModule("QtQuick.Layouts", 1, 15);
}

static const QQmlModuleRegistration registration("QtQuick.Layouts", 1, qml_register_types_QtQuick_Layouts);
