/****************************************************************************
** Meta object code from reading C++ file 'qquickanimatedsprite_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickanimatedsprite_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickanimatedsprite_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAnimatedSprite_t {
    QByteArrayData data[69];
    char stringdata0[857];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAnimatedSprite_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAnimatedSprite_t qt_meta_stringdata_QQuickAnimatedSprite = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickAnimatedSprite"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "AnimatedSprite"
QT_MOC_LITERAL(3, 48, 13), // "pausedChanged"
QT_MOC_LITERAL(4, 62, 0), // ""
QT_MOC_LITERAL(5, 63, 3), // "arg"
QT_MOC_LITERAL(6, 67, 14), // "runningChanged"
QT_MOC_LITERAL(7, 82, 18), // "interpolateChanged"
QT_MOC_LITERAL(8, 101, 13), // "sourceChanged"
QT_MOC_LITERAL(9, 115, 14), // "reverseChanged"
QT_MOC_LITERAL(10, 130, 16), // "frameSyncChanged"
QT_MOC_LITERAL(11, 147, 17), // "frameCountChanged"
QT_MOC_LITERAL(12, 165, 18), // "frameHeightChanged"
QT_MOC_LITERAL(13, 184, 17), // "frameWidthChanged"
QT_MOC_LITERAL(14, 202, 13), // "frameXChanged"
QT_MOC_LITERAL(15, 216, 13), // "frameYChanged"
QT_MOC_LITERAL(16, 230, 16), // "frameRateChanged"
QT_MOC_LITERAL(17, 247, 20), // "frameDurationChanged"
QT_MOC_LITERAL(18, 268, 12), // "loopsChanged"
QT_MOC_LITERAL(19, 281, 19), // "currentFrameChanged"
QT_MOC_LITERAL(20, 301, 21), // "finishBehaviorChanged"
QT_MOC_LITERAL(21, 323, 14), // "FinishBehavior"
QT_MOC_LITERAL(22, 338, 8), // "finished"
QT_MOC_LITERAL(23, 347, 5), // "start"
QT_MOC_LITERAL(24, 353, 4), // "stop"
QT_MOC_LITERAL(25, 358, 7), // "restart"
QT_MOC_LITERAL(26, 366, 7), // "advance"
QT_MOC_LITERAL(27, 374, 6), // "frames"
QT_MOC_LITERAL(28, 381, 5), // "pause"
QT_MOC_LITERAL(29, 387, 6), // "resume"
QT_MOC_LITERAL(30, 394, 10), // "setRunning"
QT_MOC_LITERAL(31, 405, 9), // "setPaused"
QT_MOC_LITERAL(32, 415, 14), // "setInterpolate"
QT_MOC_LITERAL(33, 430, 9), // "setSource"
QT_MOC_LITERAL(34, 440, 10), // "setReverse"
QT_MOC_LITERAL(35, 451, 12), // "setFrameSync"
QT_MOC_LITERAL(36, 464, 13), // "setFrameCount"
QT_MOC_LITERAL(37, 478, 14), // "setFrameHeight"
QT_MOC_LITERAL(38, 493, 13), // "setFrameWidth"
QT_MOC_LITERAL(39, 507, 9), // "setFrameX"
QT_MOC_LITERAL(40, 517, 9), // "setFrameY"
QT_MOC_LITERAL(41, 527, 12), // "setFrameRate"
QT_MOC_LITERAL(42, 540, 16), // "setFrameDuration"
QT_MOC_LITERAL(43, 557, 14), // "resetFrameRate"
QT_MOC_LITERAL(44, 572, 18), // "resetFrameDuration"
QT_MOC_LITERAL(45, 591, 8), // "setLoops"
QT_MOC_LITERAL(46, 600, 15), // "setCurrentFrame"
QT_MOC_LITERAL(47, 616, 12), // "createEngine"
QT_MOC_LITERAL(48, 629, 5), // "reset"
QT_MOC_LITERAL(49, 635, 7), // "running"
QT_MOC_LITERAL(50, 643, 11), // "interpolate"
QT_MOC_LITERAL(51, 655, 6), // "source"
QT_MOC_LITERAL(52, 662, 7), // "reverse"
QT_MOC_LITERAL(53, 670, 9), // "frameSync"
QT_MOC_LITERAL(54, 680, 10), // "frameCount"
QT_MOC_LITERAL(55, 691, 11), // "frameHeight"
QT_MOC_LITERAL(56, 703, 10), // "frameWidth"
QT_MOC_LITERAL(57, 714, 6), // "frameX"
QT_MOC_LITERAL(58, 721, 6), // "frameY"
QT_MOC_LITERAL(59, 728, 9), // "frameRate"
QT_MOC_LITERAL(60, 738, 13), // "frameDuration"
QT_MOC_LITERAL(61, 752, 5), // "loops"
QT_MOC_LITERAL(62, 758, 6), // "paused"
QT_MOC_LITERAL(63, 765, 12), // "currentFrame"
QT_MOC_LITERAL(64, 778, 14), // "finishBehavior"
QT_MOC_LITERAL(65, 793, 14), // "LoopParameters"
QT_MOC_LITERAL(66, 808, 8), // "Infinite"
QT_MOC_LITERAL(67, 817, 20), // "FinishAtInitialFrame"
QT_MOC_LITERAL(68, 838, 18) // "FinishAtFinalFrame"

    },
    "QQuickAnimatedSprite\0QML.Element\0"
    "AnimatedSprite\0pausedChanged\0\0arg\0"
    "runningChanged\0interpolateChanged\0"
    "sourceChanged\0reverseChanged\0"
    "frameSyncChanged\0frameCountChanged\0"
    "frameHeightChanged\0frameWidthChanged\0"
    "frameXChanged\0frameYChanged\0"
    "frameRateChanged\0frameDurationChanged\0"
    "loopsChanged\0currentFrameChanged\0"
    "finishBehaviorChanged\0FinishBehavior\0"
    "finished\0start\0stop\0restart\0advance\0"
    "frames\0pause\0resume\0setRunning\0setPaused\0"
    "setInterpolate\0setSource\0setReverse\0"
    "setFrameSync\0setFrameCount\0setFrameHeight\0"
    "setFrameWidth\0setFrameX\0setFrameY\0"
    "setFrameRate\0setFrameDuration\0"
    "resetFrameRate\0resetFrameDuration\0"
    "setLoops\0setCurrentFrame\0createEngine\0"
    "reset\0running\0interpolate\0source\0"
    "reverse\0frameSync\0frameCount\0frameHeight\0"
    "frameWidth\0frameX\0frameY\0frameRate\0"
    "frameDuration\0loops\0paused\0currentFrame\0"
    "finishBehavior\0LoopParameters\0Infinite\0"
    "FinishAtInitialFrame\0FinishAtFinalFrame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAnimatedSprite[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      43,   16, // methods
      16,  381, // properties
       2,  461, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  274,    4, 0x06 /* Public */,
       6,    1,  277,    4, 0x06 /* Public */,
       7,    1,  280,    4, 0x06 /* Public */,
       8,    1,  283,    4, 0x06 /* Public */,
       9,    1,  286,    4, 0x06 /* Public */,
      10,    1,  289,    4, 0x06 /* Public */,
      11,    1,  292,    4, 0x06 /* Public */,
      12,    1,  295,    4, 0x06 /* Public */,
      13,    1,  298,    4, 0x06 /* Public */,
      14,    1,  301,    4, 0x06 /* Public */,
      15,    1,  304,    4, 0x06 /* Public */,
      16,    1,  307,    4, 0x06 /* Public */,
      17,    1,  310,    4, 0x06 /* Public */,
      18,    1,  313,    4, 0x06 /* Public */,
      19,    1,  316,    4, 0x06 /* Public */,
      20,    1,  319,    4, 0x86 /* Public | MethodRevisioned */,
      22,    0,  322,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      23,    0,  323,    4, 0x0a /* Public */,
      24,    0,  324,    4, 0x0a /* Public */,
      25,    0,  325,    4, 0x0a /* Public */,
      26,    1,  326,    4, 0x0a /* Public */,
      26,    0,  329,    4, 0x2a /* Public | MethodCloned */,
      28,    0,  330,    4, 0x0a /* Public */,
      29,    0,  331,    4, 0x0a /* Public */,
      30,    1,  332,    4, 0x0a /* Public */,
      31,    1,  335,    4, 0x0a /* Public */,
      32,    1,  338,    4, 0x0a /* Public */,
      33,    1,  341,    4, 0x0a /* Public */,
      34,    1,  344,    4, 0x0a /* Public */,
      35,    1,  347,    4, 0x0a /* Public */,
      36,    1,  350,    4, 0x0a /* Public */,
      37,    1,  353,    4, 0x0a /* Public */,
      38,    1,  356,    4, 0x0a /* Public */,
      39,    1,  359,    4, 0x0a /* Public */,
      40,    1,  362,    4, 0x0a /* Public */,
      41,    1,  365,    4, 0x0a /* Public */,
      42,    1,  368,    4, 0x0a /* Public */,
      43,    0,  371,    4, 0x0a /* Public */,
      44,    0,  372,    4, 0x0a /* Public */,
      45,    1,  373,    4, 0x0a /* Public */,
      46,    1,  376,    4, 0x0a /* Public */,
      47,    0,  379,    4, 0x08 /* Private */,
      48,    0,  380,    4, 0x09 /* Protected */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      15,
      12,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, 0x80000000 | 21,    5,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      49, QMetaType::Bool, 0x00495103,
      50, QMetaType::Bool, 0x00495103,
      51, QMetaType::QUrl, 0x00495103,
      52, QMetaType::Bool, 0x00495103,
      53, QMetaType::Bool, 0x00495103,
      54, QMetaType::Int, 0x00495103,
      55, QMetaType::Int, 0x00495103,
      56, QMetaType::Int, 0x00495103,
      57, QMetaType::Int, 0x00495103,
      58, QMetaType::Int, 0x00495103,
      59, QMetaType::QReal, 0x00495107,
      60, QMetaType::Int, 0x00495107,
      61, QMetaType::Int, 0x00495103,
      62, QMetaType::Bool, 0x00495103,
      63, QMetaType::Int, 0x00495103,
      64, 0x80000000 | 21, 0x00c9510b,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
       0,
      14,
      15,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      15,

 // enums: name, alias, flags, count, data
      65,   65, 0x0,    1,  471,
      21,   21, 0x0,    2,  473,

 // enum data: key, value
      66, uint(QQuickAnimatedSprite::Infinite),
      67, uint(QQuickAnimatedSprite::FinishAtInitialFrame),
      68, uint(QQuickAnimatedSprite::FinishAtFinalFrame),

       0        // eod
};

void QQuickAnimatedSprite::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAnimatedSprite *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pausedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->runningChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->interpolateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->sourceChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->reverseChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->frameSyncChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->frameCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->frameHeightChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->frameWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->frameXChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->frameYChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->frameRateChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 12: _t->frameDurationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->loopsChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->currentFrameChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->finishBehaviorChanged((*reinterpret_cast< FinishBehavior(*)>(_a[1]))); break;
        case 16: _t->finished(); break;
        case 17: _t->start(); break;
        case 18: _t->stop(); break;
        case 19: _t->restart(); break;
        case 20: _t->advance((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->advance(); break;
        case 22: _t->pause(); break;
        case 23: _t->resume(); break;
        case 24: _t->setRunning((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->setPaused((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->setInterpolate((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->setSource((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 28: _t->setReverse((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 29: _t->setFrameSync((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->setFrameCount((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->setFrameHeight((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 32: _t->setFrameWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->setFrameX((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->setFrameY((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 35: _t->setFrameRate((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 36: _t->setFrameDuration((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->resetFrameRate(); break;
        case 38: _t->resetFrameDuration(); break;
        case 39: _t->setLoops((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: _t->setCurrentFrame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 41: _t->createEngine(); break;
        case 42: _t->reset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAnimatedSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::pausedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::runningChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::interpolateChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::sourceChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::reverseChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameSyncChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameCountChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameHeightChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameWidthChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameXChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameYChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameRateChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::frameDurationChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::loopsChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::currentFrameChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)(FinishBehavior );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::finishBehaviorChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickAnimatedSprite::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimatedSprite::finished)) {
                *result = 16;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAnimatedSprite *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->running(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->interpolate(); break;
        case 2: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->reverse(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->frameSync(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->frameCount(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->frameHeight(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->frameWidth(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->frameX(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->frameY(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->frameRate(); break;
        case 11: *reinterpret_cast< int*>(_v) = _t->frameDuration(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->loops(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->paused(); break;
        case 14: *reinterpret_cast< int*>(_v) = _t->currentFrame(); break;
        case 15: *reinterpret_cast< FinishBehavior*>(_v) = _t->finishBehavior(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAnimatedSprite *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRunning(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setInterpolate(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 3: _t->setReverse(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setFrameSync(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setFrameCount(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setFrameHeight(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setFrameWidth(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setFrameX(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setFrameY(*reinterpret_cast< int*>(_v)); break;
        case 10: _t->setFrameRate(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setFrameDuration(*reinterpret_cast< int*>(_v)); break;
        case 12: _t->setLoops(*reinterpret_cast< int*>(_v)); break;
        case 13: _t->setPaused(*reinterpret_cast< bool*>(_v)); break;
        case 14: _t->setCurrentFrame(*reinterpret_cast< int*>(_v)); break;
        case 15: _t->setFinishBehavior(*reinterpret_cast< FinishBehavior*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickAnimatedSprite *_t = static_cast<QQuickAnimatedSprite *>(_o);
        (void)_t;
        switch (_id) {
        case 10: _t->resetFrameRate(); break;
        case 11: _t->resetFrameDuration(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickAnimatedSprite::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickAnimatedSprite.data,
    qt_meta_data_QQuickAnimatedSprite,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAnimatedSprite::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAnimatedSprite::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAnimatedSprite.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickAnimatedSprite::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 43;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 16;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAnimatedSprite::pausedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickAnimatedSprite::runningChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickAnimatedSprite::interpolateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickAnimatedSprite::sourceChanged(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickAnimatedSprite::reverseChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickAnimatedSprite::frameSyncChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickAnimatedSprite::frameCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickAnimatedSprite::frameHeightChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickAnimatedSprite::frameWidthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickAnimatedSprite::frameXChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickAnimatedSprite::frameYChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickAnimatedSprite::frameRateChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickAnimatedSprite::frameDurationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickAnimatedSprite::loopsChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickAnimatedSprite::currentFrameChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickAnimatedSprite::finishBehaviorChanged(FinishBehavior _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickAnimatedSprite::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
