/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 5.15.4
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

static const unsigned char qt_resource_data[] = {
  // importmodule
  0x0,0x0,0x0,0x5e,
  0x69,
  0x6d,0x70,0x6f,0x72,0x74,0x20,0x51,0x74,0x51,0x6d,0x6c,0x20,0x32,0x2e,0x30,0xa,
  0x69,0x6d,0x70,0x6f,0x72,0x74,0x20,0x22,0x6d,0x6f,0x64,0x75,0x6c,0x65,0x2e,0x6d,
  0x6a,0x73,0x22,0x20,0x61,0x73,0x20,0x4d,0x6f,0x64,0x75,0x6c,0x65,0xa,0x51,0x74,
  0x4f,0x62,0x6a,0x65,0x63,0x74,0x20,0x7b,0xa,0x20,0x20,0x20,0x20,0x70,0x72,0x6f,
  0x70,0x65,0x72,0x74,0x79,0x20,0x62,0x6f,0x6f,0x6c,0x20,0x6f,0x6b,0x3a,0x20,0x4d,
  0x6f,0x64,0x75,0x6c,0x65,0x2e,0x6f,0x6b,0x28,0x29,0xa,0x7d,0xa,
    // module
  0x0,0x0,0x0,0x27,
  0xa,
  0x65,0x78,0x70,0x6f,0x72,0x74,0x20,0x66,0x75,0x6e,0x63,0x74,0x69,0x6f,0x6e,0x20,
  0x6f,0x6b,0x28,0x29,0x20,0x7b,0x20,0x72,0x65,0x74,0x75,0x72,0x6e,0x20,0x74,0x72,
  0x75,0x65,0x3b,0x20,0x7d,0xa,
    // test
  0x0,0x0,0x0,0x39,
  0x69,
  0x6d,0x70,0x6f,0x72,0x74,0x20,0x51,0x74,0x51,0x6d,0x6c,0x20,0x32,0x2e,0x30,0xa,
  0x51,0x74,0x4f,0x62,0x6a,0x65,0x63,0x74,0x20,0x7b,0xa,0x20,0x20,0x20,0x20,0x70,
  0x72,0x6f,0x70,0x65,0x72,0x74,0x79,0x20,0x69,0x6e,0x74,0x20,0x76,0x61,0x6c,0x75,
  0x65,0x3a,0x20,0x32,0x30,0xa,0x7d,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // importmodule.qml
  0x0,0x10,
  0x9,0xe,0xce,0x9c,
  0x0,0x69,
  0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x72,0x0,0x74,0x0,0x6d,0x0,0x6f,0x0,0x64,0x0,0x75,0x0,0x6c,0x0,0x65,0x0,0x2e,0x0,0x71,0x0,0x6d,0x0,0x6c,
    // module.mjs
  0x0,0xa,
  0xc,0x2a,0xdb,0x73,
  0x0,0x6d,
  0x0,0x6f,0x0,0x64,0x0,0x75,0x0,0x6c,0x0,0x65,0x0,0x2e,0x0,0x6d,0x0,0x6a,0x0,0x73,
    // test.qml
  0x0,0x8,
  0xc,0xa7,0x5b,0x7c,
  0x0,0x74,
  0x0,0x65,0x0,0x73,0x0,0x74,0x0,0x2e,0x0,0x71,0x0,0x6d,0x0,0x6c,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x3,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/importmodule.qml
  0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/module.mjs
  0x0,0x0,0x0,0x26,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x62,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/test.qml
  0x0,0x0,0x0,0x40,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x8d,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#ifdef QT_NAMESPACE
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmake_immediate)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmake_immediate)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmake_immediate)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmake_immediate)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_qmake_immediate)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_qmake_immediate)(); }
   } dummy;
}
