/****************************************************************************
** Meta object code from reading C++ file 'qquickrepeater_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickrepeater_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickrepeater_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickRepeater_t {
    QByteArrayData data[24];
    char stringdata0[247];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRepeater_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRepeater_t qt_meta_stringdata_QQuickRepeater = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickRepeater"
QT_MOC_LITERAL(1, 15, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 31, 8), // "delegate"
QT_MOC_LITERAL(3, 40, 11), // "QML.Element"
QT_MOC_LITERAL(4, 52, 8), // "Repeater"
QT_MOC_LITERAL(5, 61, 12), // "modelChanged"
QT_MOC_LITERAL(6, 74, 0), // ""
QT_MOC_LITERAL(7, 75, 15), // "delegateChanged"
QT_MOC_LITERAL(8, 91, 12), // "countChanged"
QT_MOC_LITERAL(9, 104, 9), // "itemAdded"
QT_MOC_LITERAL(10, 114, 5), // "index"
QT_MOC_LITERAL(11, 120, 11), // "QQuickItem*"
QT_MOC_LITERAL(12, 132, 4), // "item"
QT_MOC_LITERAL(13, 137, 11), // "itemRemoved"
QT_MOC_LITERAL(14, 149, 11), // "createdItem"
QT_MOC_LITERAL(15, 161, 8), // "initItem"
QT_MOC_LITERAL(16, 170, 12), // "modelUpdated"
QT_MOC_LITERAL(17, 183, 13), // "QQmlChangeSet"
QT_MOC_LITERAL(18, 197, 9), // "changeSet"
QT_MOC_LITERAL(19, 207, 5), // "reset"
QT_MOC_LITERAL(20, 213, 6), // "itemAt"
QT_MOC_LITERAL(21, 220, 5), // "model"
QT_MOC_LITERAL(22, 226, 14), // "QQmlComponent*"
QT_MOC_LITERAL(23, 241, 5) // "count"

    },
    "QQuickRepeater\0DefaultProperty\0delegate\0"
    "QML.Element\0Repeater\0modelChanged\0\0"
    "delegateChanged\0countChanged\0itemAdded\0"
    "index\0QQuickItem*\0item\0itemRemoved\0"
    "createdItem\0initItem\0modelUpdated\0"
    "QQmlChangeSet\0changeSet\0reset\0itemAt\0"
    "model\0QQmlComponent*\0count"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRepeater[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
       3,   94, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   63,    6, 0x06 /* Public */,
       7,    0,   64,    6, 0x06 /* Public */,
       8,    0,   65,    6, 0x06 /* Public */,
       9,    2,   66,    6, 0x06 /* Public */,
      13,    2,   71,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    2,   76,    6, 0x08 /* Private */,
      15,    2,   81,    6, 0x08 /* Private */,
      16,    2,   86,    6, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      20,    1,   91,    6, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 11,   10,   12,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 11,   10,   12,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   10,   12,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,    6,   12,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Bool,   18,   19,

 // methods: parameters
    0x80000000 | 11, QMetaType::Int,   10,

 // properties: name, type, flags
      21, QMetaType::QVariant, 0x00495103,
       2, 0x80000000 | 22, 0x0049510b,
      23, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,

       0        // eod
};

void QQuickRepeater::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickRepeater *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged(); break;
        case 1: _t->delegateChanged(); break;
        case 2: _t->countChanged(); break;
        case 3: _t->itemAdded((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QQuickItem*(*)>(_a[2]))); break;
        case 4: _t->itemRemoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QQuickItem*(*)>(_a[2]))); break;
        case 5: _t->createdItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 6: _t->initItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 7: _t->modelUpdated((*reinterpret_cast< const QQmlChangeSet(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: { QQuickItem* _r = _t->itemAt((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickRepeater::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRepeater::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickRepeater::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRepeater::delegateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickRepeater::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRepeater::countChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickRepeater::*)(int , QQuickItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRepeater::itemAdded)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickRepeater::*)(int , QQuickItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRepeater::itemRemoved)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickRepeater *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->count(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickRepeater *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickRepeater::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickRepeater.data,
    qt_meta_data_QQuickRepeater,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickRepeater::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickRepeater::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickRepeater.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickRepeater::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickRepeater::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickRepeater::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickRepeater::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickRepeater::itemAdded(int _t1, QQuickItem * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickRepeater::itemRemoved(int _t1, QQuickItem * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
