/****************************************************************************
** Meta object code from reading C++ file 'qquickpathview_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickpathview_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpathview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPathView_t {
    QByteArrayData data[104];
    char stringdata0[1438];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPathView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPathView_t qt_meta_stringdata_QQuickPathView = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickPathView"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "PathView"
QT_MOC_LITERAL(3, 36, 12), // "QML.Attached"
QT_MOC_LITERAL(4, 49, 22), // "QQuickPathViewAttached"
QT_MOC_LITERAL(5, 72, 19), // "currentIndexChanged"
QT_MOC_LITERAL(6, 92, 0), // ""
QT_MOC_LITERAL(7, 93, 18), // "currentItemChanged"
QT_MOC_LITERAL(8, 112, 13), // "offsetChanged"
QT_MOC_LITERAL(9, 126, 12), // "modelChanged"
QT_MOC_LITERAL(10, 139, 12), // "countChanged"
QT_MOC_LITERAL(11, 152, 11), // "pathChanged"
QT_MOC_LITERAL(12, 164, 30), // "preferredHighlightBeginChanged"
QT_MOC_LITERAL(13, 195, 28), // "preferredHighlightEndChanged"
QT_MOC_LITERAL(14, 224, 25), // "highlightRangeModeChanged"
QT_MOC_LITERAL(15, 250, 17), // "dragMarginChanged"
QT_MOC_LITERAL(16, 268, 19), // "snapPositionChanged"
QT_MOC_LITERAL(17, 288, 15), // "delegateChanged"
QT_MOC_LITERAL(18, 304, 20), // "pathItemCountChanged"
QT_MOC_LITERAL(19, 325, 27), // "maximumFlickVelocityChanged"
QT_MOC_LITERAL(20, 353, 24), // "flickDecelerationChanged"
QT_MOC_LITERAL(21, 378, 18), // "interactiveChanged"
QT_MOC_LITERAL(22, 397, 13), // "movingChanged"
QT_MOC_LITERAL(23, 411, 15), // "flickingChanged"
QT_MOC_LITERAL(24, 427, 15), // "draggingChanged"
QT_MOC_LITERAL(25, 443, 16), // "highlightChanged"
QT_MOC_LITERAL(26, 460, 20), // "highlightItemChanged"
QT_MOC_LITERAL(27, 481, 28), // "highlightMoveDurationChanged"
QT_MOC_LITERAL(28, 510, 15), // "movementStarted"
QT_MOC_LITERAL(29, 526, 13), // "movementEnded"
QT_MOC_LITERAL(30, 540, 24), // "movementDirectionChanged"
QT_MOC_LITERAL(31, 565, 12), // "flickStarted"
QT_MOC_LITERAL(32, 578, 10), // "flickEnded"
QT_MOC_LITERAL(33, 589, 11), // "dragStarted"
QT_MOC_LITERAL(34, 601, 9), // "dragEnded"
QT_MOC_LITERAL(35, 611, 15), // "snapModeChanged"
QT_MOC_LITERAL(36, 627, 21), // "cacheItemCountChanged"
QT_MOC_LITERAL(37, 649, 21), // "incrementCurrentIndex"
QT_MOC_LITERAL(38, 671, 21), // "decrementCurrentIndex"
QT_MOC_LITERAL(39, 693, 6), // "refill"
QT_MOC_LITERAL(40, 700, 6), // "ticked"
QT_MOC_LITERAL(41, 707, 14), // "movementEnding"
QT_MOC_LITERAL(42, 722, 12), // "modelUpdated"
QT_MOC_LITERAL(43, 735, 13), // "QQmlChangeSet"
QT_MOC_LITERAL(44, 749, 9), // "changeSet"
QT_MOC_LITERAL(45, 759, 5), // "reset"
QT_MOC_LITERAL(46, 765, 11), // "createdItem"
QT_MOC_LITERAL(47, 777, 5), // "index"
QT_MOC_LITERAL(48, 783, 4), // "item"
QT_MOC_LITERAL(49, 788, 8), // "initItem"
QT_MOC_LITERAL(50, 797, 14), // "destroyingItem"
QT_MOC_LITERAL(51, 812, 11), // "pathUpdated"
QT_MOC_LITERAL(52, 824, 19), // "positionViewAtIndex"
QT_MOC_LITERAL(53, 844, 4), // "mode"
QT_MOC_LITERAL(54, 849, 7), // "indexAt"
QT_MOC_LITERAL(55, 857, 1), // "x"
QT_MOC_LITERAL(56, 859, 1), // "y"
QT_MOC_LITERAL(57, 861, 6), // "itemAt"
QT_MOC_LITERAL(58, 868, 11), // "QQuickItem*"
QT_MOC_LITERAL(59, 880, 11), // "itemAtIndex"
QT_MOC_LITERAL(60, 892, 5), // "model"
QT_MOC_LITERAL(61, 898, 4), // "path"
QT_MOC_LITERAL(62, 903, 11), // "QQuickPath*"
QT_MOC_LITERAL(63, 915, 12), // "currentIndex"
QT_MOC_LITERAL(64, 928, 11), // "currentItem"
QT_MOC_LITERAL(65, 940, 6), // "offset"
QT_MOC_LITERAL(66, 947, 9), // "highlight"
QT_MOC_LITERAL(67, 957, 14), // "QQmlComponent*"
QT_MOC_LITERAL(68, 972, 13), // "highlightItem"
QT_MOC_LITERAL(69, 986, 23), // "preferredHighlightBegin"
QT_MOC_LITERAL(70, 1010, 21), // "preferredHighlightEnd"
QT_MOC_LITERAL(71, 1032, 18), // "highlightRangeMode"
QT_MOC_LITERAL(72, 1051, 18), // "HighlightRangeMode"
QT_MOC_LITERAL(73, 1070, 21), // "highlightMoveDuration"
QT_MOC_LITERAL(74, 1092, 10), // "dragMargin"
QT_MOC_LITERAL(75, 1103, 20), // "maximumFlickVelocity"
QT_MOC_LITERAL(76, 1124, 17), // "flickDeceleration"
QT_MOC_LITERAL(77, 1142, 11), // "interactive"
QT_MOC_LITERAL(78, 1154, 6), // "moving"
QT_MOC_LITERAL(79, 1161, 8), // "flicking"
QT_MOC_LITERAL(80, 1170, 8), // "dragging"
QT_MOC_LITERAL(81, 1179, 5), // "count"
QT_MOC_LITERAL(82, 1185, 8), // "delegate"
QT_MOC_LITERAL(83, 1194, 13), // "pathItemCount"
QT_MOC_LITERAL(84, 1208, 8), // "snapMode"
QT_MOC_LITERAL(85, 1217, 8), // "SnapMode"
QT_MOC_LITERAL(86, 1226, 17), // "movementDirection"
QT_MOC_LITERAL(87, 1244, 17), // "MovementDirection"
QT_MOC_LITERAL(88, 1262, 14), // "cacheItemCount"
QT_MOC_LITERAL(89, 1277, 16), // "NoHighlightRange"
QT_MOC_LITERAL(90, 1294, 10), // "ApplyRange"
QT_MOC_LITERAL(91, 1305, 20), // "StrictlyEnforceRange"
QT_MOC_LITERAL(92, 1326, 6), // "NoSnap"
QT_MOC_LITERAL(93, 1333, 10), // "SnapToItem"
QT_MOC_LITERAL(94, 1344, 11), // "SnapOneItem"
QT_MOC_LITERAL(95, 1356, 8), // "Shortest"
QT_MOC_LITERAL(96, 1365, 8), // "Negative"
QT_MOC_LITERAL(97, 1374, 8), // "Positive"
QT_MOC_LITERAL(98, 1383, 12), // "PositionMode"
QT_MOC_LITERAL(99, 1396, 9), // "Beginning"
QT_MOC_LITERAL(100, 1406, 6), // "Center"
QT_MOC_LITERAL(101, 1413, 3), // "End"
QT_MOC_LITERAL(102, 1417, 7), // "Contain"
QT_MOC_LITERAL(103, 1425, 12) // "SnapPosition"

    },
    "QQuickPathView\0QML.Element\0PathView\0"
    "QML.Attached\0QQuickPathViewAttached\0"
    "currentIndexChanged\0\0currentItemChanged\0"
    "offsetChanged\0modelChanged\0countChanged\0"
    "pathChanged\0preferredHighlightBeginChanged\0"
    "preferredHighlightEndChanged\0"
    "highlightRangeModeChanged\0dragMarginChanged\0"
    "snapPositionChanged\0delegateChanged\0"
    "pathItemCountChanged\0maximumFlickVelocityChanged\0"
    "flickDecelerationChanged\0interactiveChanged\0"
    "movingChanged\0flickingChanged\0"
    "draggingChanged\0highlightChanged\0"
    "highlightItemChanged\0highlightMoveDurationChanged\0"
    "movementStarted\0movementEnded\0"
    "movementDirectionChanged\0flickStarted\0"
    "flickEnded\0dragStarted\0dragEnded\0"
    "snapModeChanged\0cacheItemCountChanged\0"
    "incrementCurrentIndex\0decrementCurrentIndex\0"
    "refill\0ticked\0movementEnding\0modelUpdated\0"
    "QQmlChangeSet\0changeSet\0reset\0createdItem\0"
    "index\0item\0initItem\0destroyingItem\0"
    "pathUpdated\0positionViewAtIndex\0mode\0"
    "indexAt\0x\0y\0itemAt\0QQuickItem*\0"
    "itemAtIndex\0model\0path\0QQuickPath*\0"
    "currentIndex\0currentItem\0offset\0"
    "highlight\0QQmlComponent*\0highlightItem\0"
    "preferredHighlightBegin\0preferredHighlightEnd\0"
    "highlightRangeMode\0HighlightRangeMode\0"
    "highlightMoveDuration\0dragMargin\0"
    "maximumFlickVelocity\0flickDeceleration\0"
    "interactive\0moving\0flicking\0dragging\0"
    "count\0delegate\0pathItemCount\0snapMode\0"
    "SnapMode\0movementDirection\0MovementDirection\0"
    "cacheItemCount\0NoHighlightRange\0"
    "ApplyRange\0StrictlyEnforceRange\0NoSnap\0"
    "SnapToItem\0SnapOneItem\0Shortest\0"
    "Negative\0Positive\0PositionMode\0Beginning\0"
    "Center\0End\0Contain\0SnapPosition"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPathView[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      45,   18, // methods
      24,  361, // properties
       4,  481, // enums/sets
       0,    0, // constructors
       0,       // flags
      31,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,  288,    6, 0x06 /* Public */,
       7,    0,  289,    6, 0x06 /* Public */,
       8,    0,  290,    6, 0x06 /* Public */,
       9,    0,  291,    6, 0x06 /* Public */,
      10,    0,  292,    6, 0x06 /* Public */,
      11,    0,  293,    6, 0x06 /* Public */,
      12,    0,  294,    6, 0x06 /* Public */,
      13,    0,  295,    6, 0x06 /* Public */,
      14,    0,  296,    6, 0x06 /* Public */,
      15,    0,  297,    6, 0x06 /* Public */,
      16,    0,  298,    6, 0x06 /* Public */,
      17,    0,  299,    6, 0x06 /* Public */,
      18,    0,  300,    6, 0x06 /* Public */,
      19,    0,  301,    6, 0x06 /* Public */,
      20,    0,  302,    6, 0x06 /* Public */,
      21,    0,  303,    6, 0x06 /* Public */,
      22,    0,  304,    6, 0x06 /* Public */,
      23,    0,  305,    6, 0x06 /* Public */,
      24,    0,  306,    6, 0x06 /* Public */,
      25,    0,  307,    6, 0x06 /* Public */,
      26,    0,  308,    6, 0x06 /* Public */,
      27,    0,  309,    6, 0x06 /* Public */,
      28,    0,  310,    6, 0x06 /* Public */,
      29,    0,  311,    6, 0x06 /* Public */,
      30,    0,  312,    6, 0x86 /* Public | MethodRevisioned */,
      31,    0,  313,    6, 0x06 /* Public */,
      32,    0,  314,    6, 0x06 /* Public */,
      33,    0,  315,    6, 0x06 /* Public */,
      34,    0,  316,    6, 0x06 /* Public */,
      35,    0,  317,    6, 0x06 /* Public */,
      36,    0,  318,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      37,    0,  319,    6, 0x0a /* Public */,
      38,    0,  320,    6, 0x0a /* Public */,
      39,    0,  321,    6, 0x08 /* Private */,
      40,    0,  322,    6, 0x08 /* Private */,
      41,    0,  323,    6, 0x08 /* Private */,
      42,    2,  324,    6, 0x08 /* Private */,
      46,    2,  329,    6, 0x08 /* Private */,
      49,    2,  334,    6, 0x08 /* Private */,
      50,    1,  339,    6, 0x08 /* Private */,
      51,    0,  342,    6, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      52,    2,  343,    6, 0x02 /* Public */,
      54,    2,  348,    6, 0x02 /* Public */,
      57,    2,  353,    6, 0x02 /* Public */,
      59,    1,  358,    6, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       7,
       0,
       0,
       0,
       0,
       0,
       0,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // methods: revision
       0,
       0,
       0,
      13,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 43, QMetaType::Bool,   44,   45,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   47,   48,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   47,   48,
    QMetaType::Void, QMetaType::QObjectStar,   48,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   47,   53,
    QMetaType::Int, QMetaType::QReal, QMetaType::QReal,   55,   56,
    0x80000000 | 58, QMetaType::QReal, QMetaType::QReal,   55,   56,
    0x80000000 | 58, QMetaType::Int,   47,

 // properties: name, type, flags
      60, QMetaType::QVariant, 0x00495103,
      61, 0x80000000 | 62, 0x0049510b,
      63, QMetaType::Int, 0x00495103,
      64, 0x80000000 | 58, 0x00495009,
      65, QMetaType::QReal, 0x00495103,
      66, 0x80000000 | 67, 0x0049510b,
      68, 0x80000000 | 58, 0x00495009,
      69, QMetaType::QReal, 0x00495103,
      70, QMetaType::QReal, 0x00495103,
      71, 0x80000000 | 72, 0x0049510b,
      73, QMetaType::Int, 0x00495103,
      74, QMetaType::QReal, 0x00495103,
      75, QMetaType::QReal, 0x00495103,
      76, QMetaType::QReal, 0x00495103,
      77, QMetaType::Bool, 0x00495103,
      78, QMetaType::Bool, 0x00495001,
      79, QMetaType::Bool, 0x00495001,
      80, QMetaType::Bool, 0x00495001,
      81, QMetaType::Int, 0x00495001,
      82, 0x80000000 | 67, 0x0049510b,
      83, QMetaType::Int, 0x00495107,
      84, 0x80000000 | 85, 0x0049510b,
      86, 0x80000000 | 87, 0x00c9510b,
      88, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       3,
       5,
       0,
       1,
       2,
      19,
      20,
       6,
       7,
       8,
      21,
       9,
      13,
      14,
      15,
      16,
      17,
      18,
       4,
      11,
      12,
      29,
      24,
      30,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       7,
       0,

 // enums: name, alias, flags, count, data
      72,   72, 0x0,    3,  501,
      85,   85, 0x0,    3,  507,
      87,   87, 0x0,    3,  513,
      98,   98, 0x0,    5,  519,

 // enum data: key, value
      89, uint(QQuickPathView::NoHighlightRange),
      90, uint(QQuickPathView::ApplyRange),
      91, uint(QQuickPathView::StrictlyEnforceRange),
      92, uint(QQuickPathView::NoSnap),
      93, uint(QQuickPathView::SnapToItem),
      94, uint(QQuickPathView::SnapOneItem),
      95, uint(QQuickPathView::Shortest),
      96, uint(QQuickPathView::Negative),
      97, uint(QQuickPathView::Positive),
      99, uint(QQuickPathView::Beginning),
     100, uint(QQuickPathView::Center),
     101, uint(QQuickPathView::End),
     102, uint(QQuickPathView::Contain),
     103, uint(QQuickPathView::SnapPosition),

       0        // eod
};

void QQuickPathView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPathView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentIndexChanged(); break;
        case 1: _t->currentItemChanged(); break;
        case 2: _t->offsetChanged(); break;
        case 3: _t->modelChanged(); break;
        case 4: _t->countChanged(); break;
        case 5: _t->pathChanged(); break;
        case 6: _t->preferredHighlightBeginChanged(); break;
        case 7: _t->preferredHighlightEndChanged(); break;
        case 8: _t->highlightRangeModeChanged(); break;
        case 9: _t->dragMarginChanged(); break;
        case 10: _t->snapPositionChanged(); break;
        case 11: _t->delegateChanged(); break;
        case 12: _t->pathItemCountChanged(); break;
        case 13: _t->maximumFlickVelocityChanged(); break;
        case 14: _t->flickDecelerationChanged(); break;
        case 15: _t->interactiveChanged(); break;
        case 16: _t->movingChanged(); break;
        case 17: _t->flickingChanged(); break;
        case 18: _t->draggingChanged(); break;
        case 19: _t->highlightChanged(); break;
        case 20: _t->highlightItemChanged(); break;
        case 21: _t->highlightMoveDurationChanged(); break;
        case 22: _t->movementStarted(); break;
        case 23: _t->movementEnded(); break;
        case 24: _t->movementDirectionChanged(); break;
        case 25: _t->flickStarted(); break;
        case 26: _t->flickEnded(); break;
        case 27: _t->dragStarted(); break;
        case 28: _t->dragEnded(); break;
        case 29: _t->snapModeChanged(); break;
        case 30: _t->cacheItemCountChanged(); break;
        case 31: _t->incrementCurrentIndex(); break;
        case 32: _t->decrementCurrentIndex(); break;
        case 33: _t->refill(); break;
        case 34: _t->ticked(); break;
        case 35: _t->movementEnding(); break;
        case 36: _t->modelUpdated((*reinterpret_cast< const QQmlChangeSet(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 37: _t->createdItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 38: _t->initItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 39: _t->destroyingItem((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 40: _t->pathUpdated(); break;
        case 41: _t->positionViewAtIndex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 42: { int _r = _t->indexAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 43: { QQuickItem* _r = _t->itemAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 44: { QQuickItem* _r = _t->itemAtIndex((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::currentIndexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::currentItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::offsetChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::modelChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::countChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::pathChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::preferredHighlightBeginChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::preferredHighlightEndChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::highlightRangeModeChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::dragMarginChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::snapPositionChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::delegateChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::pathItemCountChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::maximumFlickVelocityChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::flickDecelerationChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::interactiveChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::movingChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::flickingChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::draggingChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::highlightChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::highlightItemChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::highlightMoveDurationChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::movementStarted)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::movementEnded)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::movementDirectionChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::flickStarted)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::flickEnded)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::dragStarted)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::dragEnded)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::snapModeChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickPathView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathView::cacheItemCountChanged)) {
                *result = 30;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 19:
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 6:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPath* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPathView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQuickPath**>(_v) = _t->path(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->currentIndex(); break;
        case 3: *reinterpret_cast< QQuickItem**>(_v) = _t->currentItem(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->offset(); break;
        case 5: *reinterpret_cast< QQmlComponent**>(_v) = _t->highlight(); break;
        case 6: *reinterpret_cast< QQuickItem**>(_v) = _t->highlightItem(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->preferredHighlightBegin(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->preferredHighlightEnd(); break;
        case 9: *reinterpret_cast< HighlightRangeMode*>(_v) = _t->highlightRangeMode(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->highlightMoveDuration(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->dragMargin(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->maximumFlickVelocity(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->flickDeceleration(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->isInteractive(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->isMoving(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isFlicking(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->isDragging(); break;
        case 18: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 19: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 20: *reinterpret_cast< int*>(_v) = _t->pathItemCount(); break;
        case 21: *reinterpret_cast< SnapMode*>(_v) = _t->snapMode(); break;
        case 22: *reinterpret_cast< MovementDirection*>(_v) = _t->movementDirection(); break;
        case 23: *reinterpret_cast< int*>(_v) = _t->cacheItemCount(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPathView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setPath(*reinterpret_cast< QQuickPath**>(_v)); break;
        case 2: _t->setCurrentIndex(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setOffset(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setHighlight(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 7: _t->setPreferredHighlightBegin(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setPreferredHighlightEnd(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setHighlightRangeMode(*reinterpret_cast< HighlightRangeMode*>(_v)); break;
        case 10: _t->setHighlightMoveDuration(*reinterpret_cast< int*>(_v)); break;
        case 11: _t->setDragMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 12: _t->setMaximumFlickVelocity(*reinterpret_cast< qreal*>(_v)); break;
        case 13: _t->setFlickDeceleration(*reinterpret_cast< qreal*>(_v)); break;
        case 14: _t->setInteractive(*reinterpret_cast< bool*>(_v)); break;
        case 19: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 20: _t->setPathItemCount(*reinterpret_cast< int*>(_v)); break;
        case 21: _t->setSnapMode(*reinterpret_cast< SnapMode*>(_v)); break;
        case 22: _t->setMovementDirection(*reinterpret_cast< MovementDirection*>(_v)); break;
        case 23: _t->setCacheItemCount(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickPathView *_t = static_cast<QQuickPathView *>(_o);
        (void)_t;
        switch (_id) {
        case 20: _t->resetPathItemCount(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPathView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickPathView.data,
    qt_meta_data_QQuickPathView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPathView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPathView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPathView.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickPathView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 45;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 24;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPathView::currentIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPathView::currentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPathView::offsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPathView::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickPathView::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickPathView::pathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickPathView::preferredHighlightBeginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickPathView::preferredHighlightEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickPathView::highlightRangeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickPathView::dragMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickPathView::snapPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickPathView::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickPathView::pathItemCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickPathView::maximumFlickVelocityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickPathView::flickDecelerationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickPathView::interactiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickPathView::movingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickPathView::flickingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QQuickPathView::draggingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QQuickPathView::highlightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickPathView::highlightItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQuickPathView::highlightMoveDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickPathView::movementStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickPathView::movementEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQuickPathView::movementDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQuickPathView::flickStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQuickPathView::flickEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QQuickPathView::dragStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQuickPathView::dragEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void QQuickPathView::snapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void QQuickPathView::cacheItemCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}
struct qt_meta_stringdata_QQuickPathViewAttached_t {
    QByteArrayData data[8];
    char stringdata0[97];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPathViewAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPathViewAttached_t qt_meta_stringdata_QQuickPathViewAttached = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickPathViewAttached"
QT_MOC_LITERAL(1, 23, 18), // "currentItemChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 11), // "pathChanged"
QT_MOC_LITERAL(4, 55, 4), // "view"
QT_MOC_LITERAL(5, 60, 15), // "QQuickPathView*"
QT_MOC_LITERAL(6, 76, 13), // "isCurrentItem"
QT_MOC_LITERAL(7, 90, 6) // "onPath"

    },
    "QQuickPathViewAttached\0currentItemChanged\0"
    "\0pathChanged\0view\0QQuickPathView*\0"
    "isCurrentItem\0onPath"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPathViewAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       3,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    0,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       4, 0x80000000 | 5, 0x00095409,
       6, QMetaType::Bool, 0x00495001,
       7, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,

       0        // eod
};

void QQuickPathViewAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPathViewAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentItemChanged(); break;
        case 1: _t->pathChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPathViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathViewAttached::currentItemChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPathViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPathViewAttached::pathChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPathView* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPathViewAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickPathView**>(_v) = _t->view(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isCurrentItem(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isOnPath(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPathViewAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPathViewAttached.data,
    qt_meta_data_QQuickPathViewAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPathViewAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPathViewAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPathViewAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPathViewAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPathViewAttached::currentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPathViewAttached::pathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
