/****************************************************************************
** Meta object code from reading C++ file 'qqmllistmodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmlmodels/qqmllistmodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmllistmodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlListModel_t {
    QByteArrayData data[27];
    char stringdata0[217];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlListModel_t qt_meta_stringdata_QQmlListModel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QQmlListModel"
QT_MOC_LITERAL(1, 14, 11), // "QML.Element"
QT_MOC_LITERAL(2, 26, 9), // "ListModel"
QT_MOC_LITERAL(3, 36, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 60, 1), // "1"
QT_MOC_LITERAL(5, 62, 12), // "countChanged"
QT_MOC_LITERAL(6, 75, 0), // ""
QT_MOC_LITERAL(7, 76, 5), // "clear"
QT_MOC_LITERAL(8, 82, 6), // "remove"
QT_MOC_LITERAL(9, 89, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(10, 105, 4), // "args"
QT_MOC_LITERAL(11, 110, 6), // "append"
QT_MOC_LITERAL(12, 117, 6), // "insert"
QT_MOC_LITERAL(13, 124, 3), // "get"
QT_MOC_LITERAL(14, 128, 8), // "QJSValue"
QT_MOC_LITERAL(15, 137, 5), // "index"
QT_MOC_LITERAL(16, 143, 3), // "set"
QT_MOC_LITERAL(17, 147, 5), // "value"
QT_MOC_LITERAL(18, 153, 11), // "setProperty"
QT_MOC_LITERAL(19, 165, 8), // "property"
QT_MOC_LITERAL(20, 174, 4), // "move"
QT_MOC_LITERAL(21, 179, 4), // "from"
QT_MOC_LITERAL(22, 184, 2), // "to"
QT_MOC_LITERAL(23, 187, 5), // "count"
QT_MOC_LITERAL(24, 193, 4), // "sync"
QT_MOC_LITERAL(25, 198, 12), // "dynamicRoles"
QT_MOC_LITERAL(26, 211, 5) // "agent"

    },
    "QQmlListModel\0QML.Element\0ListModel\0"
    "QML.AddedInMinorVersion\0""1\0countChanged\0"
    "\0clear\0remove\0QQmlV4Function*\0args\0"
    "append\0insert\0get\0QJSValue\0index\0set\0"
    "value\0setProperty\0property\0move\0from\0"
    "to\0count\0sync\0dynamicRoles\0agent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      10,   18, // methods
       3,  102, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   68,    6, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       7,    0,   69,    6, 0x02 /* Public */,
       8,    1,   70,    6, 0x02 /* Public */,
      11,    1,   73,    6, 0x02 /* Public */,
      12,    1,   76,    6, 0x02 /* Public */,
      13,    1,   79,    6, 0x02 /* Public */,
      16,    2,   82,    6, 0x02 /* Public */,
      18,    3,   87,    6, 0x02 /* Public */,
      20,    3,   94,    6, 0x02 /* Public */,
      24,    0,  101,    6, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    0x80000000 | 14, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 14,   15,   17,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::QVariant,   15,   19,   17,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   21,   22,   23,
    QMetaType::Void,

 // properties: name, type, flags
      23, QMetaType::Int, 0x00495001,
      25, QMetaType::Bool, 0x00095103,
      26, QMetaType::QObjectStar, 0x00895401,

 // properties: notify_signal_id
       0,
       0,
       0,

 // properties: revision
       0,
       0,
      14,

       0        // eod
};

void QQmlListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->clear(); break;
        case 2: _t->remove((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 3: _t->append((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 4: _t->insert((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 5: { QJSValue _r = _t->get((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->set((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QJSValue(*)>(_a[2]))); break;
        case 7: _t->setProperty((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 8: _t->move((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->sync(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlListModel::countChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->dynamicRoles(); break;
        case 2: *reinterpret_cast< QObject**>(_v) = _t->agent(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDynamicRoles(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QQmlListModel.data,
    qt_meta_data_QQmlListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlListModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QQmlListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlListModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQmlListElement_t {
    QByteArrayData data[5];
    char stringdata0[66];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlListElement_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlListElement_t qt_meta_stringdata_QQmlListElement = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQmlListElement"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 11), // "ListElement"
QT_MOC_LITERAL(3, 40, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 64, 1) // "1"

    },
    "QQmlListElement\0QML.Element\0ListElement\0"
    "QML.AddedInMinorVersion\0""1"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlListElement[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

void QQmlListElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlListElement::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlListElement.data,
    qt_meta_data_QQmlListElement,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlListElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlListElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlListElement.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlListElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
