/****************************************************************************
** Meta object code from reading C++ file 'qquickscreen_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickscreen_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickscreen_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickScreenInfo_t {
    QByteArrayData data[32];
    char stringdata0[497];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreenInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreenInfo_t qt_meta_stringdata_QQuickScreenInfo = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickScreenInfo"
QT_MOC_LITERAL(1, 17, 11), // "nameChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 19), // "manufacturerChanged"
QT_MOC_LITERAL(4, 50, 12), // "modelChanged"
QT_MOC_LITERAL(5, 63, 19), // "serialNumberChanged"
QT_MOC_LITERAL(6, 83, 12), // "widthChanged"
QT_MOC_LITERAL(7, 96, 13), // "heightChanged"
QT_MOC_LITERAL(8, 110, 22), // "desktopGeometryChanged"
QT_MOC_LITERAL(9, 133, 26), // "logicalPixelDensityChanged"
QT_MOC_LITERAL(10, 160, 19), // "pixelDensityChanged"
QT_MOC_LITERAL(11, 180, 23), // "devicePixelRatioChanged"
QT_MOC_LITERAL(12, 204, 25), // "primaryOrientationChanged"
QT_MOC_LITERAL(13, 230, 18), // "orientationChanged"
QT_MOC_LITERAL(14, 249, 15), // "virtualXChanged"
QT_MOC_LITERAL(15, 265, 15), // "virtualYChanged"
QT_MOC_LITERAL(16, 281, 4), // "name"
QT_MOC_LITERAL(17, 286, 12), // "manufacturer"
QT_MOC_LITERAL(18, 299, 5), // "model"
QT_MOC_LITERAL(19, 305, 12), // "serialNumber"
QT_MOC_LITERAL(20, 318, 5), // "width"
QT_MOC_LITERAL(21, 324, 6), // "height"
QT_MOC_LITERAL(22, 331, 21), // "desktopAvailableWidth"
QT_MOC_LITERAL(23, 353, 22), // "desktopAvailableHeight"
QT_MOC_LITERAL(24, 376, 19), // "logicalPixelDensity"
QT_MOC_LITERAL(25, 396, 12), // "pixelDensity"
QT_MOC_LITERAL(26, 409, 16), // "devicePixelRatio"
QT_MOC_LITERAL(27, 426, 18), // "primaryOrientation"
QT_MOC_LITERAL(28, 445, 21), // "Qt::ScreenOrientation"
QT_MOC_LITERAL(29, 467, 11), // "orientation"
QT_MOC_LITERAL(30, 479, 8), // "virtualX"
QT_MOC_LITERAL(31, 488, 8) // "virtualY"

    },
    "QQuickScreenInfo\0nameChanged\0\0"
    "manufacturerChanged\0modelChanged\0"
    "serialNumberChanged\0widthChanged\0"
    "heightChanged\0desktopGeometryChanged\0"
    "logicalPixelDensityChanged\0"
    "pixelDensityChanged\0devicePixelRatioChanged\0"
    "primaryOrientationChanged\0orientationChanged\0"
    "virtualXChanged\0virtualYChanged\0name\0"
    "manufacturer\0model\0serialNumber\0width\0"
    "height\0desktopAvailableWidth\0"
    "desktopAvailableHeight\0logicalPixelDensity\0"
    "pixelDensity\0devicePixelRatio\0"
    "primaryOrientation\0Qt::ScreenOrientation\0"
    "orientation\0virtualX\0virtualY"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreenInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
      15,  112, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   98,    2, 0x06 /* Public */,
       3,    0,   99,    2, 0x86 /* Public | MethodRevisioned */,
       4,    0,  100,    2, 0x86 /* Public | MethodRevisioned */,
       5,    0,  101,    2, 0x86 /* Public | MethodRevisioned */,
       6,    0,  102,    2, 0x06 /* Public */,
       7,    0,  103,    2, 0x06 /* Public */,
       8,    0,  104,    2, 0x06 /* Public */,
       9,    0,  105,    2, 0x06 /* Public */,
      10,    0,  106,    2, 0x06 /* Public */,
      11,    0,  107,    2, 0x06 /* Public */,
      12,    0,  108,    2, 0x06 /* Public */,
      13,    0,  109,    2, 0x06 /* Public */,
      14,    0,  110,    2, 0x86 /* Public | MethodRevisioned */,
      15,    0,  111,    2, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
      10,
      10,
      10,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       3,
       3,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      16, QMetaType::QString, 0x00495001,
      17, QMetaType::QString, 0x00c95001,
      18, QMetaType::QString, 0x00c95001,
      19, QMetaType::QString, 0x00c95001,
      20, QMetaType::Int, 0x00495001,
      21, QMetaType::Int, 0x00495001,
      22, QMetaType::Int, 0x00495001,
      23, QMetaType::Int, 0x00495001,
      24, QMetaType::QReal, 0x00495001,
      25, QMetaType::QReal, 0x00495001,
      26, QMetaType::QReal, 0x00495001,
      27, 0x80000000 | 28, 0x00495009,
      29, 0x80000000 | 28, 0x00495009,
      30, QMetaType::Int, 0x00c95001,
      31, QMetaType::Int, 0x00c95001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,

 // properties: revision
       0,
      10,
      10,
      10,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       3,
       3,

       0        // eod
};

void QQuickScreenInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickScreenInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->nameChanged(); break;
        case 1: _t->manufacturerChanged(); break;
        case 2: _t->modelChanged(); break;
        case 3: _t->serialNumberChanged(); break;
        case 4: _t->widthChanged(); break;
        case 5: _t->heightChanged(); break;
        case 6: _t->desktopGeometryChanged(); break;
        case 7: _t->logicalPixelDensityChanged(); break;
        case 8: _t->pixelDensityChanged(); break;
        case 9: _t->devicePixelRatioChanged(); break;
        case 10: _t->primaryOrientationChanged(); break;
        case 11: _t->orientationChanged(); break;
        case 12: _t->virtualXChanged(); break;
        case 13: _t->virtualYChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::nameChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::manufacturerChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::modelChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::serialNumberChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::widthChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::heightChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::desktopGeometryChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::logicalPixelDensityChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::pixelDensityChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::devicePixelRatioChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::primaryOrientationChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::orientationChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::virtualXChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickScreenInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenInfo::virtualYChanged)) {
                *result = 13;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickScreenInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->manufacturer(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->model(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->serialNumber(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->width(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->height(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->desktopAvailableWidth(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->desktopAvailableHeight(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->logicalPixelDensity(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->pixelDensity(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->devicePixelRatio(); break;
        case 11: *reinterpret_cast< Qt::ScreenOrientation*>(_v) = _t->primaryOrientation(); break;
        case 12: *reinterpret_cast< Qt::ScreenOrientation*>(_v) = _t->orientation(); break;
        case 13: *reinterpret_cast< int*>(_v) = _t->virtualX(); break;
        case 14: *reinterpret_cast< int*>(_v) = _t->virtualY(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickScreenInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickScreenInfo.data,
    qt_meta_data_QQuickScreenInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickScreenInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickScreenInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickScreenInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickScreenInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickScreenInfo::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickScreenInfo::manufacturerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickScreenInfo::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickScreenInfo::serialNumberChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickScreenInfo::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickScreenInfo::heightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickScreenInfo::desktopGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickScreenInfo::logicalPixelDensityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickScreenInfo::pixelDensityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickScreenInfo::devicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickScreenInfo::primaryOrientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickScreenInfo::orientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickScreenInfo::virtualXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickScreenInfo::virtualYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
struct qt_meta_stringdata_QQuickScreenAttached_t {
    QByteArrayData data[10];
    char stringdata0[136];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreenAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreenAttached_t qt_meta_stringdata_QQuickScreenAttached = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickScreenAttached"
QT_MOC_LITERAL(1, 21, 28), // "orientationUpdateMaskChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 13), // "screenChanged"
QT_MOC_LITERAL(4, 65, 8), // "QScreen*"
QT_MOC_LITERAL(5, 74, 12), // "angleBetween"
QT_MOC_LITERAL(6, 87, 1), // "a"
QT_MOC_LITERAL(7, 89, 1), // "b"
QT_MOC_LITERAL(8, 91, 21), // "orientationUpdateMask"
QT_MOC_LITERAL(9, 113, 22) // "Qt::ScreenOrientations"

    },
    "QQuickScreenAttached\0orientationUpdateMaskChanged\0"
    "\0screenChanged\0QScreen*\0angleBetween\0"
    "a\0b\0orientationUpdateMask\0"
    "Qt::ScreenOrientations"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreenAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       1,   38, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   30,    2, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
       5,    2,   33,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    2,

 // methods: parameters
    QMetaType::Int, QMetaType::Int, QMetaType::Int,    6,    7,

 // properties: name, type, flags
       8, 0x80000000 | 9, 0x0049510b,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickScreenAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickScreenAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->orientationUpdateMaskChanged(); break;
        case 1: _t->screenChanged((*reinterpret_cast< QScreen*(*)>(_a[1]))); break;
        case 2: { int _r = _t->angleBetween((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickScreenAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickScreenAttached::orientationUpdateMaskChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickScreenAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::ScreenOrientations*>(_v) = _t->orientationUpdateMask(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickScreenAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOrientationUpdateMask(*reinterpret_cast< Qt::ScreenOrientations*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickScreenAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickScreenInfo::staticMetaObject>(),
    qt_meta_stringdata_QQuickScreenAttached.data,
    qt_meta_data_QQuickScreenAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickScreenAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickScreenAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickScreenAttached.stringdata0))
        return static_cast<void*>(this);
    return QQuickScreenInfo::qt_metacast(_clname);
}

int QQuickScreenAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickScreenInfo::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickScreenAttached::orientationUpdateMaskChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQuickScreen_t {
    QByteArrayData data[3];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreen_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreen_t qt_meta_stringdata_QQuickScreen = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QQuickScreen"
QT_MOC_LITERAL(1, 13, 12), // "QML.Attached"
QT_MOC_LITERAL(2, 26, 20) // "QQuickScreenAttached"

    },
    "QQuickScreen\0QML.Attached\0"
    "QQuickScreenAttached"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreen[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickScreen::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickScreen::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickScreen.data,
    qt_meta_data_QQuickScreen,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickScreen::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickScreen::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickScreen.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickScreen::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
