/****************************************************************************
** Meta object code from reading C++ file 'qqmllistmodelworkeragent_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmlmodels/qqmllistmodelworkeragent_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmllistmodelworkeragent_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlListModelWorkerAgent_t {
    QByteArrayData data[27];
    char stringdata0[228];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlListModelWorkerAgent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlListModelWorkerAgent_t qt_meta_stringdata_QQmlListModelWorkerAgent = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQmlListModelWorkerAgent"
QT_MOC_LITERAL(1, 25, 11), // "QML.Element"
QT_MOC_LITERAL(2, 37, 9), // "anonymous"
QT_MOC_LITERAL(3, 47, 13), // "engineChanged"
QT_MOC_LITERAL(4, 61, 0), // ""
QT_MOC_LITERAL(5, 62, 21), // "QV4::ExecutionEngine*"
QT_MOC_LITERAL(6, 84, 6), // "engine"
QT_MOC_LITERAL(7, 91, 6), // "addref"
QT_MOC_LITERAL(8, 98, 7), // "release"
QT_MOC_LITERAL(9, 106, 5), // "clear"
QT_MOC_LITERAL(10, 112, 6), // "remove"
QT_MOC_LITERAL(11, 119, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(12, 135, 4), // "args"
QT_MOC_LITERAL(13, 140, 6), // "append"
QT_MOC_LITERAL(14, 147, 6), // "insert"
QT_MOC_LITERAL(15, 154, 3), // "get"
QT_MOC_LITERAL(16, 158, 8), // "QJSValue"
QT_MOC_LITERAL(17, 167, 5), // "index"
QT_MOC_LITERAL(18, 173, 3), // "set"
QT_MOC_LITERAL(19, 177, 5), // "value"
QT_MOC_LITERAL(20, 183, 11), // "setProperty"
QT_MOC_LITERAL(21, 195, 8), // "property"
QT_MOC_LITERAL(22, 204, 4), // "move"
QT_MOC_LITERAL(23, 209, 4), // "from"
QT_MOC_LITERAL(24, 214, 2), // "to"
QT_MOC_LITERAL(25, 217, 5), // "count"
QT_MOC_LITERAL(26, 223, 4) // "sync"

    },
    "QQmlListModelWorkerAgent\0QML.Element\0"
    "anonymous\0engineChanged\0\0QV4::ExecutionEngine*\0"
    "engine\0addref\0release\0clear\0remove\0"
    "QQmlV4Function*\0args\0append\0insert\0"
    "get\0QJSValue\0index\0set\0value\0setProperty\0"
    "property\0move\0from\0to\0count\0sync"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlListModelWorkerAgent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      12,   16, // methods
       2,  114, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   76,    4, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       7,    0,   79,    4, 0x02 /* Public */,
       8,    0,   80,    4, 0x02 /* Public */,
       9,    0,   81,    4, 0x02 /* Public */,
      10,    1,   82,    4, 0x02 /* Public */,
      13,    1,   85,    4, 0x02 /* Public */,
      14,    1,   88,    4, 0x02 /* Public */,
      15,    1,   91,    4, 0x02 /* Public */,
      18,    2,   94,    4, 0x02 /* Public */,
      20,    3,   99,    4, 0x02 /* Public */,
      22,    3,  106,    4, 0x02 /* Public */,
      26,    0,  113,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    0x80000000 | 16, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 16,   17,   19,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::QVariant,   17,   21,   19,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   23,   24,   25,
    QMetaType::Void,

 // properties: name, type, flags
      25, QMetaType::Int, 0x00095001,
       6, 0x80000000 | 5, 0x0049510b,

 // properties: notify_signal_id
       0,
       0,

       0        // eod
};

void QQmlListModelWorkerAgent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlListModelWorkerAgent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->engineChanged((*reinterpret_cast< QV4::ExecutionEngine*(*)>(_a[1]))); break;
        case 1: _t->addref(); break;
        case 2: _t->release(); break;
        case 3: _t->clear(); break;
        case 4: _t->remove((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 5: _t->append((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 6: _t->insert((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 7: { QJSValue _r = _t->get((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->set((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QJSValue(*)>(_a[2]))); break;
        case 9: _t->setProperty((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 10: _t->move((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 11: _t->sync(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlListModelWorkerAgent::*)(QV4::ExecutionEngine * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlListModelWorkerAgent::engineChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlListModelWorkerAgent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast< QV4::ExecutionEngine**>(_v) = _t->engine(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlListModelWorkerAgent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setEngine(*reinterpret_cast< QV4::ExecutionEngine**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlListModelWorkerAgent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlListModelWorkerAgent.data,
    qt_meta_data_QQmlListModelWorkerAgent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlListModelWorkerAgent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlListModelWorkerAgent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlListModelWorkerAgent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlListModelWorkerAgent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlListModelWorkerAgent::engineChanged(QV4::ExecutionEngine * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
