/****************************************************************************
** Meta object code from reading C++ file 'tst_qjsvalue.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../git/tests/auto/qml/qjsvalue/tst_qjsvalue.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tst_qjsvalue.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_tst_QJSValue_t {
    QByteArrayData data[95];
    char stringdata0[1763];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_tst_QJSValue_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_tst_QJSValue_t qt_meta_stringdata_tst_QJSValue = {
    {
QT_MOC_LITERAL(0, 0, 12), // "tst_QJSValue"
QT_MOC_LITERAL(1, 13, 12), // "ctor_invalid"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 24), // "ctor_undefinedWithEngine"
QT_MOC_LITERAL(4, 52, 14), // "ctor_undefined"
QT_MOC_LITERAL(5, 67, 19), // "ctor_nullWithEngine"
QT_MOC_LITERAL(6, 87, 9), // "ctor_null"
QT_MOC_LITERAL(7, 97, 19), // "ctor_boolWithEngine"
QT_MOC_LITERAL(8, 117, 9), // "ctor_bool"
QT_MOC_LITERAL(9, 127, 18), // "ctor_intWithEngine"
QT_MOC_LITERAL(10, 146, 8), // "ctor_int"
QT_MOC_LITERAL(11, 155, 19), // "ctor_uintWithEngine"
QT_MOC_LITERAL(12, 175, 9), // "ctor_uint"
QT_MOC_LITERAL(13, 185, 20), // "ctor_floatWithEngine"
QT_MOC_LITERAL(14, 206, 10), // "ctor_float"
QT_MOC_LITERAL(15, 217, 21), // "ctor_stringWithEngine"
QT_MOC_LITERAL(16, 239, 11), // "ctor_string"
QT_MOC_LITERAL(17, 251, 28), // "ctor_copyAndAssignWithEngine"
QT_MOC_LITERAL(18, 280, 18), // "ctor_copyAndAssign"
QT_MOC_LITERAL(19, 299, 8), // "toString"
QT_MOC_LITERAL(20, 308, 8), // "toNumber"
QT_MOC_LITERAL(21, 317, 9), // "toBoolean"
QT_MOC_LITERAL(22, 327, 6), // "toBool"
QT_MOC_LITERAL(23, 334, 5), // "toInt"
QT_MOC_LITERAL(24, 340, 6), // "toUInt"
QT_MOC_LITERAL(25, 347, 9), // "toVariant"
QT_MOC_LITERAL(26, 357, 25), // "toQObject_nonQObject_data"
QT_MOC_LITERAL(27, 383, 20), // "toQObject_nonQObject"
QT_MOC_LITERAL(28, 404, 9), // "toQObject"
QT_MOC_LITERAL(29, 414, 10), // "toDateTime"
QT_MOC_LITERAL(30, 425, 8), // "toRegExp"
QT_MOC_LITERAL(31, 434, 19), // "toRegularExpression"
QT_MOC_LITERAL(32, 454, 12), // "isArray_data"
QT_MOC_LITERAL(33, 467, 7), // "isArray"
QT_MOC_LITERAL(34, 475, 6), // "isDate"
QT_MOC_LITERAL(35, 482, 11), // "isDate_data"
QT_MOC_LITERAL(36, 494, 32), // "isError_propertiesOfGlobalObject"
QT_MOC_LITERAL(37, 527, 12), // "isError_data"
QT_MOC_LITERAL(38, 540, 7), // "isError"
QT_MOC_LITERAL(39, 548, 13), // "isRegExp_data"
QT_MOC_LITERAL(40, 562, 8), // "isRegExp"
QT_MOC_LITERAL(41, 571, 6), // "equals"
QT_MOC_LITERAL(42, 578, 14), // "strictlyEquals"
QT_MOC_LITERAL(43, 593, 17), // "hasProperty_basic"
QT_MOC_LITERAL(44, 611, 24), // "hasProperty_globalObject"
QT_MOC_LITERAL(45, 636, 27), // "hasProperty_changePrototype"
QT_MOC_LITERAL(46, 664, 27), // "hasProperty_QTBUG56830_data"
QT_MOC_LITERAL(47, 692, 22), // "hasProperty_QTBUG56830"
QT_MOC_LITERAL(48, 715, 20), // "deleteProperty_basic"
QT_MOC_LITERAL(49, 736, 27), // "deleteProperty_globalObject"
QT_MOC_LITERAL(50, 764, 26), // "deleteProperty_inPrototype"
QT_MOC_LITERAL(51, 791, 31), // "getSetPrototype_cyclicPrototype"
QT_MOC_LITERAL(52, 823, 35), // "getSetPrototype_evalCyclicPro..."
QT_MOC_LITERAL(53, 859, 20), // "getSetPrototype_eval"
QT_MOC_LITERAL(54, 880, 32), // "getSetPrototype_invalidPrototype"
QT_MOC_LITERAL(55, 913, 26), // "getSetPrototype_twoEngines"
QT_MOC_LITERAL(56, 940, 20), // "getSetPrototype_null"
QT_MOC_LITERAL(57, 961, 31), // "getSetPrototype_notObjectOrNull"
QT_MOC_LITERAL(58, 993, 15), // "getSetPrototype"
QT_MOC_LITERAL(59, 1009, 33), // "getSetProperty_HooliganTask16..."
QT_MOC_LITERAL(60, 1043, 33), // "getSetProperty_HooliganTask18..."
QT_MOC_LITERAL(61, 1077, 30), // "getSetProperty_propertyRemoval"
QT_MOC_LITERAL(62, 1108, 26), // "getSetProperty_resolveMode"
QT_MOC_LITERAL(63, 1135, 25), // "getSetProperty_twoEngines"
QT_MOC_LITERAL(64, 1161, 44), // "getSetProperty_gettersAndSett..."
QT_MOC_LITERAL(65, 1206, 20), // "getSetProperty_array"
QT_MOC_LITERAL(66, 1227, 14), // "getSetProperty"
QT_MOC_LITERAL(67, 1242, 13), // "call_function"
QT_MOC_LITERAL(68, 1256, 11), // "call_object"
QT_MOC_LITERAL(69, 1268, 15), // "call_newObjects"
QT_MOC_LITERAL(70, 1284, 9), // "call_this"
QT_MOC_LITERAL(71, 1294, 14), // "call_arguments"
QT_MOC_LITERAL(72, 1309, 4), // "call"
QT_MOC_LITERAL(73, 1314, 15), // "call_twoEngines"
QT_MOC_LITERAL(74, 1330, 21), // "call_nonFunction_data"
QT_MOC_LITERAL(75, 1352, 16), // "call_nonFunction"
QT_MOC_LITERAL(76, 1369, 26), // "construct_nonFunction_data"
QT_MOC_LITERAL(77, 1396, 21), // "construct_nonFunction"
QT_MOC_LITERAL(78, 1418, 16), // "construct_simple"
QT_MOC_LITERAL(79, 1435, 21), // "construct_newObjectJS"
QT_MOC_LITERAL(80, 1457, 13), // "construct_arg"
QT_MOC_LITERAL(81, 1471, 15), // "construct_proto"
QT_MOC_LITERAL(82, 1487, 19), // "construct_returnInt"
QT_MOC_LITERAL(83, 1507, 15), // "construct_throw"
QT_MOC_LITERAL(84, 1523, 20), // "construct_twoEngines"
QT_MOC_LITERAL(85, 1544, 36), // "construct_constructorThrowsPr..."
QT_MOC_LITERAL(86, 1581, 13), // "castToPointer"
QT_MOC_LITERAL(87, 1595, 18), // "prettyPrinter_data"
QT_MOC_LITERAL(88, 1614, 13), // "prettyPrinter"
QT_MOC_LITERAL(89, 1628, 13), // "engineDeleted"
QT_MOC_LITERAL(90, 1642, 18), // "valueOfWithClosure"
QT_MOC_LITERAL(91, 1661, 26), // "nestedObjectToVariant_data"
QT_MOC_LITERAL(92, 1688, 21), // "nestedObjectToVariant"
QT_MOC_LITERAL(93, 1710, 26), // "jsvalueArrayToSequenceType"
QT_MOC_LITERAL(94, 1737, 25) // "deleteFromDifferentThread"

    },
    "tst_QJSValue\0ctor_invalid\0\0"
    "ctor_undefinedWithEngine\0ctor_undefined\0"
    "ctor_nullWithEngine\0ctor_null\0"
    "ctor_boolWithEngine\0ctor_bool\0"
    "ctor_intWithEngine\0ctor_int\0"
    "ctor_uintWithEngine\0ctor_uint\0"
    "ctor_floatWithEngine\0ctor_float\0"
    "ctor_stringWithEngine\0ctor_string\0"
    "ctor_copyAndAssignWithEngine\0"
    "ctor_copyAndAssign\0toString\0toNumber\0"
    "toBoolean\0toBool\0toInt\0toUInt\0toVariant\0"
    "toQObject_nonQObject_data\0"
    "toQObject_nonQObject\0toQObject\0"
    "toDateTime\0toRegExp\0toRegularExpression\0"
    "isArray_data\0isArray\0isDate\0isDate_data\0"
    "isError_propertiesOfGlobalObject\0"
    "isError_data\0isError\0isRegExp_data\0"
    "isRegExp\0equals\0strictlyEquals\0"
    "hasProperty_basic\0hasProperty_globalObject\0"
    "hasProperty_changePrototype\0"
    "hasProperty_QTBUG56830_data\0"
    "hasProperty_QTBUG56830\0deleteProperty_basic\0"
    "deleteProperty_globalObject\0"
    "deleteProperty_inPrototype\0"
    "getSetPrototype_cyclicPrototype\0"
    "getSetPrototype_evalCyclicPrototype\0"
    "getSetPrototype_eval\0"
    "getSetPrototype_invalidPrototype\0"
    "getSetPrototype_twoEngines\0"
    "getSetPrototype_null\0"
    "getSetPrototype_notObjectOrNull\0"
    "getSetPrototype\0getSetProperty_HooliganTask162051\0"
    "getSetProperty_HooliganTask183072\0"
    "getSetProperty_propertyRemoval\0"
    "getSetProperty_resolveMode\0"
    "getSetProperty_twoEngines\0"
    "getSetProperty_gettersAndSettersThrowErrorJS\0"
    "getSetProperty_array\0getSetProperty\0"
    "call_function\0call_object\0call_newObjects\0"
    "call_this\0call_arguments\0call\0"
    "call_twoEngines\0call_nonFunction_data\0"
    "call_nonFunction\0construct_nonFunction_data\0"
    "construct_nonFunction\0construct_simple\0"
    "construct_newObjectJS\0construct_arg\0"
    "construct_proto\0construct_returnInt\0"
    "construct_throw\0construct_twoEngines\0"
    "construct_constructorThrowsPrimitive\0"
    "castToPointer\0prettyPrinter_data\0"
    "prettyPrinter\0engineDeleted\0"
    "valueOfWithClosure\0nestedObjectToVariant_data\0"
    "nestedObjectToVariant\0jsvalueArrayToSequenceType\0"
    "deleteFromDifferentThread"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_tst_QJSValue[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      93,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  479,    2, 0x08 /* Private */,
       3,    0,  480,    2, 0x08 /* Private */,
       4,    0,  481,    2, 0x08 /* Private */,
       5,    0,  482,    2, 0x08 /* Private */,
       6,    0,  483,    2, 0x08 /* Private */,
       7,    0,  484,    2, 0x08 /* Private */,
       8,    0,  485,    2, 0x08 /* Private */,
       9,    0,  486,    2, 0x08 /* Private */,
      10,    0,  487,    2, 0x08 /* Private */,
      11,    0,  488,    2, 0x08 /* Private */,
      12,    0,  489,    2, 0x08 /* Private */,
      13,    0,  490,    2, 0x08 /* Private */,
      14,    0,  491,    2, 0x08 /* Private */,
      15,    0,  492,    2, 0x08 /* Private */,
      16,    0,  493,    2, 0x08 /* Private */,
      17,    0,  494,    2, 0x08 /* Private */,
      18,    0,  495,    2, 0x08 /* Private */,
      19,    0,  496,    2, 0x08 /* Private */,
      20,    0,  497,    2, 0x08 /* Private */,
      21,    0,  498,    2, 0x08 /* Private */,
      22,    0,  499,    2, 0x08 /* Private */,
      23,    0,  500,    2, 0x08 /* Private */,
      24,    0,  501,    2, 0x08 /* Private */,
      25,    0,  502,    2, 0x08 /* Private */,
      26,    0,  503,    2, 0x08 /* Private */,
      27,    0,  504,    2, 0x08 /* Private */,
      28,    0,  505,    2, 0x08 /* Private */,
      29,    0,  506,    2, 0x08 /* Private */,
      30,    0,  507,    2, 0x08 /* Private */,
      31,    0,  508,    2, 0x08 /* Private */,
      32,    0,  509,    2, 0x08 /* Private */,
      33,    0,  510,    2, 0x08 /* Private */,
      34,    0,  511,    2, 0x08 /* Private */,
      35,    0,  512,    2, 0x08 /* Private */,
      36,    0,  513,    2, 0x08 /* Private */,
      37,    0,  514,    2, 0x08 /* Private */,
      38,    0,  515,    2, 0x08 /* Private */,
      39,    0,  516,    2, 0x08 /* Private */,
      40,    0,  517,    2, 0x08 /* Private */,
      41,    0,  518,    2, 0x08 /* Private */,
      42,    0,  519,    2, 0x08 /* Private */,
      43,    0,  520,    2, 0x08 /* Private */,
      44,    0,  521,    2, 0x08 /* Private */,
      45,    0,  522,    2, 0x08 /* Private */,
      46,    0,  523,    2, 0x08 /* Private */,
      47,    0,  524,    2, 0x08 /* Private */,
      48,    0,  525,    2, 0x08 /* Private */,
      49,    0,  526,    2, 0x08 /* Private */,
      50,    0,  527,    2, 0x08 /* Private */,
      51,    0,  528,    2, 0x08 /* Private */,
      52,    0,  529,    2, 0x08 /* Private */,
      53,    0,  530,    2, 0x08 /* Private */,
      54,    0,  531,    2, 0x08 /* Private */,
      55,    0,  532,    2, 0x08 /* Private */,
      56,    0,  533,    2, 0x08 /* Private */,
      57,    0,  534,    2, 0x08 /* Private */,
      58,    0,  535,    2, 0x08 /* Private */,
      59,    0,  536,    2, 0x08 /* Private */,
      60,    0,  537,    2, 0x08 /* Private */,
      61,    0,  538,    2, 0x08 /* Private */,
      62,    0,  539,    2, 0x08 /* Private */,
      63,    0,  540,    2, 0x08 /* Private */,
      64,    0,  541,    2, 0x08 /* Private */,
      65,    0,  542,    2, 0x08 /* Private */,
      66,    0,  543,    2, 0x08 /* Private */,
      67,    0,  544,    2, 0x08 /* Private */,
      68,    0,  545,    2, 0x08 /* Private */,
      69,    0,  546,    2, 0x08 /* Private */,
      70,    0,  547,    2, 0x08 /* Private */,
      71,    0,  548,    2, 0x08 /* Private */,
      72,    0,  549,    2, 0x08 /* Private */,
      73,    0,  550,    2, 0x08 /* Private */,
      74,    0,  551,    2, 0x08 /* Private */,
      75,    0,  552,    2, 0x08 /* Private */,
      76,    0,  553,    2, 0x08 /* Private */,
      77,    0,  554,    2, 0x08 /* Private */,
      78,    0,  555,    2, 0x08 /* Private */,
      79,    0,  556,    2, 0x08 /* Private */,
      80,    0,  557,    2, 0x08 /* Private */,
      81,    0,  558,    2, 0x08 /* Private */,
      82,    0,  559,    2, 0x08 /* Private */,
      83,    0,  560,    2, 0x08 /* Private */,
      84,    0,  561,    2, 0x08 /* Private */,
      85,    0,  562,    2, 0x08 /* Private */,
      86,    0,  563,    2, 0x08 /* Private */,
      87,    0,  564,    2, 0x08 /* Private */,
      88,    0,  565,    2, 0x08 /* Private */,
      89,    0,  566,    2, 0x08 /* Private */,
      90,    0,  567,    2, 0x08 /* Private */,
      91,    0,  568,    2, 0x08 /* Private */,
      92,    0,  569,    2, 0x08 /* Private */,
      93,    0,  570,    2, 0x08 /* Private */,
      94,    0,  571,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void tst_QJSValue::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<tst_QJSValue *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ctor_invalid(); break;
        case 1: _t->ctor_undefinedWithEngine(); break;
        case 2: _t->ctor_undefined(); break;
        case 3: _t->ctor_nullWithEngine(); break;
        case 4: _t->ctor_null(); break;
        case 5: _t->ctor_boolWithEngine(); break;
        case 6: _t->ctor_bool(); break;
        case 7: _t->ctor_intWithEngine(); break;
        case 8: _t->ctor_int(); break;
        case 9: _t->ctor_uintWithEngine(); break;
        case 10: _t->ctor_uint(); break;
        case 11: _t->ctor_floatWithEngine(); break;
        case 12: _t->ctor_float(); break;
        case 13: _t->ctor_stringWithEngine(); break;
        case 14: _t->ctor_string(); break;
        case 15: _t->ctor_copyAndAssignWithEngine(); break;
        case 16: _t->ctor_copyAndAssign(); break;
        case 17: _t->toString(); break;
        case 18: _t->toNumber(); break;
        case 19: _t->toBoolean(); break;
        case 20: _t->toBool(); break;
        case 21: _t->toInt(); break;
        case 22: _t->toUInt(); break;
        case 23: _t->toVariant(); break;
        case 24: _t->toQObject_nonQObject_data(); break;
        case 25: _t->toQObject_nonQObject(); break;
        case 26: _t->toQObject(); break;
        case 27: _t->toDateTime(); break;
        case 28: _t->toRegExp(); break;
        case 29: _t->toRegularExpression(); break;
        case 30: _t->isArray_data(); break;
        case 31: _t->isArray(); break;
        case 32: _t->isDate(); break;
        case 33: _t->isDate_data(); break;
        case 34: _t->isError_propertiesOfGlobalObject(); break;
        case 35: _t->isError_data(); break;
        case 36: _t->isError(); break;
        case 37: _t->isRegExp_data(); break;
        case 38: _t->isRegExp(); break;
        case 39: _t->equals(); break;
        case 40: _t->strictlyEquals(); break;
        case 41: _t->hasProperty_basic(); break;
        case 42: _t->hasProperty_globalObject(); break;
        case 43: _t->hasProperty_changePrototype(); break;
        case 44: _t->hasProperty_QTBUG56830_data(); break;
        case 45: _t->hasProperty_QTBUG56830(); break;
        case 46: _t->deleteProperty_basic(); break;
        case 47: _t->deleteProperty_globalObject(); break;
        case 48: _t->deleteProperty_inPrototype(); break;
        case 49: _t->getSetPrototype_cyclicPrototype(); break;
        case 50: _t->getSetPrototype_evalCyclicPrototype(); break;
        case 51: _t->getSetPrototype_eval(); break;
        case 52: _t->getSetPrototype_invalidPrototype(); break;
        case 53: _t->getSetPrototype_twoEngines(); break;
        case 54: _t->getSetPrototype_null(); break;
        case 55: _t->getSetPrototype_notObjectOrNull(); break;
        case 56: _t->getSetPrototype(); break;
        case 57: _t->getSetProperty_HooliganTask162051(); break;
        case 58: _t->getSetProperty_HooliganTask183072(); break;
        case 59: _t->getSetProperty_propertyRemoval(); break;
        case 60: _t->getSetProperty_resolveMode(); break;
        case 61: _t->getSetProperty_twoEngines(); break;
        case 62: _t->getSetProperty_gettersAndSettersThrowErrorJS(); break;
        case 63: _t->getSetProperty_array(); break;
        case 64: _t->getSetProperty(); break;
        case 65: _t->call_function(); break;
        case 66: _t->call_object(); break;
        case 67: _t->call_newObjects(); break;
        case 68: _t->call_this(); break;
        case 69: _t->call_arguments(); break;
        case 70: _t->call(); break;
        case 71: _t->call_twoEngines(); break;
        case 72: _t->call_nonFunction_data(); break;
        case 73: _t->call_nonFunction(); break;
        case 74: _t->construct_nonFunction_data(); break;
        case 75: _t->construct_nonFunction(); break;
        case 76: _t->construct_simple(); break;
        case 77: _t->construct_newObjectJS(); break;
        case 78: _t->construct_arg(); break;
        case 79: _t->construct_proto(); break;
        case 80: _t->construct_returnInt(); break;
        case 81: _t->construct_throw(); break;
        case 82: _t->construct_twoEngines(); break;
        case 83: _t->construct_constructorThrowsPrimitive(); break;
        case 84: _t->castToPointer(); break;
        case 85: _t->prettyPrinter_data(); break;
        case 86: _t->prettyPrinter(); break;
        case 87: _t->engineDeleted(); break;
        case 88: _t->valueOfWithClosure(); break;
        case 89: _t->nestedObjectToVariant_data(); break;
        case 90: _t->nestedObjectToVariant(); break;
        case 91: _t->jsvalueArrayToSequenceType(); break;
        case 92: _t->deleteFromDifferentThread(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject tst_QJSValue::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_tst_QJSValue.data,
    qt_meta_data_tst_QJSValue,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *tst_QJSValue::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *tst_QJSValue::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_tst_QJSValue.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int tst_QJSValue::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 93)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 93;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 93)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 93;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
