/****************************************************************************
** Meta object code from reading C++ file 'qquicktableview_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquicktableview_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktableview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTableView_t {
    QByteArrayData data[40];
    char stringdata0[590];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTableView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTableView_t qt_meta_stringdata_QQuickTableView = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickTableView"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "TableView"
QT_MOC_LITERAL(3, 38, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 62, 2), // "12"
QT_MOC_LITERAL(5, 65, 12), // "QML.Attached"
QT_MOC_LITERAL(6, 78, 23), // "QQuickTableViewAttached"
QT_MOC_LITERAL(7, 102, 11), // "rowsChanged"
QT_MOC_LITERAL(8, 114, 0), // ""
QT_MOC_LITERAL(9, 115, 14), // "columnsChanged"
QT_MOC_LITERAL(10, 130, 17), // "rowSpacingChanged"
QT_MOC_LITERAL(11, 148, 20), // "columnSpacingChanged"
QT_MOC_LITERAL(12, 169, 24), // "rowHeightProviderChanged"
QT_MOC_LITERAL(13, 194, 26), // "columnWidthProviderChanged"
QT_MOC_LITERAL(14, 221, 12), // "modelChanged"
QT_MOC_LITERAL(15, 234, 15), // "delegateChanged"
QT_MOC_LITERAL(16, 250, 17), // "reuseItemsChanged"
QT_MOC_LITERAL(17, 268, 15), // "syncViewChanged"
QT_MOC_LITERAL(18, 284, 20), // "syncDirectionChanged"
QT_MOC_LITERAL(19, 305, 21), // "_q_componentFinalized"
QT_MOC_LITERAL(20, 327, 11), // "forceLayout"
QT_MOC_LITERAL(21, 339, 19), // "contentWidthChanged"
QT_MOC_LITERAL(22, 359, 20), // "contentHeightChanged"
QT_MOC_LITERAL(23, 380, 4), // "rows"
QT_MOC_LITERAL(24, 385, 7), // "columns"
QT_MOC_LITERAL(25, 393, 10), // "rowSpacing"
QT_MOC_LITERAL(26, 404, 13), // "columnSpacing"
QT_MOC_LITERAL(27, 418, 17), // "rowHeightProvider"
QT_MOC_LITERAL(28, 436, 8), // "QJSValue"
QT_MOC_LITERAL(29, 445, 19), // "columnWidthProvider"
QT_MOC_LITERAL(30, 465, 5), // "model"
QT_MOC_LITERAL(31, 471, 8), // "delegate"
QT_MOC_LITERAL(32, 480, 14), // "QQmlComponent*"
QT_MOC_LITERAL(33, 495, 10), // "reuseItems"
QT_MOC_LITERAL(34, 506, 12), // "contentWidth"
QT_MOC_LITERAL(35, 519, 13), // "contentHeight"
QT_MOC_LITERAL(36, 533, 8), // "syncView"
QT_MOC_LITERAL(37, 542, 16), // "QQuickTableView*"
QT_MOC_LITERAL(38, 559, 13), // "syncDirection"
QT_MOC_LITERAL(39, 573, 16) // "Qt::Orientations"

    },
    "QQuickTableView\0QML.Element\0TableView\0"
    "QML.AddedInMinorVersion\0""12\0QML.Attached\0"
    "QQuickTableViewAttached\0rowsChanged\0"
    "\0columnsChanged\0rowSpacingChanged\0"
    "columnSpacingChanged\0rowHeightProviderChanged\0"
    "columnWidthProviderChanged\0modelChanged\0"
    "delegateChanged\0reuseItemsChanged\0"
    "syncViewChanged\0syncDirectionChanged\0"
    "_q_componentFinalized\0forceLayout\0"
    "contentWidthChanged\0contentHeightChanged\0"
    "rows\0columns\0rowSpacing\0columnSpacing\0"
    "rowHeightProvider\0QJSValue\0"
    "columnWidthProvider\0model\0delegate\0"
    "QQmlComponent*\0reuseItems\0contentWidth\0"
    "contentHeight\0syncView\0QQuickTableView*\0"
    "syncDirection\0Qt::Orientations"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTableView[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
      13,   20, // methods
      13,  111, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   98,    8, 0x06 /* Public */,
       9,    0,   99,    8, 0x06 /* Public */,
      10,    0,  100,    8, 0x06 /* Public */,
      11,    0,  101,    8, 0x06 /* Public */,
      12,    0,  102,    8, 0x06 /* Public */,
      13,    0,  103,    8, 0x06 /* Public */,
      14,    0,  104,    8, 0x06 /* Public */,
      15,    0,  105,    8, 0x06 /* Public */,
      16,    0,  106,    8, 0x06 /* Public */,
      17,    0,  107,    8, 0x86 /* Public | MethodRevisioned */,
      18,    0,  108,    8, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      19,    0,  109,    8, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      20,    0,  110,    8, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      14,
      14,

 // slots: revision
       0,

 // methods: revision
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
      23, QMetaType::Int, 0x00495001,
      24, QMetaType::Int, 0x00495001,
      25, QMetaType::QReal, 0x00495103,
      26, QMetaType::QReal, 0x00495103,
      27, 0x80000000 | 28, 0x0049510b,
      29, 0x80000000 | 28, 0x0049510b,
      30, QMetaType::QVariant, 0x00495103,
      31, 0x80000000 | 32, 0x0049510b,
      33, QMetaType::Bool, 0x00495103,
      34, QMetaType::QReal, 0x00495103,
      35, QMetaType::QReal, 0x00495103,
      36, 0x80000000 | 37, 0x00c9510b,
      38, 0x80000000 | 39, 0x00c9510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
    1879048213,
    1879048214,
       9,
      10,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      14,
      14,

       0        // eod
};

void QQuickTableView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTableView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->rowsChanged(); break;
        case 1: _t->columnsChanged(); break;
        case 2: _t->rowSpacingChanged(); break;
        case 3: _t->columnSpacingChanged(); break;
        case 4: _t->rowHeightProviderChanged(); break;
        case 5: _t->columnWidthProviderChanged(); break;
        case 6: _t->modelChanged(); break;
        case 7: _t->delegateChanged(); break;
        case 8: _t->reuseItemsChanged(); break;
        case 9: _t->syncViewChanged(); break;
        case 10: _t->syncDirectionChanged(); break;
        case 11: _t->d_func()->_q_componentFinalized(); break;
        case 12: _t->forceLayout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::rowsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::columnsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::rowSpacingChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::columnSpacingChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::rowHeightProviderChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::columnWidthProviderChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::modelChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::delegateChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::reuseItemsChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::syncViewChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickTableView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableView::syncDirectionChanged)) {
                *result = 10;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 11:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTableView* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTableView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->rows(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->columns(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->rowSpacing(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->columnSpacing(); break;
        case 4: *reinterpret_cast< QJSValue*>(_v) = _t->rowHeightProvider(); break;
        case 5: *reinterpret_cast< QJSValue*>(_v) = _t->columnWidthProvider(); break;
        case 6: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 7: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->reuseItems(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 11: *reinterpret_cast< QQuickTableView**>(_v) = _t->syncView(); break;
        case 12: *reinterpret_cast< Qt::Orientations*>(_v) = _t->syncDirection(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTableView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setRowSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setColumnSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setRowHeightProvider(*reinterpret_cast< QJSValue*>(_v)); break;
        case 5: _t->setColumnWidthProvider(*reinterpret_cast< QJSValue*>(_v)); break;
        case 6: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 7: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 8: _t->setReuseItems(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setContentWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setContentHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setSyncView(*reinterpret_cast< QQuickTableView**>(_v)); break;
        case 12: _t->setSyncDirection(*reinterpret_cast< Qt::Orientations*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTableView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickFlickable::staticMetaObject>(),
    qt_meta_stringdata_QQuickTableView.data,
    qt_meta_data_QQuickTableView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTableView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTableView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTableView.stringdata0))
        return static_cast<void*>(this);
    return QQuickFlickable::qt_metacast(_clname);
}

int QQuickTableView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickFlickable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTableView::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTableView::columnsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTableView::rowSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickTableView::columnSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickTableView::rowHeightProviderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickTableView::columnWidthProviderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickTableView::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickTableView::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickTableView::reuseItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickTableView::syncViewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickTableView::syncDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
// If you get a compile error in this function it can be because either
//     a) You are using a NOTIFY signal that does not exist. Fix it.
//     b) You are using a NOTIFY signal that does exist (in a parent class) but has a non-empty parameter list. This is a moc limitation.
Q_DECL_UNUSED static void checkNotifySignalValidity_QQuickTableView(QQuickTableView *t) {
    t->contentWidthChanged();
    t->contentHeightChanged();
}
struct qt_meta_stringdata_QQuickTableViewAttached_t {
    QByteArrayData data[7];
    char stringdata0[73];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTableViewAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTableViewAttached_t qt_meta_stringdata_QQuickTableViewAttached = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickTableViewAttached"
QT_MOC_LITERAL(1, 24, 11), // "viewChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 6), // "pooled"
QT_MOC_LITERAL(4, 44, 6), // "reused"
QT_MOC_LITERAL(5, 51, 4), // "view"
QT_MOC_LITERAL(6, 56, 16) // "QQuickTableView*"

    },
    "QQuickTableViewAttached\0viewChanged\0"
    "\0pooled\0reused\0view\0QQuickTableView*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTableViewAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       1,   32, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,
       4,    0,   31,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       5, 0x80000000 | 6, 0x00495009,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickTableViewAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTableViewAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->viewChanged(); break;
        case 1: _t->pooled(); break;
        case 2: _t->reused(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTableViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableViewAttached::viewChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTableViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableViewAttached::pooled)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTableViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTableViewAttached::reused)) {
                *result = 2;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTableView* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTableViewAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickTableView**>(_v) = _t->view(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTableViewAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickTableViewAttached.data,
    qt_meta_data_QQuickTableViewAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTableViewAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTableViewAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTableViewAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickTableViewAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTableViewAttached::viewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTableViewAttached::pooled()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTableViewAttached::reused()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
