/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qquickshape_p.h>

void qml_register_types_QtQuick_Shapes()
{
    qmlRegisterTypesAndRevisions<QQuickShape>("QtQuick.Shapes", 1);
    qmlRegisterTypesAndRevisions<QQuickShapeConicalGradient>("QtQuick.Shapes", 1);
    qmlRegisterTypesAndRevisions<QQuickShapeGradient>("QtQuick.Shapes", 1);
    qmlRegisterTypesAndRevisions<QQuickShapeLinearGradient>("QtQuick.Shapes", 1);
    qmlRegisterTypesAndRevisions<QQuickShapePath>("QtQuick.Shapes", 1);
    qmlRegisterTypesAndRevisions<QQuickShapeRadialGradient>("QtQuick.Shapes", 1);
    qmlRegisterModule("QtQuick.Shapes", 1, 15);
}

static const QQmlModuleRegistration registration("QtQuick.Shapes", 1, qml_register_types_QtQuick_Shapes);
