/****************************************************************************
** Meta object code from reading C++ file 'qquickwheelhandler_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/handlers/qquickwheelhandler_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickwheelhandler_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickWheelHandler_t {
    QByteArrayData data[26];
    char stringdata0[420];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWheelHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWheelHandler_t qt_meta_stringdata_QQuickWheelHandler = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickWheelHandler"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "WheelHandler"
QT_MOC_LITERAL(3, 44, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 68, 2), // "14"
QT_MOC_LITERAL(5, 71, 5), // "wheel"
QT_MOC_LITERAL(6, 77, 0), // ""
QT_MOC_LITERAL(7, 78, 25), // "QQuickPointerScrollEvent*"
QT_MOC_LITERAL(8, 104, 5), // "event"
QT_MOC_LITERAL(9, 110, 18), // "orientationChanged"
QT_MOC_LITERAL(10, 129, 17), // "invertibleChanged"
QT_MOC_LITERAL(11, 147, 20), // "activeTimeoutChanged"
QT_MOC_LITERAL(12, 168, 15), // "rotationChanged"
QT_MOC_LITERAL(13, 184, 20), // "rotationScaleChanged"
QT_MOC_LITERAL(14, 205, 15), // "propertyChanged"
QT_MOC_LITERAL(15, 221, 28), // "targetScaleMultiplierChanged"
QT_MOC_LITERAL(16, 250, 34), // "targetTransformAroundCursorCh..."
QT_MOC_LITERAL(17, 285, 11), // "orientation"
QT_MOC_LITERAL(18, 297, 15), // "Qt::Orientation"
QT_MOC_LITERAL(19, 313, 10), // "invertible"
QT_MOC_LITERAL(20, 324, 13), // "activeTimeout"
QT_MOC_LITERAL(21, 338, 8), // "rotation"
QT_MOC_LITERAL(22, 347, 13), // "rotationScale"
QT_MOC_LITERAL(23, 361, 8), // "property"
QT_MOC_LITERAL(24, 370, 21), // "targetScaleMultiplier"
QT_MOC_LITERAL(25, 392, 27) // "targetTransformAroundCursor"

    },
    "QQuickWheelHandler\0QML.Element\0"
    "WheelHandler\0QML.AddedInMinorVersion\0"
    "14\0wheel\0\0QQuickPointerScrollEvent*\0"
    "event\0orientationChanged\0invertibleChanged\0"
    "activeTimeoutChanged\0rotationChanged\0"
    "rotationScaleChanged\0propertyChanged\0"
    "targetScaleMultiplierChanged\0"
    "targetTransformAroundCursorChanged\0"
    "orientation\0Qt::Orientation\0invertible\0"
    "activeTimeout\0rotation\0rotationScale\0"
    "property\0targetScaleMultiplier\0"
    "targetTransformAroundCursor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWheelHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
       8,   74, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   63,    6, 0x06 /* Public */,
       9,    0,   66,    6, 0x06 /* Public */,
      10,    0,   67,    6, 0x06 /* Public */,
      11,    0,   68,    6, 0x06 /* Public */,
      12,    0,   69,    6, 0x06 /* Public */,
      13,    0,   70,    6, 0x06 /* Public */,
      14,    0,   71,    6, 0x06 /* Public */,
      15,    0,   72,    6, 0x06 /* Public */,
      16,    0,   73,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      17, 0x80000000 | 18, 0x0049510b,
      19, QMetaType::Bool, 0x00495103,
      20, QMetaType::QReal, 0x00495103,
      21, QMetaType::QReal, 0x00495103,
      22, QMetaType::QReal, 0x00495103,
      23, QMetaType::QString, 0x00495103,
      24, QMetaType::QReal, 0x00495103,
      25, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,

       0        // eod
};

void QQuickWheelHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickWheelHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->wheel((*reinterpret_cast< QQuickPointerScrollEvent*(*)>(_a[1]))); break;
        case 1: _t->orientationChanged(); break;
        case 2: _t->invertibleChanged(); break;
        case 3: _t->activeTimeoutChanged(); break;
        case 4: _t->rotationChanged(); break;
        case 5: _t->rotationScaleChanged(); break;
        case 6: _t->propertyChanged(); break;
        case 7: _t->targetScaleMultiplierChanged(); break;
        case 8: _t->targetTransformAroundCursorChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPointerScrollEvent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickWheelHandler::*)(QQuickPointerScrollEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::wheel)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::orientationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::invertibleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::activeTimeoutChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::rotationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::rotationScaleChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::propertyChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::targetScaleMultiplierChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickWheelHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWheelHandler::targetTransformAroundCursorChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickWheelHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::Orientation*>(_v) = _t->orientation(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isInvertible(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->activeTimeout(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->rotationScale(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->property(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->targetScaleMultiplier(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->isTargetTransformAroundCursor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickWheelHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOrientation(*reinterpret_cast< Qt::Orientation*>(_v)); break;
        case 1: _t->setInvertible(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setActiveTimeout(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setRotationScale(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setProperty(*reinterpret_cast< QString*>(_v)); break;
        case 6: _t->setTargetScaleMultiplier(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setTargetTransformAroundCursor(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickWheelHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointHandler::staticMetaObject>(),
    qt_meta_stringdata_QQuickWheelHandler.data,
    qt_meta_data_QQuickWheelHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickWheelHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickWheelHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickWheelHandler.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointHandler::qt_metacast(_clname);
}

int QQuickWheelHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickWheelHandler::wheel(QQuickPointerScrollEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickWheelHandler::orientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickWheelHandler::invertibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickWheelHandler::activeTimeoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickWheelHandler::rotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickWheelHandler::rotationScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickWheelHandler::propertyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickWheelHandler::targetScaleMultiplierChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickWheelHandler::targetTransformAroundCursorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
