/****************************************************************************
** Meta object code from reading C++ file 'qquickitemsmodule_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickitemsmodule_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitemsmodule_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QPointingDeviceUniqueIdForeign_t {
    QByteArrayData data[11];
    char stringdata0[237];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QPointingDeviceUniqueIdForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QPointingDeviceUniqueIdForeign_t qt_meta_stringdata_QPointingDeviceUniqueIdForeign = {
    {
QT_MOC_LITERAL(0, 0, 30), // "QPointingDeviceUniqueIdForeign"
QT_MOC_LITERAL(1, 31, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 43, 23), // "QPointingDeviceUniqueId"
QT_MOC_LITERAL(3, 67, 11), // "QML.Element"
QT_MOC_LITERAL(4, 79, 22), // "PointingDeviceUniqueId"
QT_MOC_LITERAL(5, 102, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 126, 1), // "9"
QT_MOC_LITERAL(7, 128, 13), // "QML.Creatable"
QT_MOC_LITERAL(8, 142, 5), // "false"
QT_MOC_LITERAL(9, 148, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(10, 170, 66) // "PointingDeviceUniqueId is onl..."

    },
    "QPointingDeviceUniqueIdForeign\0"
    "QML.Foreign\0QPointingDeviceUniqueId\0"
    "QML.Element\0PointingDeviceUniqueId\0"
    "QML.AddedInMinorVersion\0""9\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "PointingDeviceUniqueId is only available via read-only properties."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QPointingDeviceUniqueIdForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,   10,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QPointingDeviceUniqueIdForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QPointingDeviceUniqueIdForeign.data,
    qt_meta_data_QPointingDeviceUniqueIdForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
