/****************************************************************************
** Meta object code from reading C++ file 'qquickfolderlistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/folderlistmodel/qquickfolderlistmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickfolderlistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickFolderListModel_t {
    QByteArrayData data[52];
    char stringdata0[565];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickFolderListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickFolderListModel_t qt_meta_stringdata_QQuickFolderListModel = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickFolderListModel"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "FolderListModel"
QT_MOC_LITERAL(3, 50, 13), // "folderChanged"
QT_MOC_LITERAL(4, 64, 0), // ""
QT_MOC_LITERAL(5, 65, 15), // "rowCountChanged"
QT_MOC_LITERAL(6, 81, 12), // "countChanged"
QT_MOC_LITERAL(7, 94, 13), // "statusChanged"
QT_MOC_LITERAL(8, 108, 19), // "_q_directoryChanged"
QT_MOC_LITERAL(9, 128, 9), // "directory"
QT_MOC_LITERAL(10, 138, 19), // "QList<FileProperty>"
QT_MOC_LITERAL(11, 158, 4), // "list"
QT_MOC_LITERAL(12, 163, 19), // "_q_directoryUpdated"
QT_MOC_LITERAL(13, 183, 9), // "fromIndex"
QT_MOC_LITERAL(14, 193, 7), // "toIndex"
QT_MOC_LITERAL(15, 201, 15), // "_q_sortFinished"
QT_MOC_LITERAL(16, 217, 16), // "_q_statusChanged"
QT_MOC_LITERAL(17, 234, 29), // "QQuickFolderListModel::Status"
QT_MOC_LITERAL(18, 264, 1), // "s"
QT_MOC_LITERAL(19, 266, 8), // "isFolder"
QT_MOC_LITERAL(20, 275, 5), // "index"
QT_MOC_LITERAL(21, 281, 3), // "get"
QT_MOC_LITERAL(22, 285, 3), // "idx"
QT_MOC_LITERAL(23, 289, 8), // "property"
QT_MOC_LITERAL(24, 298, 7), // "indexOf"
QT_MOC_LITERAL(25, 306, 4), // "file"
QT_MOC_LITERAL(26, 311, 6), // "folder"
QT_MOC_LITERAL(27, 318, 10), // "rootFolder"
QT_MOC_LITERAL(28, 329, 12), // "parentFolder"
QT_MOC_LITERAL(29, 342, 11), // "nameFilters"
QT_MOC_LITERAL(30, 354, 9), // "sortField"
QT_MOC_LITERAL(31, 364, 9), // "SortField"
QT_MOC_LITERAL(32, 374, 12), // "sortReversed"
QT_MOC_LITERAL(33, 387, 9), // "showFiles"
QT_MOC_LITERAL(34, 397, 8), // "showDirs"
QT_MOC_LITERAL(35, 406, 13), // "showDirsFirst"
QT_MOC_LITERAL(36, 420, 16), // "showDotAndDotDot"
QT_MOC_LITERAL(37, 437, 10), // "showHidden"
QT_MOC_LITERAL(38, 448, 16), // "showOnlyReadable"
QT_MOC_LITERAL(39, 465, 13), // "caseSensitive"
QT_MOC_LITERAL(40, 479, 5), // "count"
QT_MOC_LITERAL(41, 485, 6), // "status"
QT_MOC_LITERAL(42, 492, 6), // "Status"
QT_MOC_LITERAL(43, 499, 17), // "sortCaseSensitive"
QT_MOC_LITERAL(44, 517, 8), // "Unsorted"
QT_MOC_LITERAL(45, 526, 4), // "Name"
QT_MOC_LITERAL(46, 531, 4), // "Time"
QT_MOC_LITERAL(47, 536, 4), // "Size"
QT_MOC_LITERAL(48, 541, 4), // "Type"
QT_MOC_LITERAL(49, 546, 4), // "Null"
QT_MOC_LITERAL(50, 551, 5), // "Ready"
QT_MOC_LITERAL(51, 557, 7) // "Loading"

    },
    "QQuickFolderListModel\0QML.Element\0"
    "FolderListModel\0folderChanged\0\0"
    "rowCountChanged\0countChanged\0statusChanged\0"
    "_q_directoryChanged\0directory\0"
    "QList<FileProperty>\0list\0_q_directoryUpdated\0"
    "fromIndex\0toIndex\0_q_sortFinished\0"
    "_q_statusChanged\0QQuickFolderListModel::Status\0"
    "s\0isFolder\0index\0get\0idx\0property\0"
    "indexOf\0file\0folder\0rootFolder\0"
    "parentFolder\0nameFilters\0sortField\0"
    "SortField\0sortReversed\0showFiles\0"
    "showDirs\0showDirsFirst\0showDotAndDotDot\0"
    "showHidden\0showOnlyReadable\0caseSensitive\0"
    "count\0status\0Status\0sortCaseSensitive\0"
    "Unsorted\0Name\0Time\0Size\0Type\0Null\0"
    "Ready\0Loading"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickFolderListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      11,   16, // methods
      16,  117, // properties
       2,  197, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   82,    4, 0x06 /* Public */,
       5,    0,   83,    4, 0x06 /* Public */,
       6,    0,   84,    4, 0x86 /* Public | MethodRevisioned */,
       7,    0,   85,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
       8,    2,   86,    4, 0x08 /* Private */,
      12,    4,   91,    4, 0x08 /* Private */,
      15,    1,  100,    4, 0x08 /* Private */,
      16,    1,  103,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      19,    1,  106,    4, 0x02 /* Public */,
      21,    2,  109,    4, 0x02 /* Public */,
      24,    1,  114,    4, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       1,
      11,

 // slots: revision
       0,
       0,
       0,
       0,

 // methods: revision
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, 0x80000000 | 10,    9,   11,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 10, QMetaType::Int, QMetaType::Int,    9,   11,   13,   14,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 17,   18,

 // methods: parameters
    QMetaType::Bool, QMetaType::Int,   20,
    QMetaType::QVariant, QMetaType::Int, QMetaType::QString,   22,   23,
    QMetaType::Int, QMetaType::QUrl,   25,

 // properties: name, type, flags
      26, QMetaType::QUrl, 0x00495103,
      27, QMetaType::QUrl, 0x00095103,
      28, QMetaType::QUrl, 0x00495001,
      29, QMetaType::QStringList, 0x00095103,
      30, 0x80000000 | 31, 0x0009510b,
      32, QMetaType::Bool, 0x00095103,
      33, QMetaType::Bool, 0x00895103,
      34, QMetaType::Bool, 0x00095103,
      35, QMetaType::Bool, 0x00095103,
      36, QMetaType::Bool, 0x00095103,
      37, QMetaType::Bool, 0x00895103,
      38, QMetaType::Bool, 0x00095103,
      39, QMetaType::Bool, 0x00895103,
      40, QMetaType::Int, 0x00495001,
      41, 0x80000000 | 42, 0x00c95009,
      43, QMetaType::Bool, 0x00895103,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       2,
       3,
       0,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       1,
       0,
       2,
       0,
      11,
      12,

 // enums: name, alias, flags, count, data
      31,   31, 0x0,    5,  207,
      42,   42, 0x0,    3,  217,

 // enum data: key, value
      44, uint(QQuickFolderListModel::Unsorted),
      45, uint(QQuickFolderListModel::Name),
      46, uint(QQuickFolderListModel::Time),
      47, uint(QQuickFolderListModel::Size),
      48, uint(QQuickFolderListModel::Type),
      49, uint(QQuickFolderListModel::Null),
      50, uint(QQuickFolderListModel::Ready),
      51, uint(QQuickFolderListModel::Loading),

       0        // eod
};

void QQuickFolderListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickFolderListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->folderChanged(); break;
        case 1: _t->rowCountChanged(); break;
        case 2: _t->countChanged(); break;
        case 3: _t->statusChanged(); break;
        case 4: _t->d_func()->_q_directoryChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<FileProperty>(*)>(_a[2]))); break;
        case 5: _t->d_func()->_q_directoryUpdated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<FileProperty>(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 6: _t->d_func()->_q_sortFinished((*reinterpret_cast< const QList<FileProperty>(*)>(_a[1]))); break;
        case 7: _t->d_func()->_q_statusChanged((*reinterpret_cast< QQuickFolderListModel::Status(*)>(_a[1]))); break;
        case 8: { bool _r = _t->isFolder((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { QVariant _r = _t->get((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->indexOf((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickFolderListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFolderListModel::folderChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickFolderListModel::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFolderListModel::rowCountChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickFolderListModel::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFolderListModel::countChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickFolderListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFolderListModel::statusChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickFolderListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->folder(); break;
        case 1: *reinterpret_cast< QUrl*>(_v) = _t->rootFolder(); break;
        case 2: *reinterpret_cast< QUrl*>(_v) = _t->parentFolder(); break;
        case 3: *reinterpret_cast< QStringList*>(_v) = _t->nameFilters(); break;
        case 4: *reinterpret_cast< SortField*>(_v) = _t->sortField(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->sortReversed(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->showFiles(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->showDirs(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->showDirsFirst(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->showDotAndDotDot(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->showHidden(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->showOnlyReadable(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->caseSensitive(); break;
        case 13: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 14: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->sortCaseSensitive(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickFolderListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFolder(*reinterpret_cast< QUrl*>(_v)); break;
        case 1: _t->setRootFolder(*reinterpret_cast< QUrl*>(_v)); break;
        case 3: _t->setNameFilters(*reinterpret_cast< QStringList*>(_v)); break;
        case 4: _t->setSortField(*reinterpret_cast< SortField*>(_v)); break;
        case 5: _t->setSortReversed(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setShowFiles(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setShowDirs(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setShowDirsFirst(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setShowDotAndDotDot(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setShowHidden(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setShowOnlyReadable(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setCaseSensitive(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->setSortCaseSensitive(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickFolderListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QQuickFolderListModel.data,
    qt_meta_data_QQuickFolderListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickFolderListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickFolderListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickFolderListModel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QQuickFolderListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 16;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickFolderListModel::folderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickFolderListModel::rowCountChanged()const
{
    QMetaObject::activate(const_cast< QQuickFolderListModel *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickFolderListModel::countChanged()const
{
    QMetaObject::activate(const_cast< QQuickFolderListModel *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickFolderListModel::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
