/****************************************************************************
** Meta object code from reading C++ file 'qquickanchors_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickanchors_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickanchors_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAnchors_t {
    QByteArrayData data[54];
    char stringdata0[752];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAnchors_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAnchors_t qt_meta_stringdata_QQuickAnchors = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QQuickAnchors"
QT_MOC_LITERAL(1, 14, 11), // "QML.Element"
QT_MOC_LITERAL(2, 26, 9), // "anonymous"
QT_MOC_LITERAL(3, 36, 11), // "leftChanged"
QT_MOC_LITERAL(4, 48, 0), // ""
QT_MOC_LITERAL(5, 49, 12), // "rightChanged"
QT_MOC_LITERAL(6, 62, 10), // "topChanged"
QT_MOC_LITERAL(7, 73, 13), // "bottomChanged"
QT_MOC_LITERAL(8, 87, 21), // "verticalCenterChanged"
QT_MOC_LITERAL(9, 109, 23), // "horizontalCenterChanged"
QT_MOC_LITERAL(10, 133, 15), // "baselineChanged"
QT_MOC_LITERAL(11, 149, 11), // "fillChanged"
QT_MOC_LITERAL(12, 161, 15), // "centerInChanged"
QT_MOC_LITERAL(13, 177, 17), // "leftMarginChanged"
QT_MOC_LITERAL(14, 195, 18), // "rightMarginChanged"
QT_MOC_LITERAL(15, 214, 16), // "topMarginChanged"
QT_MOC_LITERAL(16, 231, 19), // "bottomMarginChanged"
QT_MOC_LITERAL(17, 251, 14), // "marginsChanged"
QT_MOC_LITERAL(18, 266, 27), // "verticalCenterOffsetChanged"
QT_MOC_LITERAL(19, 294, 29), // "horizontalCenterOffsetChanged"
QT_MOC_LITERAL(20, 324, 21), // "baselineOffsetChanged"
QT_MOC_LITERAL(21, 346, 20), // "centerAlignedChanged"
QT_MOC_LITERAL(22, 367, 4), // "left"
QT_MOC_LITERAL(23, 372, 16), // "QQuickAnchorLine"
QT_MOC_LITERAL(24, 389, 5), // "right"
QT_MOC_LITERAL(25, 395, 16), // "horizontalCenter"
QT_MOC_LITERAL(26, 412, 3), // "top"
QT_MOC_LITERAL(27, 416, 6), // "bottom"
QT_MOC_LITERAL(28, 423, 14), // "verticalCenter"
QT_MOC_LITERAL(29, 438, 8), // "baseline"
QT_MOC_LITERAL(30, 447, 7), // "margins"
QT_MOC_LITERAL(31, 455, 10), // "leftMargin"
QT_MOC_LITERAL(32, 466, 11), // "rightMargin"
QT_MOC_LITERAL(33, 478, 22), // "horizontalCenterOffset"
QT_MOC_LITERAL(34, 501, 9), // "topMargin"
QT_MOC_LITERAL(35, 511, 12), // "bottomMargin"
QT_MOC_LITERAL(36, 524, 20), // "verticalCenterOffset"
QT_MOC_LITERAL(37, 545, 14), // "baselineOffset"
QT_MOC_LITERAL(38, 560, 4), // "fill"
QT_MOC_LITERAL(39, 565, 11), // "QQuickItem*"
QT_MOC_LITERAL(40, 577, 8), // "centerIn"
QT_MOC_LITERAL(41, 586, 17), // "alignWhenCentered"
QT_MOC_LITERAL(42, 604, 7), // "Anchors"
QT_MOC_LITERAL(43, 612, 6), // "Anchor"
QT_MOC_LITERAL(44, 619, 13), // "InvalidAnchor"
QT_MOC_LITERAL(45, 633, 10), // "LeftAnchor"
QT_MOC_LITERAL(46, 644, 11), // "RightAnchor"
QT_MOC_LITERAL(47, 656, 9), // "TopAnchor"
QT_MOC_LITERAL(48, 666, 12), // "BottomAnchor"
QT_MOC_LITERAL(49, 679, 13), // "HCenterAnchor"
QT_MOC_LITERAL(50, 693, 13), // "VCenterAnchor"
QT_MOC_LITERAL(51, 707, 14), // "BaselineAnchor"
QT_MOC_LITERAL(52, 722, 15), // "Horizontal_Mask"
QT_MOC_LITERAL(53, 738, 13) // "Vertical_Mask"

    },
    "QQuickAnchors\0QML.Element\0anonymous\0"
    "leftChanged\0\0rightChanged\0topChanged\0"
    "bottomChanged\0verticalCenterChanged\0"
    "horizontalCenterChanged\0baselineChanged\0"
    "fillChanged\0centerInChanged\0"
    "leftMarginChanged\0rightMarginChanged\0"
    "topMarginChanged\0bottomMarginChanged\0"
    "marginsChanged\0verticalCenterOffsetChanged\0"
    "horizontalCenterOffsetChanged\0"
    "baselineOffsetChanged\0centerAlignedChanged\0"
    "left\0QQuickAnchorLine\0right\0"
    "horizontalCenter\0top\0bottom\0verticalCenter\0"
    "baseline\0margins\0leftMargin\0rightMargin\0"
    "horizontalCenterOffset\0topMargin\0"
    "bottomMargin\0verticalCenterOffset\0"
    "baselineOffset\0fill\0QQuickItem*\0"
    "centerIn\0alignWhenCentered\0Anchors\0"
    "Anchor\0InvalidAnchor\0LeftAnchor\0"
    "RightAnchor\0TopAnchor\0BottomAnchor\0"
    "HCenterAnchor\0VCenterAnchor\0BaselineAnchor\0"
    "Horizontal_Mask\0Vertical_Mask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAnchors[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      18,   16, // methods
      18,  124, // properties
       1,  196, // enums/sets
       0,    0, // constructors
       0,       // flags
      18,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  106,    4, 0x06 /* Public */,
       5,    0,  107,    4, 0x06 /* Public */,
       6,    0,  108,    4, 0x06 /* Public */,
       7,    0,  109,    4, 0x06 /* Public */,
       8,    0,  110,    4, 0x06 /* Public */,
       9,    0,  111,    4, 0x06 /* Public */,
      10,    0,  112,    4, 0x06 /* Public */,
      11,    0,  113,    4, 0x06 /* Public */,
      12,    0,  114,    4, 0x06 /* Public */,
      13,    0,  115,    4, 0x06 /* Public */,
      14,    0,  116,    4, 0x06 /* Public */,
      15,    0,  117,    4, 0x06 /* Public */,
      16,    0,  118,    4, 0x06 /* Public */,
      17,    0,  119,    4, 0x06 /* Public */,
      18,    0,  120,    4, 0x06 /* Public */,
      19,    0,  121,    4, 0x06 /* Public */,
      20,    0,  122,    4, 0x06 /* Public */,
      21,    0,  123,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      22, 0x80000000 | 23, 0x0049510f,
      24, 0x80000000 | 23, 0x0049510f,
      25, 0x80000000 | 23, 0x0049510f,
      26, 0x80000000 | 23, 0x0049510f,
      27, 0x80000000 | 23, 0x0049510f,
      28, 0x80000000 | 23, 0x0049510f,
      29, 0x80000000 | 23, 0x0049510f,
      30, QMetaType::QReal, 0x00495103,
      31, QMetaType::QReal, 0x00495107,
      32, QMetaType::QReal, 0x00495107,
      33, QMetaType::QReal, 0x00495103,
      34, QMetaType::QReal, 0x00495107,
      35, QMetaType::QReal, 0x00495107,
      36, QMetaType::QReal, 0x00495103,
      37, QMetaType::QReal, 0x00495103,
      38, 0x80000000 | 39, 0x0049510f,
      40, 0x80000000 | 39, 0x0049510f,
      41, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       5,
       2,
       3,
       4,
       6,
      13,
       9,
      10,
      15,
      11,
      12,
      14,
      16,
       7,
       8,
      17,

 // enums: name, alias, flags, count, data
      42,   43, 0x1,   10,  201,

 // enum data: key, value
      44, uint(QQuickAnchors::InvalidAnchor),
      45, uint(QQuickAnchors::LeftAnchor),
      46, uint(QQuickAnchors::RightAnchor),
      47, uint(QQuickAnchors::TopAnchor),
      48, uint(QQuickAnchors::BottomAnchor),
      49, uint(QQuickAnchors::HCenterAnchor),
      50, uint(QQuickAnchors::VCenterAnchor),
      51, uint(QQuickAnchors::BaselineAnchor),
      52, uint(QQuickAnchors::Horizontal_Mask),
      53, uint(QQuickAnchors::Vertical_Mask),

       0        // eod
};

void QQuickAnchors::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAnchors *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->leftChanged(); break;
        case 1: _t->rightChanged(); break;
        case 2: _t->topChanged(); break;
        case 3: _t->bottomChanged(); break;
        case 4: _t->verticalCenterChanged(); break;
        case 5: _t->horizontalCenterChanged(); break;
        case 6: _t->baselineChanged(); break;
        case 7: _t->fillChanged(); break;
        case 8: _t->centerInChanged(); break;
        case 9: _t->leftMarginChanged(); break;
        case 10: _t->rightMarginChanged(); break;
        case 11: _t->topMarginChanged(); break;
        case 12: _t->bottomMarginChanged(); break;
        case 13: _t->marginsChanged(); break;
        case 14: _t->verticalCenterOffsetChanged(); break;
        case 15: _t->horizontalCenterOffsetChanged(); break;
        case 16: _t->baselineOffsetChanged(); break;
        case 17: _t->centerAlignedChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::leftChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::rightChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::topChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::bottomChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::verticalCenterChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::horizontalCenterChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::baselineChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::fillChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::centerInChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::leftMarginChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::rightMarginChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::topMarginChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::bottomMarginChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::marginsChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::verticalCenterOffsetChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::horizontalCenterOffsetChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::baselineOffsetChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickAnchors::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnchors::centerAlignedChanged)) {
                *result = 17;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAnchors *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->left(); break;
        case 1: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->right(); break;
        case 2: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->horizontalCenter(); break;
        case 3: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->top(); break;
        case 4: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->bottom(); break;
        case 5: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->verticalCenter(); break;
        case 6: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->baseline(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->margins(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->leftMargin(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->rightMargin(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->horizontalCenterOffset(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->topMargin(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->bottomMargin(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->verticalCenterOffset(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->baselineOffset(); break;
        case 15: *reinterpret_cast< QQuickItem**>(_v) = _t->fill(); break;
        case 16: *reinterpret_cast< QQuickItem**>(_v) = _t->centerIn(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->alignWhenCentered(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAnchors *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLeft(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 1: _t->setRight(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 2: _t->setHorizontalCenter(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 3: _t->setTop(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 4: _t->setBottom(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 5: _t->setVerticalCenter(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 6: _t->setBaseline(*reinterpret_cast< QQuickAnchorLine*>(_v)); break;
        case 7: _t->setMargins(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setLeftMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setRightMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setHorizontalCenterOffset(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setTopMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 12: _t->setBottomMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 13: _t->setVerticalCenterOffset(*reinterpret_cast< qreal*>(_v)); break;
        case 14: _t->setBaselineOffset(*reinterpret_cast< qreal*>(_v)); break;
        case 15: _t->setFill(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 16: _t->setCenterIn(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 17: _t->setAlignWhenCentered(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickAnchors *_t = static_cast<QQuickAnchors *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetLeft(); break;
        case 1: _t->resetRight(); break;
        case 2: _t->resetHorizontalCenter(); break;
        case 3: _t->resetTop(); break;
        case 4: _t->resetBottom(); break;
        case 5: _t->resetVerticalCenter(); break;
        case 6: _t->resetBaseline(); break;
        case 8: _t->resetLeftMargin(); break;
        case 9: _t->resetRightMargin(); break;
        case 11: _t->resetTopMargin(); break;
        case 12: _t->resetBottomMargin(); break;
        case 15: _t->resetFill(); break;
        case 16: _t->resetCenterIn(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickAnchors::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickAnchors.data,
    qt_meta_data_QQuickAnchors,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAnchors::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAnchors::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAnchors.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickAnchors::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 18;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 18;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 18;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 18;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 18;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAnchors::leftChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickAnchors::rightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickAnchors::topChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickAnchors::bottomChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickAnchors::verticalCenterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickAnchors::horizontalCenterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickAnchors::baselineChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickAnchors::fillChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickAnchors::centerInChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickAnchors::leftMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickAnchors::rightMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickAnchors::topMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickAnchors::bottomMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickAnchors::marginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickAnchors::verticalCenterOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickAnchors::horizontalCenterOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickAnchors::baselineOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickAnchors::centerAlignedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
