/****************************************************************************
** Meta object code from reading C++ file 'qqmllocale_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qml/qml/qqmllocale_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmllocale_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlLocale_t {
    QByteArrayData data[38];
    char stringdata0[600];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlLocale_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlLocale_t qt_meta_stringdata_QQmlLocale = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQmlLocale"
QT_MOC_LITERAL(1, 11, 11), // "QML.Element"
QT_MOC_LITERAL(2, 23, 6), // "Locale"
QT_MOC_LITERAL(3, 30, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 44, 5), // "false"
QT_MOC_LITERAL(5, 50, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 72, 47), // "Locale cannot be instantiated..."
QT_MOC_LITERAL(7, 120, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 144, 1), // "2"
QT_MOC_LITERAL(9, 146, 17), // "MeasurementSystem"
QT_MOC_LITERAL(10, 164, 12), // "MetricSystem"
QT_MOC_LITERAL(11, 177, 14), // "ImperialSystem"
QT_MOC_LITERAL(12, 192, 16), // "ImperialUSSystem"
QT_MOC_LITERAL(13, 209, 16), // "ImperialUKSystem"
QT_MOC_LITERAL(14, 226, 10), // "FormatType"
QT_MOC_LITERAL(15, 237, 10), // "LongFormat"
QT_MOC_LITERAL(16, 248, 11), // "ShortFormat"
QT_MOC_LITERAL(17, 260, 12), // "NarrowFormat"
QT_MOC_LITERAL(18, 273, 20), // "CurrencySymbolFormat"
QT_MOC_LITERAL(19, 294, 15), // "CurrencyIsoCode"
QT_MOC_LITERAL(20, 310, 14), // "CurrencySymbol"
QT_MOC_LITERAL(21, 325, 19), // "CurrencyDisplayName"
QT_MOC_LITERAL(22, 345, 9), // "DayOfWeek"
QT_MOC_LITERAL(23, 355, 6), // "Sunday"
QT_MOC_LITERAL(24, 362, 6), // "Monday"
QT_MOC_LITERAL(25, 369, 7), // "Tuesday"
QT_MOC_LITERAL(26, 377, 9), // "Wednesday"
QT_MOC_LITERAL(27, 387, 8), // "Thursday"
QT_MOC_LITERAL(28, 396, 6), // "Friday"
QT_MOC_LITERAL(29, 403, 8), // "Saturday"
QT_MOC_LITERAL(30, 412, 13), // "NumberOptions"
QT_MOC_LITERAL(31, 426, 20), // "DefaultNumberOptions"
QT_MOC_LITERAL(32, 447, 18), // "OmitGroupSeparator"
QT_MOC_LITERAL(33, 466, 20), // "RejectGroupSeparator"
QT_MOC_LITERAL(34, 487, 25), // "OmitLeadingZeroInExponent"
QT_MOC_LITERAL(35, 513, 27), // "RejectLeadingZeroInExponent"
QT_MOC_LITERAL(36, 541, 29), // "IncludeTrailingZeroesAfterDot"
QT_MOC_LITERAL(37, 571, 28) // "RejectTrailingZeroesAfterDot"

    },
    "QQmlLocale\0QML.Element\0Locale\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Locale cannot be instantiated. Use Qt.locale().\0"
    "QML.AddedInMinorVersion\0""2\0MeasurementSystem\0"
    "MetricSystem\0ImperialSystem\0"
    "ImperialUSSystem\0ImperialUKSystem\0"
    "FormatType\0LongFormat\0ShortFormat\0"
    "NarrowFormat\0CurrencySymbolFormat\0"
    "CurrencyIsoCode\0CurrencySymbol\0"
    "CurrencyDisplayName\0DayOfWeek\0Sunday\0"
    "Monday\0Tuesday\0Wednesday\0Thursday\0"
    "Friday\0Saturday\0NumberOptions\0"
    "DefaultNumberOptions\0OmitGroupSeparator\0"
    "RejectGroupSeparator\0OmitLeadingZeroInExponent\0"
    "RejectLeadingZeroInExponent\0"
    "IncludeTrailingZeroesAfterDot\0"
    "RejectTrailingZeroesAfterDot"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlLocale[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       5,   22, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    4,   47,
      14,   14, 0x0,    3,   55,
      18,   18, 0x0,    3,   61,
      22,   22, 0x0,    7,   67,
      30,   30, 0x0,    7,   81,

 // enum data: key, value
      10, uint(QQmlLocale::MetricSystem),
      11, uint(QQmlLocale::ImperialSystem),
      12, uint(QQmlLocale::ImperialUSSystem),
      13, uint(QQmlLocale::ImperialUKSystem),
      15, uint(QQmlLocale::LongFormat),
      16, uint(QQmlLocale::ShortFormat),
      17, uint(QQmlLocale::NarrowFormat),
      19, uint(QQmlLocale::CurrencyIsoCode),
      20, uint(QQmlLocale::CurrencySymbol),
      21, uint(QQmlLocale::CurrencyDisplayName),
      23, uint(QQmlLocale::Sunday),
      24, uint(QQmlLocale::Monday),
      25, uint(QQmlLocale::Tuesday),
      26, uint(QQmlLocale::Wednesday),
      27, uint(QQmlLocale::Thursday),
      28, uint(QQmlLocale::Friday),
      29, uint(QQmlLocale::Saturday),
      31, uint(QQmlLocale::DefaultNumberOptions),
      32, uint(QQmlLocale::OmitGroupSeparator),
      33, uint(QQmlLocale::RejectGroupSeparator),
      34, uint(QQmlLocale::OmitLeadingZeroInExponent),
      35, uint(QQmlLocale::RejectLeadingZeroInExponent),
      36, uint(QQmlLocale::IncludeTrailingZeroesAfterDot),
      37, uint(QQmlLocale::RejectTrailingZeroesAfterDot),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQmlLocale::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQmlLocale.data,
    qt_meta_data_QQmlLocale,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
