/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qquickage_p.h>
#include <qquickangledirection_p.h>
#include <qquickcumulativedirection_p.h>
#include <qquickcustomaffector_p.h>
#include <qquickcustomparticle_p.h>
#include <qquickdirection_p.h>
#include <qquickellipseextruder_p.h>
#include <qquickfriction_p.h>
#include <qquickgravity_p.h>
#include <qquickgroupgoal_p.h>
#include <qquickimageparticle_p.h>
#include <qquickitemparticle_p.h>
#include <qquicklineextruder_p.h>
#include <qquickmaskextruder_p.h>
#include <qquickparticleaffector_p.h>
#include <qquickparticleemitter_p.h>
#include <qquickparticleextruder_p.h>
#include <qquickparticlegroup_p.h>
#include <qquickparticlepainter_p.h>
#include <qquickparticlesystem_p.h>
#include <qquickpointattractor_p.h>
#include <qquickpointdirection_p.h>
#include <qquickrectangleextruder_p.h>
#include <qquickspritegoal_p.h>
#include <qquicktargetdirection_p.h>
#include <qquicktrailemitter_p.h>
#include <qquickturbulence_p.h>
#include <qquickwander_p.h>

void qml_register_types_QtQuick_Particles()
{
    qmlRegisterTypesAndRevisions<QQuickAgeAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickAngleDirection>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickAttractorAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickCumulativeDirection>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickCustomAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickCustomParticle>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickDirection>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickEllipseExtruder>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickFrictionAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickGravityAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickGroupGoalAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickImageParticle>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickItemParticle>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickLineExtruder>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickMaskExtruder>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticleAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticleEmitter>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticleExtruder>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticleGroup>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticlePainter>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickParticleSystem>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickPointDirection>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickRectangleExtruder>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickSpriteGoalAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickTargetDirection>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickTrailEmitter>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickTurbulenceAffector>("QtQuick.Particles", 2);
    qmlRegisterTypesAndRevisions<QQuickWanderAffector>("QtQuick.Particles", 2);
    qmlRegisterModule("QtQuick.Particles", 2, 15);
}

static const QQmlModuleRegistration registration("QtQuick.Particles", 2, qml_register_types_QtQuick_Particles);
