/****************************************************************************
** Meta object code from reading C++ file 'qquickitemparticle_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/particles/qquickitemparticle_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitemparticle_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickItemParticle_t {
    QByteArrayData data[21];
    char stringdata0[220];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItemParticle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItemParticle_t qt_meta_stringdata_QQuickItemParticle = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickItemParticle"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "ItemParticle"
QT_MOC_LITERAL(3, 44, 12), // "QML.Attached"
QT_MOC_LITERAL(4, 57, 26), // "QQuickItemParticleAttached"
QT_MOC_LITERAL(5, 84, 11), // "fadeChanged"
QT_MOC_LITERAL(6, 96, 0), // ""
QT_MOC_LITERAL(7, 97, 15), // "delegateChanged"
QT_MOC_LITERAL(8, 113, 14), // "QQmlComponent*"
QT_MOC_LITERAL(9, 128, 3), // "arg"
QT_MOC_LITERAL(10, 132, 6), // "freeze"
QT_MOC_LITERAL(11, 139, 11), // "QQuickItem*"
QT_MOC_LITERAL(12, 151, 4), // "item"
QT_MOC_LITERAL(13, 156, 8), // "unfreeze"
QT_MOC_LITERAL(14, 165, 4), // "take"
QT_MOC_LITERAL(15, 170, 10), // "prioritize"
QT_MOC_LITERAL(16, 181, 4), // "give"
QT_MOC_LITERAL(17, 186, 7), // "setFade"
QT_MOC_LITERAL(18, 194, 11), // "setDelegate"
QT_MOC_LITERAL(19, 206, 4), // "fade"
QT_MOC_LITERAL(20, 211, 8) // "delegate"

    },
    "QQuickItemParticle\0QML.Element\0"
    "ItemParticle\0QML.Attached\0"
    "QQuickItemParticleAttached\0fadeChanged\0"
    "\0delegateChanged\0QQmlComponent*\0arg\0"
    "freeze\0QQuickItem*\0item\0unfreeze\0take\0"
    "prioritize\0give\0setFade\0setDelegate\0"
    "fade\0delegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItemParticle[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
       2,   90, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   63,    6, 0x06 /* Public */,
       7,    1,   64,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   67,    6, 0x0a /* Public */,
      13,    1,   70,    6, 0x0a /* Public */,
      14,    2,   73,    6, 0x0a /* Public */,
      14,    1,   78,    6, 0x2a /* Public | MethodCloned */,
      16,    1,   81,    6, 0x0a /* Public */,
      17,    1,   84,    6, 0x0a /* Public */,
      18,    1,   87,    6, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11, QMetaType::Bool,   12,   15,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, 0x80000000 | 8,    9,

 // properties: name, type, flags
      19, QMetaType::Bool, 0x00495103,
      20, 0x80000000 | 8, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QQuickItemParticle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItemParticle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fadeChanged(); break;
        case 1: _t->delegateChanged((*reinterpret_cast< QQmlComponent*(*)>(_a[1]))); break;
        case 2: _t->freeze((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 3: _t->unfreeze((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 4: _t->take((*reinterpret_cast< QQuickItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->take((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 6: _t->give((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 7: _t->setFade((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->setDelegate((*reinterpret_cast< QQmlComponent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItemParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemParticle::fadeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItemParticle::*)(QQmlComponent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemParticle::delegateChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItemParticle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->fade(); break;
        case 1: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickItemParticle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFade(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickItemParticle::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickParticlePainter::staticMetaObject>(),
    qt_meta_stringdata_QQuickItemParticle.data,
    qt_meta_data_QQuickItemParticle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickItemParticle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItemParticle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItemParticle.stringdata0))
        return static_cast<void*>(this);
    return QQuickParticlePainter::qt_metacast(_clname);
}

int QQuickItemParticle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickParticlePainter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItemParticle::fadeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickItemParticle::delegateChanged(QQmlComponent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_QQuickItemParticleAttached_t {
    QByteArrayData data[6];
    char stringdata0[75];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItemParticleAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItemParticleAttached_t qt_meta_stringdata_QQuickItemParticleAttached = {
    {
QT_MOC_LITERAL(0, 0, 26), // "QQuickItemParticleAttached"
QT_MOC_LITERAL(1, 27, 8), // "detached"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 8), // "attached"
QT_MOC_LITERAL(4, 46, 8), // "particle"
QT_MOC_LITERAL(5, 55, 19) // "QQuickItemParticle*"

    },
    "QQuickItemParticleAttached\0detached\0"
    "\0attached\0particle\0QQuickItemParticle*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItemParticleAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       1,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    0,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       4, 0x80000000 | 5, 0x00095409,

       0        // eod
};

void QQuickItemParticleAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItemParticleAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->detached(); break;
        case 1: _t->attached(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItemParticleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemParticleAttached::detached)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItemParticleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemParticleAttached::attached)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItemParticle* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItemParticleAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItemParticle**>(_v) = _t->particle(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickItemParticleAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickItemParticleAttached.data,
    qt_meta_data_QQuickItemParticleAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickItemParticleAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItemParticleAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItemParticleAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickItemParticleAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItemParticleAttached::detached()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickItemParticleAttached::attached()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
