/****************************************************************************
** Meta object code from reading C++ file 'qquickworkerscript_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmlworkerscript/qquickworkerscript_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickworkerscript_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickWorkerScriptEngine_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWorkerScriptEngine_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWorkerScriptEngine_t qt_meta_stringdata_QQuickWorkerScriptEngine = {
    {
QT_MOC_LITERAL(0, 0, 24) // "QQuickWorkerScriptEngine"

    },
    "QQuickWorkerScriptEngine"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWorkerScriptEngine[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickWorkerScriptEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickWorkerScriptEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_QQuickWorkerScriptEngine.data,
    qt_meta_data_QQuickWorkerScriptEngine,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickWorkerScriptEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickWorkerScriptEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickWorkerScriptEngine.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int QQuickWorkerScriptEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickWorkerScript_t {
    QByteArrayData data[13];
    char stringdata0[144];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWorkerScript_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWorkerScript_t qt_meta_stringdata_QQuickWorkerScript = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickWorkerScript"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "WorkerScript"
QT_MOC_LITERAL(3, 44, 13), // "sourceChanged"
QT_MOC_LITERAL(4, 58, 0), // ""
QT_MOC_LITERAL(5, 59, 12), // "readyChanged"
QT_MOC_LITERAL(6, 72, 7), // "message"
QT_MOC_LITERAL(7, 80, 8), // "QJSValue"
QT_MOC_LITERAL(8, 89, 13), // "messageObject"
QT_MOC_LITERAL(9, 103, 11), // "sendMessage"
QT_MOC_LITERAL(10, 115, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(11, 131, 6), // "source"
QT_MOC_LITERAL(12, 138, 5) // "ready"

    },
    "QQuickWorkerScript\0QML.Element\0"
    "WorkerScript\0sourceChanged\0\0readyChanged\0"
    "message\0QJSValue\0messageObject\0"
    "sendMessage\0QQmlV4Function*\0source\0"
    "ready"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWorkerScript[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
       2,   48, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   40,    4, 0x06 /* Public */,
       5,    0,   41,    4, 0x86 /* Public | MethodRevisioned */,
       6,    1,   42,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,   45,    4, 0x0a /* Public */,

 // signals: revision
       0,
      15,
       0,

 // slots: revision
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10,    4,

 // properties: name, type, flags
      11, QMetaType::QUrl, 0x00495103,
      12, QMetaType::Bool, 0x00c95001,

 // properties: notify_signal_id
       0,
       1,

 // properties: revision
       0,
      15,

       0        // eod
};

void QQuickWorkerScript::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickWorkerScript *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceChanged(); break;
        case 1: _t->readyChanged(); break;
        case 2: _t->message((*reinterpret_cast< const QJSValue(*)>(_a[1]))); break;
        case 3: _t->sendMessage((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickWorkerScript::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWorkerScript::sourceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickWorkerScript::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWorkerScript::readyChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickWorkerScript::*)(const QJSValue & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWorkerScript::message)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickWorkerScript *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->ready(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickWorkerScript *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickWorkerScript::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickWorkerScript.data,
    qt_meta_data_QQuickWorkerScript,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickWorkerScript::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickWorkerScript::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickWorkerScript.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickWorkerScript::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickWorkerScript::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickWorkerScript::readyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickWorkerScript::message(const QJSValue & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
